#!/usr/bin/haserl
Content-Type: text/javascript

Scada.Objects = {};
var $O = Scada.Objects;

$O.autoUpdate = false;
$O.updateTime = ('<? cat /tmp/gs-groupmonitor.time 2>/dev/null ?>').trim();

$O.storeFields = [ 'id', 'name', 'address', 'datatype', 'comment', 'datadec', 'tagcache', 'disablelog', 'export', 'visparams', 'updatetime', 'units', 'enums' ];

$O.store = new Ext.data.JsonStore({
  url: Scada.reqUrl('objects'),
  root: 'data',
  fields: $O.storeFields
});

$O.extObjStore = new Ext.data.ArrayStore({
  fields: [ 'id', 'text', 'datatype' ],
  data: []
});

$O.extNumObjStore = new Ext.data.ArrayStore({
  fields: [ 'id', 'text', 'datatype' ],
  data: []
});

$O.gridStore = new Ext.ux.data.PagingJsonStore({
  data: [],
  fields: $O.storeFields,
  sortInfo: {
    field: 'address',
    direction: 'ASC'
  }
});

$O.updateList = {};
$O.updateTimers = {};

$O.dataTypes = new Ext.data.ArrayStore({
  fields: ['value', 'display', 'extraCls'],
  idIndex: 0,
  data: [
    ['',   $tr('objects_datatype_'), 'unspec' ],
    ['1',  $tr('objects_datatype_1'), 'main'],
    ['1001',  $tr('objects_datatype_1001'), 'sub'],
    ['1002',  $tr('objects_datatype_1002'), 'sub'],
    ['1003',  $tr('objects_datatype_1003'), 'sub'],
    ['1004',  $tr('objects_datatype_1004'), 'sub'],
    ['1005',  $tr('objects_datatype_1005'), 'sub'],
    ['1006',  $tr('objects_datatype_1006'), 'sub'],
    ['1007',  $tr('objects_datatype_1007'), 'sub'],
    ['1008',  $tr('objects_datatype_1008'), 'sub'],
    ['1009',  $tr('objects_datatype_1009'), 'sub'],
    ['1010',  $tr('objects_datatype_1010'), 'sub'],
    ['1011',  $tr('objects_datatype_1011'), 'sub'],
    ['1012',  $tr('objects_datatype_1012'), 'sub'],
    ['1013',  $tr('objects_datatype_1013'), 'sub'],
    ['1014',  $tr('objects_datatype_1014'), 'sub'],
    ['2',  $tr('objects_datatype_2'), 'main'],
    ['3',  $tr('objects_datatype_3'), 'main'],
    ['3007',  $tr('objects_datatype_3007'), 'sub'],
    ['4',  $tr('objects_datatype_4'), 'main'],
    ['5',  $tr('objects_datatype_5'), 'main'],
    ['5001',  $tr('objects_datatype_5001'), 'sub'],
    ['5003',  $tr('objects_datatype_5003'), 'sub'],
    ['6',  $tr('objects_datatype_6'), 'main'],
    ['7',  $tr('objects_datatype_7'), 'main'],
    ['8',  $tr('objects_datatype_8'), 'main'],
    ['9',  $tr('objects_datatype_9'), 'main'],
    ['9001',  $tr('objects_datatype_9001'), 'sub'],
    ['254', $tr('objects_datatype_254'), 'main'],
    ['10', $tr('objects_datatype_10'), 'main'],
    ['11', $tr('objects_datatype_11'), 'main'],
    ['12', $tr('objects_datatype_12'), 'main'],
    ['13', $tr('objects_datatype_13'), 'main'],
    ['14', $tr('objects_datatype_14'), 'main'],
    ['15', $tr('objects_datatype_15'), 'main'],
    ['16', $tr('objects_datatype_16'), 'main'],
    ['16002', $tr('objects_datatype_16002'), 'sub']
  ]
});

$O.dtSelect = $def.select({
  fieldLabel: $tr('objects_datatype'),
  name: 'datatype',
  emptyText: $tr('objects_datatype_'),
  triggerAction: 'all',
  hiddenName: 'datatype',
  valueField: 'value',
  displayField: 'display',
  mode: 'local',
  tpl: '<tpl for="."><div class="x-combo-list-item scada-combo-item-{extraCls}">{display}</div></tpl>'
});

$O.pagination = new Ext.ux.PagingToolbar({
  pageSize: Config.perpage,
  store: $O.gridStore,
  displayInfo: true,
  displayMsg: $tr('objects_pages_hint'),
  emptyMsg: $tr('objects_pages_empty'),
  refreshText: false, firstText: false, lastText: false, nextText: false, prevText: false,
  prependButtons: true,
  items: [
    { text: $tr('objects_add'), id: 'objects-add', iconCls: 'icon-add' }, '-',
    { text: $tr('objects_auto_update_on'), id: 'objects-auto-update', iconCls: 'icon-accept', enableToggle: true }, '-',
    { text: $tr('utilities_reset_button'), id: 'reset-objects', iconCls: 'icon-clear' }, '-'
  ]
});

$O.filterReset = function() {
  $O.filters.name.setValue('');
  $O.filters.dt.setValue('');
  $O.filters.tags.setValue('');

  $O.filterBy = null;

  $O.gridStore.filterBy($O.filterCb);
};

$O.filterApply = function() {
  var name = $O.filters.name.getValue().trim(),
      dt = Ext.get('objects-filter-dt-hidden').getValue().trim(),
      tags = $O.filters.tags.getValue().trim();

  if (!name && !dt && !tags) {
    $O.filterBy = null;
  }
  else {
    $O.filterBy = {};

    if (name) {
      var address = Scada.encodeGroupAddress(name.replace('*', '1', 'g'));

      if (address) {
        var reg = name.replace('*', '[0-9]+', 'g');
        $O.filterBy.address = new RegExp('^' + reg + '$', 'i');
      }
      else {
        $O.filterBy.name = new RegExp(Ext.escapeRe(name), 'i');
      }
    }

    if (dt) {
      $O.filterBy.dt = Scada.normalizeDt(dt);
    }

    if (tags) {
      $O.filterBy.tags = tags.toList();
    }
  }

  $O.pagination.moveFirst();
  $O.gridStore.filterBy($O.filterCb);
};

$O.filterListener = {
  specialkey: function(field, event) {
    var key = event.getKey();
    if (key == event.ENTER) {
      $O.filterApply();
    }
    else if (key == event.ESC) {
      $O.filterReset();
    }
  }
};

$O.filter = new Ext.form.FormPanel({
  title: $tr('objects_filter'),
  region: 'west',
  collapsible: true,
  width: 188,
  margins: '10 10 10 10',
  cmargins: '10 10 10 10',
  labelAlign: 'top',
  padding: 10,
  defaults: {
    width: 166
  },
  items: [
    { id: 'objects-filter-name', xtype: 'textfield', fieldLabel: $tr('objects_filter_name_ga'), name: 'address', allowBlank: true, listeners: $O.filterListener },
    $extend($clone($O.dtSelect), { id: 'objects-filter-dt', hiddenId: 'objects-filter-dt-hidden', store: $O.dataTypes }),
    { id: 'objects-filter-tags', xtype: 'tagbox', fieldLabel: $tr('objects_filter_tags'), name: 'tags', allowBlank: true, listeners: $O.filterListener },
    {
      xtype: 'container',
      layout: 'hbox',
      layoutConfig: {
        pack: 'end',
        defaultMargins: '5 0 0 5',
      },
      items: [
        { text: $tr('objects_filter_submit'), xtype: 'button', handler: $O.filterApply },
        { text: $tr('objects_filter_reset'), xtype: 'button', handler: $O.filterReset }
      ]
    }
  ]
});

$O.checkCallback = function(id, field, value) {
  $O.objectId = id;

  Ext.Ajax.request({
    url: Scada.reqUrl('objects', 'check'),
    params: Scada.encodeData({ id: id, field: field, value: value }),
    success: $O.update
  });
}

$O.checkScript = function(record) {
  var addr, found = false;

  addr = Scada.decodeGroupAddress(record.data.address);
  Ext.each($S.eStore.getAll(), function(item) {
    if (item.data.params == addr) {
      found = true;
    }
  });

  return found;
};

$O.grid = new Ext.grid.GridPanel({
  region: 'center',
  border: true,
  enableColumnHide: false,
  enableColumnMove: false,
  store: $O.gridStore,
  id: 'objects-grid',
  ctCls: 'scada-list-view-grid',
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('objects_address'), width: 0.08, dataIndex: 'address', renderer: Scada.decodeGroupAddress },
      { header: $tr('objects_name'), width: 0.12, dataIndex: 'name' },
      { header: $tr('objects_event_script'), width: 0.04, xtype: 'boolclscolumn', trueCls: 'scada-cell-editor', falseCls: 'scada-cell-editor-disabled', dataIndex: 'script', getValue: $O.checkScript },
      { header: $tr('objects_datatype'), width: 0.12, dataIndex: 'datatype', trPrefix: 'objects_datatype', xtype: 'translatecolumn' },
      { header: $tr('objects_value'), width: 0.08, dataIndex: 'datadec', xtype: 'knxdtcolumn' },
      { header: $tr('objects_disablelog'), width: 0.05, dataIndex: 'disablelog', xtype: 'checkcolumn', invert: true, callback: $O.checkCallback },
      { header: $tr('objects_export'), width: 0.05, dataIndex: 'export', xtype: 'checkcolumn', callback: $O.checkCallback },
      { header: $tr('objects_tags'), width: 0.10, dataIndex: 'tagcache' },
      { header: $tr('objects_comment'), width: 0.10, dataIndex: 'comment' },
      { header: $tr('objects_updatetime'), width: 0.10, dataIndex: 'updatetime', xtype: 'timestampcolumn' },
      { header: $tr('objects_setvalue'), width: 0.04, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-object-edit', dataIndex: 'setvalue' },
      { header: $tr('objects_vis_params_short'), width: 0.04, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-object-vis', trueCls: 'x-grid3-cell-object-vis', dataIndex: 'visparams' },
      { header: $tr('objects_enums_short'), width: 0.04, xtype: 'boolclscolumn', trueCls: 'scada-cell-customvals', falseCls: 'scada-cell-customvals', dataIndex: 'enums' },
      { header: '', width: 0.04, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-delete', dataIndex: 'delete' }
    ]
  }),
  bbar: $O.pagination,
  margins: '10 10 10 0',
  viewConfig: {
    forceFit: true,
    markDirty: false,
    getRowClass: function(record) {
      var cls, ut, pt, id = record.id;

      ut = record.data.updatetime.toNumber();
      pt = $O.updateList[ id ] ? $O.updateList[ id ] : ut;

      if (pt != ut || $O.updateTimers[ id ]) {
        cls = 'objects-row-updated';

        if (pt != ut) {
          $O.updateTimers[ id ] = 5;
        }
      }
      else if (!record.data.name || !record.data.datatype) {
        cls = 'objects-row-unspec';
      }
      else {
        cls = 'objects-row-normal';
      }

      $O.updateList[ id ] = ut;
      return cls;
    }
  },
  sm: new Ext.grid.RowSelectionModel({ singleSelect: true })
});

$O.formWindow = new Ext.Window(
  $def.window({
    title: $tr('objects_form'), width: 600, height: 530,
    items: $def.form({
      id: 'objects-form',
      url: Scada.reqUrl('objects', 'save'),
      buttons: $def.formButtons('objects-form'),
      items: [
        { fieldLabel: $tr('objects_name'), name: 'name', allowBlank: false, invalidText: $tr('objects_error_name') },
        { fieldLabel: $tr('objects_address'), name: 'address', id: 'objects-form-address', invalidText: $tr('objects_error_address'), allowBlank: false, vtype: 'knxga' },
        $extend($O.dtSelect, { store: $O.dataTypes }),
        { fieldLabel: $tr('objects_units'), name: 'units' },
        { fieldLabel: $tr('objects_disablelog'), name: 'disablelog', xtype: 'checkbox' },
        { fieldLabel: $tr('objects_export'), name: 'export', xtype: 'checkbox' },
        { fieldLabel: $tr('objects_pollinterval'), name: 'pollinterval', xtype: 'spinnerfield', allowBlank: true, defaultValue: '', minValue: 1, maxValue: 3600, width: 60 },
        { fieldLabel: $tr('objects_tags'), name: 'tags', xtype: 'tagbox', id: 'objects-form-tags' },
        { fieldLabel: $tr('objects_value'), name: 'data', xtype: 'displayfield', id: 'objects-form-data' },
        { fieldLabel: $tr('objects_comment'), name: 'comment', xtype: 'textarea', width: 405, height: 200 },
        { name: 'id', xtype: 'hidden' }
      ]
    })
  })
);

$O.container = new Ext.Container({
  id: 'Objects',
  title: $tr('objects_title'),
  layout: 'border',
  items: [ $O.filter, $O.grid ]
});

$O.submitForm = function() {
  $O.form.submit({ mode: 'json', waitMsg: $tr('loading') });
};

$O.showForm = function(objectId) {
  $O.form.reset();
  $O.formAddress.disable();
  $O.formWindow.show().center();

  $O.form.objectId = objectId;

  if (objectId) {
    $O.form.load({
      url: Scada.reqUrl('objects', 'form'),
      params: Scada.encodeData({ id: objectId }),
      waitMsg: $tr('loading')
    });
  }
  else if ($O.rawTags) {
    Ext.getCmp('objects-form-tags').updateTags($O.rawTags);
  }
};

$O.hideForm = function() {
  $O.formWindow.hide();
};

$O.update = function(full) {
  var params = {};

  if (full === true) {
    $O.store.removeAll();
    $O.gridStore.removeAll();
  }
  else {
    if ($O.updateTime) {
      params.updatetime = $O.updateTime;
    }

    if ($O.objectId) {
      params.objectid = $O.objectId;
      $O.objectId = null;
    }
  }

  $O.store.load({ add: true, params: params });
};

$O.deleteItem = function(id) {
  var record;

  record = $O.store.getById(id);
  record && $O.store.remove(record);

  record = $O.gridStore.getById(id);
  record && $O.gridStore.remove(record);
}

$O.onConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    $O.deleteItem(this.id);

    Ext.Ajax.request({
      url: Scada.reqUrl('objects', 'delete'),
      params: Scada.encodeData({ id: this.id })
    });
  }
};

$O.isAsciiString = function(value, maxlength) {
  if (typeof value != 'string') {
    return false;
  }

  var i = value.length;
  if (i > maxlength) {
    return false;
  }

  while (i--) {
    if (value.charCodeAt(i - 1) > 255) {
      return false;
    }
  }

  return true;
};

$O.setValueValidate = function(value) {
  var dt, bounds, rawdt;

  rawdt = $O.setValueForm.findField('datatype').getValue();

  dt = Scada.normalizeDt(rawdt);
  bounds = Scada.dtGetBounds(rawdt);

  if (bounds) {
    value = (dt.major == Scada.dt.float16 || dt.major == Scada.dt.float32) ? parseFloat(value) : parseInt(value, 10);
    return !(isNaN(value) || (value < bounds[ 0 ]) || (value > bounds[ 1 ]));
  }
  else if (dt.major == Scada.dt.char) {
    return $O.isAsciiString(value, 1);
  }
  else if (dt.major == Scada.dt.string) {
    return $O.isAsciiString(value, 14);
  }

  return true;
};

$O.setValueFormWindow = new Ext.Window(
  $def.window({
    title: $tr('objects_form_setvalue'), width: 400, height: 250,
    items: $def.form({
      id: 'objects-form-setvalue',
      buttons: $def.formButtons('objects-form-setvalue'),
      items: [
        $def.fieldset({
          items: [
            { xtype: 'displayfield', name: 'name', fieldLabel: $tr('objects_name') },
            { xtype: 'displayfield', name: 'decodedaddress', fieldLabel: $tr('objects_address') },
            { xtype: 'displayfield', name: 'decodeddatatype', fieldLabel: $tr('objects_datatype') }
          ]
        }),
        $def.fieldset({
          id: 'objects-form-setvalue-group-bool',
          items: $def.select({ name: 'boolvalue', fieldLabel: $tr('objects_setvalue_newvalue'), store: [['0', $tr('dt_1_2_0')], ['1', $tr('dt_1_2_1')]] })
        }),
        $def.fieldset({
          id: 'objects-form-setvalue-group-text',
          items: { xtype: 'textfield', name: 'textvalue', fieldLabel: $tr('objects_setvalue_newvalue'), validator: $O.setValueValidate }
        }),
        $def.fieldset({
          id: 'objects-form-setvalue-group-date', items: { xtype: 'datefield', name: 'datevalue', fieldLabel: $tr('objects_setvalue_datevalue'), format: 'd.m.Y', allowBlank: false }
        }),
        $def.fieldset({
          id: 'objects-form-setvalue-group-slider',
          items: {
            xtype: 'sliderfield', name: 'slidervalue', fieldLabel: $tr('objects_setvalue_newvalue'), width: 200,
            minValue: 0, maxValue: 255, plugins: new Ext.slider.Tip(), id: 'objects-form-setvalue-slider'
          }
        }),
        $def.fieldset({
          id: 'objects-form-setvalue-group-spinner',
          items: {
            xtype: 'spinnerfield', name: 'spinnervalue', fieldLabel: $tr('objects_setvalue_newvalue'),
            minValue: 0, maxValue: 255, id: 'objects-form-setvalue-spinner'
          }
        }),
        $def.fieldset({
          id: 'objects-form-setvalue-group-time',
          items: [
            $def.select({
              name: 'timevalued', fieldLabel: $tr('objects_setvalue_weekday'), width: 158,
              store: [
                [0, $tr('dt_noday')], [1, $tr('dt_monday')], [2, $tr('dt_tuesday')], [3, $tr('dt_wednesday')],
                [4, $tr('dt_thursday')], [5, $tr('dt_friday')], [6, $tr('dt_saturday')], [7, $tr('dt_sunday')]
              ]
            }),
            {
              xtype: 'panel', layout: 'column', border: false, fieldLabel: $tr('objects_setvalue_timevalue'), items: [
                { xtype: 'spinnerfield', name: 'timevalueh', zeroPad: true, padLength: 2, minValue: 0, maxValue: 23, width: 50, columnwidth: 0.23 },
                { columnwidth: 0.05, border: false, html: '&nbsp;' },
                { xtype: 'spinnerfield', name: 'timevaluem', zeroPad: true, padLength: 2, minValue: 0, maxValue: 59, width: 50, columnwidth: 0.23 },
                { columnwidth: 0.05, border: false, html: '&nbsp;' },
                { xtype: 'spinnerfield', name: 'timevalues', zeroPad: true, padLength: 2, minValue: 0, maxValue: 59, width: 50, columnwidth: 0.23 }
              ]
            }
          ]
        }),
        $def.fieldset({
          id: 'objects-form-setvalue-group-access',
          items: { xtype: 'displayfield', name: 'accessvalue', value: 'Not implemented' }
        }),
        { xtype: 'hidden', name: 'type' },
        { xtype: 'hidden', name: 'datatype' },
        { xtype: 'hidden', name: 'address' },
        { xtype: 'hidden', name: 'value' }
      ]
    })
  })
);

$O.showSetValueFormRow = function(row) {
  var rawdt = row.get('datatype').toNumber(), addr;

  if (rawdt) {
    addr = Scada.decodeGroupAddress(row.get('address'));

    $O.showSetValueForm(data = {
      row: row.data,
      datatype: rawdt,
      decodeddatatype: $tr('objects_datatype_' + rawdt),
      address: addr,
      decodedaddress: addr,
      name: row.get('name')
    });
  }
  else {
    Ext.Msg.alert(
      $tr('objects_setvalue_error'),
      $tr('objects_setvalue_error_text')
    );
  }
};

$O.showSetValueForm = function(data) {
  var form = $O.setValueForm;

  form.reset();
  $O.setValueFormWindow.show().center();

  var params = data.row.params ? data.row.params : (Ext.util.JSON.decode(data.row.visparams) || {});

  data.type = $O.setValueGetFieldType(data, params);
  if (data.type == 'inline-slider') {
    data.type = 'slider';
  }

  var type, row = data.row, dt = row.statobj ? row.statobj.value : row.datadec;
  for (type in $O.setValueGroups) {
    $O.setValueGroups[ type ].setVisible(data.type == type);
  }

  switch (data.type) {
    case 'slider':
    case 'spinner':
      data[ data.type + 'value' ] = dt;

      var bounds = Scada.dtGetBounds(row.rawdatatype || row.datatype, params);
      if (bounds) {
        var field = form.findField(data.type + 'value');
        field.setMinValue(bounds[ 0 ]);
        field.setMaxValue(bounds[ 1 ]);
      }

      if (data.type == 'slider') {
        field.setValue(data.slidervalue, false);
      }
    break;
    case 'bool':
      data.boolvalue = dt ? 0 : 1;
    break;
    case 'date':
      dt = new Date();
      data.datevalue = dt.format('d.m.Y');
    break;
    case 'time':
      data.timevalued = dt.day;
      data.timevalueh = dt.hour;
      data.timevaluem = dt.minute;
      data.timevalues = dt.second;
    break;
    default:
      data.textvalue = dt.toString();
    break;
  }

  form.setValues(data);
};

$O.setValueSubmitForm = function() {
  var type, field, value, form;

  form = $O.setValueForm;
  type = form.findField('type').getValue();
  if (type == 'inline-slider') {
    type = 'slider';
  }

  switch (type) {
    case 'text':
      field = form.findField('textvalue');
      value = field.getValue();
    break;
    case 'bool':
      value = form.findField('boolvalue').getValue().toNumber() ? 'true' : 'false';
    break;
    case 'date':
      field = form.findField('datevalue');
      var dt = field.getValue();

      if (field.isValid()) {
        value = Ext.util.JSON.encode({
          day: dt.getDate(),
          month: dt.getMonth() + 1,
          year: dt.getFullYear()
        });
      }
    break;
    case 'time':
      value = Ext.util.JSON.encode({
        day: (form.findField('timevalued').getValue() || 0),
        hour: (form.findField('timevalueh').getValue() || 0),
        minute: (form.findField('timevaluem').getValue() || 0),
        second: (form.findField('timevalues').getValue() || 0)
      });
    break;
    case 'slider':
    case 'spinner':
      field = form.findField(type + 'value');
      type = 'text';
      value = field.getValue().toNumber();
    break;
    default:
      value = '';
    break;
  }

  if (!field || field.isValid()) {
    var w = $O.setValueFormWindow;
    w.setDisabled(true);

    Ext.Ajax.request({
      url: Scada.reqUrl('objects', 'setvalue'),
      method: 'post',
      params: {
        data: Ext.util.JSON.encode({
          value: value,
          type: type,
          datatype: form.findField('datatype').getValue(),
          address: form.findField('address').getValue()
        })
      },
      success: function() {
        w.setDisabled(false);
        w.hide();
      },
      failure: function() {
        w.setDisabled(false);
        w.hide();
      }
    });
  }
};

$O.setValueGroupTypes = [ 'bool', 'time', 'text', 'slider', 'spinner', 'date', 'access' ];

$O.setValueGetFieldType = function(obj, params) {
  var dt = Scada.normalizeDt(obj.datatype), mapping = {};

  mapping[ Scada.dt.bool ] = 'bool';
  mapping[ Scada.dt.time ] = 'time';
  mapping[ Scada.dt.date ] = 'date';
  mapping[ Scada.dt.access ] = 'access';

  if (Scada.dtIsNumeric(dt.major)) {
    return params.control || (dt.major == Scada.dt.uint8 && dt.minor == Scada.minorDt.scale ? 'slider' : 'spinner');
  }
  else {
    return (mapping[ dt.major ] || 'text');
  }
};

$O.setValueHideForm = function() {
  $O.setValueFormWindow.hide();
};

$O.setValueInit = function() {
  $O.setValueForm = Ext.getCmp('objects-form-setvalue').getForm();

  Ext.getCmp('objects-form-setvalue-submit').setHandler($O.setValueSubmitForm);
  Ext.getCmp('objects-form-setvalue-cancel').setHandler($O.setValueHideForm);

  $O.setValueGroups = {};
  var g = 'objects-form-setvalue-group-';
  Ext.each($O.setValueGroupTypes, function(k) {
    $O.setValueGroups[ k ] = Ext.getCmp(g + k);
  });

  $O.setValueForm.on('actioncomplete', function(form, action) {
    if (action.type == 'submit') {
      $O.update();
      $O.setValueHideForm();
    }
  });
};

$O.checkUpdate = function() {
  Ext.Ajax.request({
    url: Scada.genUrl('objects-update'),
    params: { upd: $O.updateTime },
    success: $O.checkUpdateCb,
    failure: $O.checkUpdateCb,
    timeout: 90 * 1000
  });
};

$O.listeners = {};

$O.addListener = function(id, fn) {
  $O.listeners[ id ] = fn;
};

$O.removeListener = function(id) {
  delete $O.listeners[ id ];
};

$O.notify = function() {
  var id;
  for (id in $O.listeners) {
    $O.listeners[ id ]();
  }
};

$O.checkUpdateCb = function(result) {
  if (result && result.responseText) {
    var updateTime = result.responseText.trim();

    if (updateTime != $O.updateTime) {
      if ($O.autoUpdate) {
        $O.update();
      }

      $O.updateTime = updateTime;
      $O.notify();
    }
  }

  $O.checkUpdate.defer(1000);
};

$O.filterCb = function(r) {
  if (!$O.filterBy) {
    return true;
  }

  var data = r.data, dt = $O.filterBy.dt, match = true;

  if (dt) {
    var odt = Scada.normalizeDt(data.datatype || 0);

    if (odt.major != dt.major) {
      return false;
    }

    if (dt.minor && odt.minor != dt.minor) {
      return false;
    }
  }

  if ($O.filterBy.name && !$O.filterBy.name.test(data.name)) {
    return false;
  }

  if ($O.filterBy.address) {
    var addr = Scada.decodeGroupAddress(data.id);
    if (!addr.match($O.filterBy.address)) {
      return false;
    }
  }

  if ($O.filterBy.tags) {
    var matches = 0, search = $O.filterBy.tags, tags = data.tagcache.toList();

    Ext.each(search, function(tag) {
      if (tags.indexOf(tag) >= 0) {
        console.log(tags, tag);
        matches++;
      }
    });

    if (matches != search.length) {
      return false;
    }
  }

  return true;
};

$O.submitVisParams = function(fw, action, form) {
  $O.objectId = form.findField('id').getValue();
  $O.update();
}

$O.visParamsFW_bool = new ExtFormWindow({
  id: 'objects-vis-params-bool',
  window: {
    title: $tr('objects_vis_params'),
    width: 400,
    height: 150
  },
  form: {
    url: Scada.reqUrl('objects', 'setvisparams'),
    labelWidth: 150,
    defaults: { width: 180 },
    items: [
      { xtype: 'hidden', name: 'id' },
      { xtype: 'displayfield', name: 'name', fieldLabel: $tr('objects_obj_title') },
      $def.select({
        name: 'control',
        hiddenName: 'control',
        fieldLabel: $tr('objects_vis_params_control'),
        store: [
          [ 'toggle', $tr('objects_vis_params_control_toggle') ],
          [ 'checkbox', $tr('objects_vis_params_control_checkbox') ]
        ]
      })
    ]
  },
  events: { onFormSubmitComplete: $O.submitVisParams, onFormSubmitFailed: $O.submitVisParams }
});

$O.visParamsFW_bit4 = new ExtFormWindow({
  id: 'objects-vis-params-bit4',
  window: {
    title: $tr('objects_vis_params'),
    width: 400,
    height: 150
  },
  form: {
    url: Scada.reqUrl('objects', 'setvisparams'),
    labelWidth: 150,
    defaults: { width: 180 },
    items: [
      { xtype: 'hidden', name: 'id' },
      { xtype: 'displayfield', name: 'name', fieldLabel: $tr('objects_obj_title') },
      $def.select({
        name: 'step',
        hiddenName: 'step',
        fieldLabel: $tr('objects_vis_params_bit4_step'),
        store: [ [ '7', '1%' ], [ '6', '3%' ], [ '5', '6%' ], [ '4', '12%' ], [ '3', '25%' ], [ '2', '50%' ], [ '1', '100%' ] ]
      })
    ]
  },
  events: { onFormSubmitComplete: $O.submitVisParams, onFormSubmitFailed: $O.submitVisParams }
});

$O.visParamsFW_numeric = new ExtFormWindow({
  id: 'objects-vis-params-numeric',
  window: {
    title: $tr('objects_vis_params'),
    width: 400,
    height: 230
  },
  form: {
    url: Scada.reqUrl('objects', 'setvisparams'),
    labelWidth: 150,
    defaults: { width: 180 },
    items: [
      { xtype: 'hidden', name: 'id' },
      { xtype: 'displayfield', name: 'name', fieldLabel: $tr('objects_obj_title') },
      $def.select({
        name: 'control',
        hiddenName: 'control',
        fieldLabel: $tr('objects_vis_params_control'),
        store: [
          [ 'spinner', $tr('objects_vis_params_control_step') ],
          [ 'slider', $tr('objects_vis_params_control_slider') ]
        ]
      }),
      { xtype: 'spinnerfield', name: 'min', width: 100, fieldLabel: $tr('objects_vis_params_min') },
      { xtype: 'spinnerfield', name: 'max', width: 100, fieldLabel: $tr('objects_vis_params_max') },
      { xtype: 'spinnerfield', name: 'step', width: 100, fieldLabel: $tr('objects_vis_params_step') }
    ]
  },
  events: { onFormSubmitComplete: $O.submitVisParams, onFormSubmitFailed: $O.submitVisParams }
});

$O.showVisParams = function(obj) {
  var dt = Scada.normalizeDt(obj.datatype), win, form, values = { id: obj.id }, params, bounds;

  values.name = Scada.decodeGroupAddress(obj.address);
  if (obj.name) {
    values.name = obj.name + ' (' + values.name + ')';
  }

  params = Ext.util.JSON.decode(obj.visparams) || {};

  if (dt.major == Scada.dt.bool) {
    win = $O.visParamsFW_bool;
    values.control = params.control || 'toggle';
  }
  else if (dt.major == Scada.dt.bit4) {
    win = $O.visParamsFW_bit4;
    values.step = params.step || 3;
  }
  else if (Scada.dtIsNumeric(dt.major)) {
    win = $O.visParamsFW_numeric;

    bounds = Scada.dtGetBounds(obj.datatype);
    values.min = params.min || bounds[ 0 ];
    values.max = params.max || bounds[ 1 ];
    values.step = params.step || '';
    values.control = params.control || (dt.major == Scada.dt.uint8 && dt.minor == Scada.minorDt.scale ? 'slider' : 'spinner');
  }

  if (win) {
    form = win.formPanel.getForm();

    form.reset();

    if (Scada.dtIsNumeric(dt.major)) {
      form.findField('min').setMinValue(bounds[ 0 ]);
      form.findField('min').setMaxValue(bounds[ 1 ]);

      form.findField('max').setMinValue(bounds[ 0 ]);
      form.findField('max').setMaxValue(bounds[ 1 ]);
    }

    win.show();
    form.setValues(values);
  }
  else {
    Ext.Msg.alert(
      $tr('objects_vis_params'),
      $tr('objects_vis_params_alert')
    );
  }
}

$O.enumsId = 0;

$O.enumsDeleteRow = function(cmp) {
  var fw = $O.enumsWindow;

  if (cmp.items) {
    cmp.items.each(function(item) {
      fw.form.remove(item, true);
    });
  }

  fw.form.remove(cmp, true);
  fw.formPanel.remove(cmp, true);
}

$O.enumsDeleteItem = function() {
  var id = this.id.replace('_delete', ''), cmp = Ext.getCmp(id);
  $O.enumsDeleteRow(cmp);

  if ($O.enumsButton) {
    $O.enumsButton.show();
  }
}

$O.enumsSubmit = function(fw) {
  var values, store = {}, enums = {}, count = 0;

  if (fw.form.isValid()) {
    values = fw.form.getValues();

    Ext.iterate(values, function(key, value) {
      var chunks, name, id;

      if (key == 'default') {
        enums['default'] = value;
      }
      else {
        chunks = key.split('_');
        name = chunks[0];
        id = chunks[1];

        if (!store[ id ]) {
          store[ id ] = {};
        }

        store[ id ][ name ] = name == 'text' ? value : value.toNumber();
      }
    });

    Ext.iterate(store, function(id, values) {
      count++;
      enums[ values.value ] = values.text;
    });

    fw.window.setDisabled(true);

    Ext.Ajax.request({
      url: Scada.reqUrl('objects', 'setenums'),
      params: Scada.encodeData({
        id: $O.enumsObj.id,
        enums: (count ? Ext.util.JSON.encode(enums) : '')
      }),
      success: function() {
        $O.objectId = $O.enumsObj.id;
        $O.update();

        fw.window.setDisabled(false);
        fw.hide();
      }
    });
  }
}

$O.enumsAddItem = function(value, text) {
  var id = $O.enumsId, fp = $O.enumsWindow.formPanel, items = fp.items, index = items.length - 1, bounds, prev, max;
  bounds = Scada.dtGetBounds($O.enumsObjDt);

  if (value == 'default') {
    $O.enumsWindow.form.setValues({ 'default': text });
    return;
  }
  else if (typeof value != 'string' && typeof value != 'number') {
    value = 0;
    text = '';

    if (index > 0) {
      try {
        items = items.items[ index - 1 ].items.items;
        prev = items[ 1 ].getValue().toNumber();

        if (prev !== null) {
          value = prev + 1;
        }
      }
      catch (e) {}
    }
  }

  value = value.toNumber() || 0;
  value = value.constrain(bounds[0], bounds[1]);

  fp.insert(index, {
    id: 'objects_enums_row_' + id,
    xtype: 'compositefield',
    hideLabel: true,
    width: 420,
    items: [
      { xtype: 'displayfield', value: $tr('objects_enums_value'), width: 80 },
      { xtype: 'spinnerfield', minValue: bounds[0], maxValue: bounds[1], allowBlank: false, allowDecimals: false, name: 'value_' + id, width: 80, fieldLabel: $tr('vis_min_value'), value: value },
      { xtype: 'displayfield', value: $tr('objects_enums_text') },
      { xtype: 'textfield', name: 'text_' + id, value: text },
      { xtype: 'button', iconCls: 'icon-delete', width: 16, id: 'objects_enums_row_' + id + '_delete', handler: $O.enumsDeleteItem }
    ]
  });

  try { fp.doLayout(); } catch (e) {}
  $O.enumsId++;

  max = bounds[ 1 ] - bounds[ 0 ];
  if (max < index) {
    $O.enumsButton.hide();
  }
}

$O.enumsShow = function(obj) {
  var fp = $O.enumsWindow.formPanel, dt = Scada.normalizeDt(obj.datatype), items;

  if ((dt.major == Scada.dt.bool || Scada.dtIsNumeric(dt.major)) && dt.major != Scada.dt.float16 && dt.major != Scada.dt.float32) {
    $O.enumsObj = obj;
    $O.enumsObjDt = dt;

    fp.items.each($O.enumsDeleteRow);

    fp.add({ xtype: 'textfield', name: 'default', fieldLabel: $tr('objects_enums_default'), width: 145 });

    $O.enumsButton = fp.add({
      xtype: 'button',
      text: $tr('objects_enums_add'),
      handler: $O.enumsAddItem,
      iconCls: 'icon-add',
      width: 'auto'
    });

    $O.enumsWindow.show();

    try {
      items = Ext.util.JSON.decode(obj.enums);
      if (typeof items == 'object') {
        Ext.iterate(items, $O.enumsAddItem);
      }
    }
    catch (e) {}

    try { fp.doLayout(); } catch (e) {}
  }
  else {
    Ext.Msg.alert(
      $tr('objects_enums'),
      $tr('objects_enums_error')
    );
  }
}

$O.enumsWindow = new ExtFormWindow({
  id: 'scada-objects-enums-window',
  window: {
    title: $tr('objects_enums'),
    width: 500,
    height: 400,
  },
  form: {
    autoScroll: true,
    labelWidth: 80,
    items: []
  },
  events: { onSubmit: $O.enumsSubmit }
});

$O.onBlur = function() {
  var items, objData = [], numObjData = [];
  items = $O.store.getRange().sort(function(a, b) { return a.id - b.id; });

  $O.extObjStore.removeAll();
  $O.extNumObjStore.removeAll();

  Ext.each(items, function(item) {
    var data = item.data, dt, row, text;

    text = Scada.decodeGroupAddress(data.id);
    if (data.name) {
      text += ' ' + data.name;
    }

    row = [ data.id, text, data.datatype ];
    objData.push(row);

    if (data.datatype) {
      dt = Scada.normalizeDt(data.datatype);

      if (Scada.dtIsNumeric(dt.major)) {
        numObjData.push(row);
      }
    }
  });

  $O.extObjStore.loadData(objData);
  $O.extNumObjStore.loadData(numObjData);
}

$O.updateRowHighlight = function() {
  Ext.iterate($O.updateTimers, function(id) {
    $O.updateTimers[ id ]--;
    if (!$O.updateTimers[ id ]) {
      delete $O.updateTimers[ id ];
    }
  });

  if ($O.grid.rendered) {
    $O.grid.getView().refresh();
  }
}

$O.init = function() {
  $O.checkUpdate();

  $O.formPanel = Ext.getCmp('objects-form');
  $O.form = $O.formPanel.getForm();
  $O.formData = Ext.getCmp('objects-form-data');
  $O.formAddress = Ext.getCmp('objects-form-address');

  Ext.getCmp('objects-form-submit').setHandler($O.submitForm);
  Ext.getCmp('objects-form-cancel').setHandler($O.hideForm);

  Ext.getCmp('objects-add').setHandler(function() {
    var addr = Scada.findFreeAddress();
    $O.showForm();
    $O.formData.setValue('&mdash;');
    if (addr) {
      addr = Scada.decodeGroupAddress(addr);
      $O.formAddress.setValue(addr);
    }
    $O.formAddress.enable();
  });

  Ext.getCmp('objects-auto-update').on('toggle', function() {
    $O.autoUpdate = !this.pressed;
    this.setIconClass($O.autoUpdate ? 'icon-accept' : 'icon-delete');
    this.setText($tr('objects_auto_update_' + ($O.autoUpdate ? 'on' : 'off')));
  });

  $O.form.on('actioncomplete', function(form, action) {
    if (action.type == 'load') {
      var objectValue = '&mdash;', objectAddress = '&mdash;';
      if (action.result.data) {
        var data = action.result.data;
        if (data.datatype) {
          objectValue = Scada.decodeValue(data.datadec, data.datatype.toNumber(), data.units, data.enums);
        }
        if (data.address) {
          objectAddress = Scada.decodeGroupAddress(data.address);
        }
      }

      $O.formData.setValue(objectValue);
      $O.formAddress.setValue(objectAddress);
    }
    else if (action.type == 'submit') {
      $O.hideForm.apply(this);
      $O.objectId = $O.form.objectId;
      $O.update();
    }
  });

  $O.grid.on('cellclick', function(grid, rowIndex, columnIndex) {
    var row = grid.getStore().getAt(rowIndex);
    var id = row.get('id').toNumber();
    var fieldName = grid.getColumnModel().getDataIndex(columnIndex);

    if (fieldName == 'delete') {
      Ext.Msg.confirm(
        $tr('objects_delete_title'),
        $tr('objects_delete_confirm'),
        $O.onConfirm,
        { id: id }
      );
    }
    else if (fieldName == 'setvalue') {
      $O.showSetValueFormRow(row);
    }
    else if (fieldName == 'visparams') {
      $O.showVisParams(row.data);
    }
    else if (fieldName == 'enums') {
      $O.enumsShow(row.data);
    }
    else if (fieldName == 'script') {
      $S.showEditor(id, true);
    }
    else {
      $O.showForm.apply(this, [ id ]);
    }
  });

  $O.setValueInit();
  Ext.getCmp('objects-form-setvalue-slider').slider.on('change', function(s, v) {
    if (!isNaN(v)) {
      Ext.getCmp('objects-form-setvalue-slider').label.update($tr('objects_setvalue_newvalue') + ' (' + v + '):');
    }
  });

  $O.store.on('beforeload', function() {
    $O.pagination.refresh.disable();
    $O.pagination.disableAll();
  });

  $O.filters = {
    name: Ext.getCmp('objects-filter-name'),
    dt: Ext.getCmp('objects-filter-dt'),
    tags: Ext.getCmp('objects-filter-tags')
  };

  $O.store.on('load', function(store) {
    var data = store.reader.jsonData.data, sort = $O.gridStore.getSortState();

    $O.rawTags = store.reader.jsonData;
    $O.filters.tags.updateTags($O.rawTags);

    $O.gridStore.loadData(data, true);
    $O.gridStore.sort(sort.field, sort.direction);
    $O.gridStore.filterBy($O.filterCb);

    if (!$O.pagination.init) {
      $O.pagination.init = true;
      $O.pagination.moveFirst();
    }

    $O.pagination.refresh.enable();

    if (!$O.extStoreReady) {
      $O.onBlur();
      $O.autoUpdate = true;
      $O.extStoreReady = true;
    }
  });

  $O.pagination.refresh.handler = $O.update;
  $O.store.load();
  setInterval($O.updateRowHighlight, 1000);
};

Scada.Modules.push($O);
