#!/usr/bin/haserl
Content-Type: text/javascript

Scada.Schedulers = {};
var $SC = Scada.Schedulers;

$SC.npad = function(v) {
  return v < 10 ? '0' + v.toString() : v;
}

$SC.months = [
  '',
  $tr('scripting_dt_month_jan'),
  $tr('scripting_dt_month_feb'),
  $tr('scripting_dt_month_mar'),
  $tr('scripting_dt_month_apr'),
  $tr('scripting_dt_month_may'),
  $tr('scripting_dt_month_jun'),
  $tr('scripting_dt_month_jul'),
  $tr('scripting_dt_month_aug'),
  $tr('scripting_dt_month_sep'),
  $tr('scripting_dt_month_oct'),
  $tr('scripting_dt_month_nov'),
  $tr('scripting_dt_month_dec')
];

$SC.monthStore = [];
(function() {
  var i;
  for (i = 1; i <= 12; i++) {
    $SC.monthStore.push([ i, $SC.months[ i ] ]);
  }
})();

$SC.mainStore = new Ext.data.JsonStore({
  url: Scada.reqUrl('schedulers'),
  root: 'data',
  fields: [ 'id', 'name', 'active', 'object', 'object_name', 'start_month', 'start_day', 'end_month', 'end_day' ]
});

$SC.holidaysStore = new Ext.data.JsonStore({
  url: Scada.reqUrl('schedulers', 'holidays-main'),
  root: 'data',
  fields: [ 'id', 'name', 'year', 'month', 'day' ]
});

$SC.eventsStore = new Ext.data.JsonStore({
  url: Scada.reqUrl('schedulers', 'events-main'),
  root: 'data',
  fields: [ 'id', 'active', 'scheduler', { name: 'start_hour', type: 'integer' }, { name: 'start_min', type: 'integer' }, 'value', 'start_days' ]
});

$SC.buttons = [
  {
    id: 'schedulers-btn-main',
    text: $tr('schedulers_title'),
    iconCls: 'scada-scripting-btn-scheduled',
    xtype: 'splitbutton', scale: 'large', iconAlign: 'bottom', arrowAlign: 'bottom', minWidth: 120,
    menu: {
      items: [
        { text: $tr('scripting_list_view'), iconCls: 'icon-list-view', id: 'schedulers-menu-main-list' },
        { text: $tr('schedulers_btn_main_add'), iconCls: 'icon-add', id: 'schedulers-menu-main-add' }
      ]
    }
  },
  {
    id: 'schedulers-btn-holidays',
    text: $tr('schedulers_holidays'),
    iconCls: 'scada-scripting-btn-event',
    xtype: 'splitbutton', scale: 'large', iconAlign: 'bottom', arrowAlign: 'bottom', minWidth: 120,
    menu: {
      items: [
        { text: $tr('scripting_list_view'), iconCls: 'icon-list-view', id: 'schedulers-menu-holidays-list' },
        { text: $tr('schedulers_btn_holidays_add'), iconCls: 'icon-add', id: 'schedulers-menu-holidays-add'  }
      ]
    }
  }
];

$SC.dateRenderer = function(nr, cell, item, row, col) {
  var data = item.data, key = (col == 2) ? 'start_' : 'end_', month = data[ key + 'month' ];
  return $SC.npad(data[ key + 'day' ]) + ' ' + $SC.months[ month ];
}

$SC.objRenderer = function(nr, cell, item) {
  var data = item.data, suffix = '';
  if (data.object_name) {
    suffix = ' (' + data.object_name + ')';
  }

  return Scada.decodeGroupAddress(data.object) + suffix;
}

$SC.mainGrid = new Ext.grid.GridPanel($def.grid({
  id: 'schedulers-grid-main',
  store: $SC.mainStore,
  ctCls: 'scada-list-view-grid',
  loadMask: true,
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('schedulers_name'), width: 0.20, dataIndex: 'name' },
      { header: $tr('schedulers_object'), width: 0.25, renderer: $SC.objRenderer },
      { header: $tr('schedulers_start'), width: 0.20, renderer: $SC.dateRenderer },
      { header: $tr('schedulers_end'), width: 0.20, renderer: $SC.dateRenderer },
      { header: $tr('schedulers_events'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-object-edit', dataIndex: 'events' },
      { header: $tr('schedulers_active'), width: 0.05, xtype: 'boolclscolumn', dataIndex: 'active' },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-delete', dataIndex: 'delete' }
    ]
  })
}));

$SC.holidayDateRenderer = function(nr, cell, item) {
  var data = item.data;
  return $SC.npad(data.day) + ' ' + $SC.months[ data.month ] + (data.year > 0 ? (' ' + data.year) : '');
}

$SC.holidaysGrid = new Ext.grid.GridPanel($def.grid({
  id: 'schedulers-grid-holidays',
  store: $SC.holidaysStore,
  ctCls: 'scada-list-view-grid',
  loadMask: true,
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('schedulers_name'), width: 0.45, dataIndex: 'name' },
      { header: $tr('schedulers_date'), width: 0.50, renderer: $SC.holidayDateRenderer },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-delete', dataIndex: 'delete' }
    ]
  })
}));

$SC.timeRenderer = function(nr, cell, item) {
  var data = item.data;
  return $SC.npad(data.start_hour) + ':' + $SC.npad(data.start_min);
}

$SC.dayRenderer = function(nr, cell, item) {
  var data = item.data, i, days = [];

  for (i = 0; i < 8; i++) {
    if (data.start_days[ i ] == '1') {
      days.push($tr('scripting_dt_weekday_' + (i + 1)));
    }
  }

  return days.join(', ');
}

$SC.eventsGrid = new Ext.grid.GridPanel($def.grid({
  id: 'schedulers-grid-events',
  store: $SC.eventsStore,
  ctCls: 'scada-list-view-grid',
  loadMask: true,
  border: false,
  bbar: [
    { text: $tr('schedulers_events_add'), id: 'schedulers-add-events', iconCls: 'icon-add' }
  ],
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('schedulers_time'), width: 0.20, renderer: $SC.timeRenderer },
      { header: $tr('schedulers_event_days'), width: 0.40, renderer: $SC.dayRenderer },
      { header: $tr('schedulers_value'), width: 0.20, dataIndex: 'value' },
      { header: $tr('schedulers_active'), width: 0.10, xtype: 'boolclscolumn', dataIndex: 'active' },
      { header: '', width: 0.10, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-delete', dataIndex: 'delete' }
    ]
  })
}));

$SC.eventsWindow = new Ext.Window($def.window({
  height: 300,
  items: $SC.eventsGrid
}));

$SC.mainPanel = new Ext.Panel({
  id: 'schedulers-container',
  layout: 'card',
  margins: '10 10 10 10',
  border: false,
  activeItem: 0,
  flex: 1,
  items: [ $SC.mainGrid, $SC.holidaysGrid ]
});

$SC.container = new Ext.Container({
  id: 'Schedulers',
  title: $tr('schedulers_title'),
  layout: 'vbox',
  layoutConfig: { align : 'stretch' },
  items: [
    {
      layout: 'column',
      ctCls: 'x-scripting-menu',
      autoHeight: false,
      border: false,
      margins: '10 10 0 10',
      height: 71,
      items: $SC.buttons
    },
    $SC.mainPanel
  ]
});

$SC.mainStoreReload = function() {
  $SC.mainStore.reload();
}

$SC.eventsStoreReload = function() {
  $SC.eventsStore.reload();
}

$SC.holidaysStoreReload = function() {
  $SC.holidaysStore.reload();
}

$SC.mainFW = new ExtFormWindow({
  id: 'schedulers-form',
  window: {
    title: $tr('schedulers_form'),
    width: 460,
    height: 230
  },
  form: {
    url: Scada.reqUrl('schedulers', 'save'),
    loadUrl: Scada.reqUrl('schedulers', 'form'),
    items: [
      $def.select({
        fieldLabel: $tr('schedulers_object'), name: 'object', allowBlank: false, hiddenName: 'object', editable: true,
        mode: 'local', valueField: 'id', displayField: 'text', store: $O.extObjStore
      }),
      { name: 'active', fieldLabel: $tr('schedulers_active'), xtype: 'checkbox' },
      { name: 'name', fieldLabel: $tr('schedulers_name'), allowBlank: false },
      {
        xtype: 'compositefield',
        fieldLabel: $tr('schedulers_start'),
        items: [
          { name: 'start_day', xtype: 'spinnerfield', allowBlank: false, minValue: 1, maxValue: 31, zeroPad: true, padLength: 2, width: 60 },
          $def.select({ name: 'start_month', hiddenName: 'start_month', allowBlank: false, store: $SC.monthStore, width: 185 })
        ]
      },
      {
        xtype: 'compositefield',
        fieldLabel: $tr('schedulers_end'),
        items: [
          { name: 'end_day', xtype: 'spinnerfield', allowBlank: false, minValue: 1, maxValue: 31, zeroPad: true, padLength: 2, width: 60 },
          $def.select({ name: 'end_month', hiddenName: 'end_month', allowBlank: false, store: $SC.monthStore, width: 185 })
        ]
      },
      { name: 'id', xtype: 'hidden' }
    ]
  },
  events: { onFormSubmitComplete: $SC.mainStoreReload, onFormSubmitFailed: $SC.mainStoreReload }
});

$SC.eventsFW = new ExtFormWindow({
  id: 'schedulers-events-form',
  window: {
    title: $tr('schedulers_events_form'),
    width: 460,
    height: 230
  },
  form: {
    url: Scada.reqUrl('schedulers', 'events-save'),
    loadUrl: Scada.reqUrl('schedulers', 'events-form'),
    items: [
      { name: 'active', fieldLabel: $tr('schedulers_active'), xtype: 'checkbox' },
      { name: 'value', fieldLabel: $tr('schedulers_value'), allowBlank: false },
      {
        xtype: 'compositefield',
        fieldLabel: $tr('schedulers_time'),
        items: [
          { name: 'start_hour', xtype: 'spinnerfield', allowBlank: false, minValue: 0, maxValue: 23, zeroPad: true, padLength: 2, width: 50 },
          { name: 'start_min', xtype: 'spinnerfield', allowBlank: false, minValue: 0, maxValue: 59, zeroPad: true, padLength: 2, width: 50 },
        ]
      },
      {
        xtype: 'checkboxgroup',
        fieldLabel: $tr('schedulers_event_days'),
        columns: 5,
        items: [
          { boxLabel: $tr('scripting_dt_weekday_1'), name: 'day_1' },
          { boxLabel: $tr('scripting_dt_weekday_2'), name: 'day_2' },
          { boxLabel: $tr('scripting_dt_weekday_3'), name: 'day_3' },
          { boxLabel: $tr('scripting_dt_weekday_4'), name: 'day_4' },
          { boxLabel: $tr('scripting_dt_weekday_5'), name: 'day_5' },
          { boxLabel: $tr('scripting_dt_weekday_6'), name: 'day_6' },
          { boxLabel: $tr('scripting_dt_weekday_7'), name: 'day_7' },
          { boxLabel: $tr('scripting_dt_weekday_8'), name: 'day_8' }
        ]
      },
      { name: 'id', xtype: 'hidden' },
      { name: 'scheduler', xtype: 'hidden' }
    ]
  },
  events: { onFormSubmitComplete: $SC.eventsStoreReload, onFormSubmitFailed: $SC.eventsStoreReload }
});

$SC.holidaysFW = new ExtFormWindow({
  id: 'schedulers-holidays-form',
  window: {
    title: $tr('schedulers_holidays_form'),
    width: 460,
    height: 180
  },
  form: {
    url: Scada.reqUrl('schedulers', 'holidays-save'),
    loadUrl: Scada.reqUrl('schedulers', 'holidays-form'),
    items: [
      { name: 'name', fieldLabel: $tr('schedulers_name'), allowBlank: false },
      {
        xtype: 'compositefield',
        fieldLabel: $tr('schedulers_date'),
        items: [
          { name: 'day', xtype: 'spinnerfield', allowBlank: false, minValue: 1, maxValue: 31, zeroPad: true, padLength: 2, width: 60 },
          $def.select({ name: 'month', hiddenName: 'month', allowBlank: false, store: $SC.monthStore, width: 100 }),
          { name: 'year', xtype: 'numberfield', allowDecimals: false, width: 80 },
        ]
      },
      $def.formInfoPanel('schedulers_date_hint'),
      { name: 'id', xtype: 'hidden' }
    ]
  },
  events: { onFormSubmitComplete: $SC.holidaysStoreReload, onFormSubmitFailed: $SC.holidaysStoreReload }
});

$SC.onConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    $SC.mainGrid.loadMask.show();

    Ext.Ajax.request({
      url: Scada.reqUrl('schedulers', this.mode),
      success: $SC.mainStoreReload,
      failure: $SC.mainStoreReload,
      params: Scada.encodeData({ id: this.id })
    });
  }
}

$SC.onEventsConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    $SC.eventsGrid.loadMask.show();

    Ext.Ajax.request({
      url: Scada.reqUrl('schedulers', 'events-' + this.mode),
      success: $SC.eventsStoreReload,
      failure: $SC.eventsStoreReload,
      params: Scada.encodeData({ id: this.id })
    });
  }
}

$SC.onHolidaysConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    $SC.holidaysGrid.loadMask.show();

    Ext.Ajax.request({
      url: Scada.reqUrl('schedulers', 'holidays-delete'),
      success: $SC.holidaysStoreReload,
      failure: $SC.holidaysStoreReload,
      params: Scada.encodeData({ id: this.id })
    });
  }
}

$SC.onFocus = function() {
  if (!$SC.ready) {
    $SC.mainStore.load();
    $SC.eventsStore.load();
    $SC.holidaysStore.load();

    $SC.ready = true;
  }
}

$SC.init = function() {
  Ext.getCmp('schedulers-btn-main').on('click', function() {
    $SC.mainPanel.layout.setActiveItem('schedulers-grid-main');
  });
  Ext.getCmp('schedulers-menu-main-list').on('click', function() {
    $SC.mainPanel.layout.setActiveItem('schedulers-grid-main');
  });
  Ext.getCmp('schedulers-menu-main-add').on('click', function() {
    $SC.mainFW.setLoadParams();
    $SC.mainFW.show();
  });

  Ext.getCmp('schedulers-btn-holidays').on('click', function() {
    $SC.mainPanel.layout.setActiveItem('schedulers-grid-holidays');
  });
  Ext.getCmp('schedulers-menu-holidays-list').on('click', function() {
    $SC.mainPanel.layout.setActiveItem('schedulers-grid-holidays');
  });
  Ext.getCmp('schedulers-menu-holidays-add').on('click', function() {
    $SC.holidaysFW.setLoadParams();
    $SC.holidaysFW.show();
  });

  Ext.getCmp('schedulers-add-events').on('click', function() {
    $SC.eventsFW.setLoadParams({ scheduler: $SC.eventsSchedulerId });
    $SC.eventsFW.show();
  });

  $SC.mainGrid.on('cellclick', function(grid, rowIndex, columnIndex) {
    var item = grid.getStore().getAt(rowIndex), id = item.get('id').toNumber(), fieldName = grid.getColumnModel().getDataIndex(columnIndex);

    if (fieldName == 'delete' || fieldName == 'active') {
      var mode = fieldName == 'active' ? 'status' : 'delete';
      Ext.Msg.confirm(
        $tr('schedulers_' + mode + '_title'),
        $tr('schedulers_' + mode + '_confirm'),
        $SC.onConfirm,
        { name: this.name, id: id, mode: mode }
      );
    }
    else if (fieldName == 'events') {
      $SC.eventsSchedulerId = id;
      $SC.eventsStore.filter('scheduler', id);
      $SC.eventsWindow.setTitle($tr('schedulers_events_title') + ' ' + item.data.name);
      $SC.eventsWindow.show();
    }
    else {
      $SC.mainFW.setLoadParams({ id: id });
      $SC.mainFW.show();
    }
  });

  $SC.eventsStore.on('load', function() {
    if ($SC.eventsSchedulerId) {
      $SC.eventsStore.filter('scheduler', $SC.eventsSchedulerId);
    }
  });

  $SC.eventsGrid.on('cellclick', function(grid, rowIndex, columnIndex) {
    var item = grid.getStore().getAt(rowIndex), id = item.get('id').toNumber(), fieldName = grid.getColumnModel().getDataIndex(columnIndex);

    if (fieldName == 'delete' || fieldName == 'active') {
      var mode = fieldName == 'active' ? 'status' : 'delete';
      Ext.Msg.confirm(
        $tr('schedulers_events_' + mode + '_title'),
        $tr('schedulers_events_' + mode + '_confirm'),
        $SC.onEventsConfirm,
        { name: this.name, id: id, mode: mode }
      );
    }
    else {
      $SC.eventsFW.setLoadParams({ id: id, scheduler: $SC.eventsSchedulerId });
      $SC.eventsFW.show();
    }
  });

  $SC.holidaysGrid.on('cellclick', function(grid, rowIndex, columnIndex) {
    var id = grid.getStore().getAt(rowIndex).get('id').toNumber();
    var fieldName = grid.getColumnModel().getDataIndex(columnIndex);

    if (fieldName == 'delete') {
      Ext.Msg.confirm(
        $tr('schedulers_holidays_delete_title'),
        $tr('schedulers_holidays_delete_confirm'),
        $SC.onHolidaysConfirm,
        { name: this.name, id: id }
      );
    }
    else {
      $SC.holidaysFW.setLoadParams({ id: id });
      $SC.holidaysFW.show();
    }
  });
}

Scada.Modules.push($SC);
