#!/usr/bin/haserl
Content-Type: text/javascript

Scada.Trends = {};
var $T = Scada.Trends;

$T.countData = {
  minutely: [ [ 60, $tr('trends_hour_1') ], [ 1440, $tr('trends_day_1') ], [ 4320, $tr('trends_day_3', 3) ], [ 14400, $tr('trends_day_3', 10) ] ],
  hourly: [ [ 720, $tr('trends_day_3', 30) ], [ 4320, $tr('trends_day_3', 180) ], [ 8760, $tr('trends_year_1', 3) ], [ 17520, $tr('trends_year_2', 2) ], [ 43800, $tr('trends_year_3', 5) ] ],
  daily: [ [ 30, $tr('trends_day_3', 30) ], [ 180, $tr('trends_day_3', 180) ], [ 365, $tr('trends_year_1', 3) ], [ 730, $tr('trends_year_2', 2) ], [ 1825, $tr('trends_year_3', 5) ] ],
  monthly: [ [ 12, $tr('trends_year_1') ], [ 60, $tr('trends_year_3', 5) ], [ 120, $tr('trends_year_3', 10) ] ]
};

$T.store = new Ext.data.JsonStore({
  url: Scada.reqUrl('trends'),
  root: 'data',
  fields: [
    'id',
    'name',
    'type',
    'precision',
    'object',
    'object_name',
    'count_minutely',
    'count_hourly',
    'count_daily',
    'count_monthly',
    { name: 'created', type: 'date', dateFormat: 'timestamp' }
  ]
});

$T.storeReload = function() {
  $T.store.reload();
}

$T.objRenderer = function(nr, cell, item) {
  var data = item.data, suffix = '';
  if (data.object_name) {
    suffix = ' (' + data.object_name + ')';
  }

  return Scada.decodeGroupAddress(data.object) + suffix;
}

$T.sizeRenderer = function(nr, cell, item) {
  var data = item.data, size;

  size = (data.count_minutely + data.count_hourly + data.count_daily + data.count_monthly) * 8;
  size = size / 1024;

  return Math.round(size + 1) + ' KB';
}

$T.typeRenderer = function(nr, cell, item) {
  return item.data.type == 'C' ? $tr('trends_counter') : $tr('trends_absolute');
}

$T.countRenderer = function(nr, cell, item, row, col) {
  var i, key, items, value;

  key = [ 0, 0, 0, 0, 'minutely', 'hourly', 'daily', 'monthly' ][ col ];
  items = $T.countData[ key ];

  if (items) {
    value = item.data[ 'count_' + key ];
    for (i = 0; i < items.length; i++) {
      if (items[ i ][ 0 ] == value) {
        return items[ i ][ 1 ];
      }
    }
  }

  return $tr('unknown');
}

$T.mainFW = new ExtFormWindow({
  id: 'trends-form',
  window: {
    title: $tr('trends_form'),
    width: 460,
    height: 310
  },
  form: {
    url: Scada.reqUrl('trends', 'save'),
    loadUrl: Scada.reqUrl('trends', 'form'),
    items: [
      $def.select({
        fieldLabel: $tr('schedulers_object'), name: 'object', allowBlank: false, hiddenName: 'object', editable: true,
        mode: 'local', valueField: 'id', displayField: 'text', store: $O.extObjStore
      }),
      { name: 'name', fieldLabel: $tr('trends_name'), allowBlank: false },
      $def.select({ id: 'trends-type', name: 'type', hiddenName: 'type', fieldLabel: $tr('trends_type'), allowBlank: false, store: [ [ 'C', $tr('trends_counter') ], [ 'G', $tr('trends_absolute') ] ] }),
      { name: 'precision', fieldLabel: $tr('trends_precision'), allowBlank: false, xtype: 'spinnerfield', value: 2, minValue: 0, maxValue: 8 },
      $def.select({ name: 'count_minutely', hiddenName: 'count_minutely', fieldLabel: $tr('trends_count_minutely'), allowBlank: false, store: $T.countData.minutely }),
      $def.select({ name: 'count_hourly', hiddenName: 'count_hourly', fieldLabel: $tr('trends_count_hourly'), allowBlank: false, store: $T.countData.hourly }),
      $def.select({ name: 'count_daily', hiddenName: 'count_daily', fieldLabel: $tr('trends_count_daily'), allowBlank: false, store: $T.countData.daily }),
      $def.select({ name: 'count_monthly', hiddenName: 'count_monthly', fieldLabel: $tr('trends_count_monthly'), allowBlank: false, store: $T.countData.monthly }),
      { name: 'id', xtype: 'hidden' }
    ]
  },
  events: { onFormSubmitComplete: $T.storeReload, onFormSubmitFailed: $T.storeReload }
});

$T.grid = new Ext.grid.GridPanel($def.grid({
  id: 'trends-grid',
  store: $T.store,
  ctCls: 'scada-list-view-grid',
  loadMask: true,
  border: true,
  bbar: [
    { text: $tr('trends_add'), id: 'trends-add', iconCls: 'icon-add' }
  ],
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('trends_name'), width: 0.11, dataIndex: 'name' },
      { header: $tr('trends_object'), width: 0.15, renderer: $T.objRenderer },
      { header: $tr('trends_type'), width: 0.11, renderer: $T.typeRenderer },
      { header: $tr('trends_precision'), width: 0.08, dataIndex: 'precision' },
      { header: $tr('trends_count_minutely'), width: 0.08, renderer: $T.countRenderer },
      { header: $tr('trends_count_hourly'), width: 0.08, renderer: $T.countRenderer },
      { header: $tr('trends_count_daily'), width: 0.08, renderer: $T.countRenderer },
      { header: $tr('trends_count_monthly'), width: 0.08, renderer: $T.countRenderer },
      { header: $tr('trends_size'), width: 0.08, renderer: $T.sizeRenderer },
      { header: $tr('trends_created'), width: 0.08, dataIndex: 'created', xtype: 'datecolumn', format: 'Y.m.d H:i' },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-delete', dataIndex: 'delete' }
    ]
  })
}));

$T.container = new Ext.Container({
  id: 'Trends',
  title: $tr('trends_title'),
  layout: 'border',
  items: $T.grid
});

$T.onFocus = function() {
  if (!$T.ready) {
    $T.store.load();
    $T.ready = true;
  }
}

$T.onConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    $T.grid.loadMask.show();

    Ext.Ajax.request({
      url: Scada.reqUrl('trends', 'delete'),
      success: $T.storeReload,
      failure: $T.storeReload,
      params: Scada.encodeData({ id: this.id })
    });
  }
}

$T.init = function() {
  Ext.getCmp('trends-add').on('click', function() {
    $T.mainFW.setLoadParams();
    $T.mainFW.show();
    $T.typeSelect.setDisabled(false);
    $T.mainFW.form.findField('precision').setDisabled(false);
  });

  $T.typeSelect = Ext.getCmp('trends-type');

  $T.grid.on('cellclick', function(grid, rowIndex, columnIndex) {
    var id = grid.getStore().getAt(rowIndex).get('id').toNumber();
    var fieldName = grid.getColumnModel().getDataIndex(columnIndex);

    if (fieldName == 'delete') {
      Ext.Msg.confirm(
        $tr('trends_delete_title'),
        $tr('trends_delete_confirm'),
        $T.onConfirm,
        { name: this.name, id: id }
      );
    }
    else {
      $T.mainFW.setLoadParams({ id: id });
      $T.mainFW.show();
      $T.typeSelect.setDisabled(true);
      $T.mainFW.form.findField('precision').setDisabled(true);
    }
  });
}

Scada.Modules.push($T);
