#!/usr/bin/haserl
Content-Type: text/javascript

Scada.Enocean = {};
var $EN = Scada.Enocean;

$EN.profiles = <? lua /lib/genohm-scada/plugins/enocean/web/profiles.lua ?>;
$EN.sorter = function(a, b) {
  if (a[0] > b[0]) {
    return 1;
  }
  else if (a[0] < b[0]) {
    return -1;
  }
  else {
    return 0;
  }
}

$EN.reqUrl = function(request) {
  return '/cgi-bin/scada/general/enocean.cgi?request=' + request;
};

$EN.profiles.sort($EN.sorter);

$EN.eep = {};
$EN.eepList = [];
$EN.eepSingle = {};
$EN.eepObjects = {};
$EN.eepOptions = {};

$EN.sortObjs = function(objs) {
  var res = [];

  Ext.iterate(objs || {}, function(name, dpt) {
    res.push([ name, dpt ]);
  });

  return res.sort($EN.sorter);
};

Ext.each($EN.profiles, function(v) {
  var eep = v[ 0 ], name = v[ 1 ], objs = v[ 2 ], opts = v[ 3 ], single = v[ 4 ];

  $EN.eep[ eep ] = name;
  $EN.eepList.push([ eep, eep + ' ' + name ]);

  $EN.eepOptions[ eep ] = opts;
  $EN.eepObjects[ eep ] = $EN.sortObjs(objs);

  if (single) {
    $EN.eepSingle[ eep ] = true;
  }
});

$EN.translateEep = function(eep) {
  if ($EN.eep[ eep ]) {
    return eep + ' ' + $EN.eep[ eep ];
  }
  else {
    return eep ? eep + ' ' + 'Unsupported' : 'Not set';
  }
};

$EN.MainPanel = new Ext.TabPanel({
  border: false,
  region: 'center',
  margins: '5 0 0 0',
  flex: 1
});

$EN.container = new Ext.Container({
  id: 'Enocean',
  title: 'Enocean',
  layout: 'border',
  items: $EN.MainPanel
});

$EN.autoFillAddress = function(btn) {
  var combo = btn.combo, store = [], value;
  combo.setValue('');

  Ext.iterate(btn.form.getValues(), function(field, value) {
    if (value && field.indexOf('address_') == 0) {
      if (value.toNumber() != value) {
        value = Scada.encodeGroupAddress(value);
      }

      store.push(value);
    }
  });

  value = Scada.findFreeAddress(store);
  if (value) {
    combo.setValue(Scada.decodeGroupAddress(value));
  }
};

$EN.createObjectSelect = function(index, value, dpt, form) {
  var combo, btn, config, store, name, items = [], found;

  dpt = Scada.normalizeDt(dpt).major;
  Ext.each($O.extObjStore.getAll(), function(obj) {
    var data = obj.data, d = Scada.normalizeDt(data.datatype);

    if (d.major == dpt) {
      items.push([ data.id, data.text ]);

      if (data.id == value) {
        found = true;
      }
    }
  });

  store = new Ext.data.ArrayStore({
    fields: [ 'id', 'text' ],
    data: items
  });

  name = 'address_' + index;

  // if (value && !found) {
    // value = Scada.decodeGroupAddress(value);
  // }

  config = $def.select({
    allowBlank: true,
    displayField: 'text',
    editable: true,
    fieldLabel: 'Group address',
    forceSelection: false,
    hiddenName: name,
    invalidText: 'Object with different datatype is already mapped to this address',
    mode: 'local',
    name: name,
    regexp: true,
    store: store,
    value: value,
    valueField: 'id',
    vtype: 'knxgaName',
    width: 215
  });

  combo = new Ext.form.ComboBox(config);
  btn = {
    combo: combo,
    form: form,
    handler: $EN.autoFillAddress,
    iconCls: 'icon-add',
    tooltip: 'Create new object',
    tooltipType: 'title',
    width: 16,
    xtype: 'button'
  };

  return {
    items: [ combo, btn ],
    xtype: 'compositefield',
    width: 245
  };
};

$EN.showMappingForm = function(base, data, mode) {
  var eep = data.eep, objects, mapping, options = {}, formOptions = [], win, fp, form, mh, h;
  if (!eep || !$EN.eepObjects[ eep ] ) {
    Ext.Msg.alert('Error', 'EEP must be set before setting mapping');
    return;
  }

  objects = $EN.eepObjects[ eep ];

  if (mode == 'tx') {
    base.mappingId = data.id;
  }
  else {
    base.mappingAddress = data.address;
  }

  base.mappingObjects = objects;

  fp = base.mappingFormPanel;
  fp.removeAll(true);

  form = fp.form;
  mapping = Ext.util.JSON.decode(data.mapping) || {};

  Ext.each(objects, function(object, index) {
    var name = object[ 0 ], dpt = $tr('objects_datatype_' + object[ 1 ]), m = mapping[ name ] || {};

    fp.add({
      xtype: 'fieldset',
      title: name + ' &ndash; ' + dpt,
      autoHeight: true,
      autoWidth: true,
      padding: 5,
      bodyStyle: 'margin: 0 0 10px !important',
      labelWidth: 180,
      items: [
        $EN.createObjectSelect(index, m.address || '', object[ 1 ], form),
        {
          fieldLabel: (mode == 'tx' ? 'Send telegram' : 'Write to bus'),
          name: ('write_' + index),
          checked: m.write,
          xtype: 'checkbox'
        }
      ]
    });
  });

  if (data.options) {
    options = Ext.util.JSON.decode(data.options) || {};
  }

  if ($EN.eepOptions[ eep ]) {
    Ext.iterate($EN.eepOptions[ eep ], function(name, option) {
      if (!option.show || option.show == mode) {
        var pos = option.position ? (option.position - 1) : formOptions.length;
        if (option.type == 'boolean') {
          formOptions[ pos ] = {
            xtype: 'checkbox',
            name: name,
            fieldLabel: name,
            checked: options[ name ] || option['default']
          };
        }
        else {
          formOptions[ pos ] = {
            xtype: 'spinnerfield',
            name: name,
            fieldLabel: name,
            minValue: option.min,
            maxValue: option.max,
            width: 120,
            value: options[ name ] || option['default']
          };
        }
      }
    });
  }

  if (!$EN.eepSingle[ eep ] && mode == 'tx') {
    formOptions.push({
      xtype: 'spinnerfield',
      name: 'timer',
      fieldLabel: 'Send timer (seconds)',
      minValue: 10,
      maxValue: 1000,
      width: 120,
      value: (options && options['timer']) ? options['timer'] : 100,
    });
  }

  if (formOptions.length) {
    fp.add({
      xtype: 'fieldset',
      title: 'Options',
      autoHeight: true,
      autoWidth: true,
      padding: 5,
      labelWidth: 180,
      items: formOptions
    });
  }

  fp.add(
    { name: 'mapping', xtype: 'hidden' },
    { name: (mode == 'tx' ? 'id' : 'address'), xtype: 'hidden' },
    { name: 'options', xtype: 'hidden' },
    { name: 'eep', xtype: 'hidden', value: eep }
  );

  win = base.mappingFormWindow;
  if (win.rendered) {
    win.setHeight(160);
  }

  win.show();
  fp.doLayout();

  mh = Scada.Viewport.getHeight();
  h = form.el.dom.scrollHeight;

  win.setHeight(Math.min(mh, h + 75)).center();
};

$EN.submitMappingForm = function() {
  var base = this;
  if (!base.mappingObjects || !base.mappingForm.isValid()) {
    return;
  }

  var data = base.mappingForm.getValues(), mapping = {}, options = {}, eep = data.eep, valid = true;
  Ext.each(base.mappingObjects, function(object, index) {
    var field = 'address_' + index, address = data[ field ], write = !!data[ 'write_' + index ],
    name = object[ 0 ], ndt = Scada.normalizeDt(object[ 1 ]);

    if (address) {
      var obj = $O.store.getById(Scada.encodeGroupAddress(address));

      if (obj) {
        var odt = Scada.normalizeDt(obj.data.datatype);

        if (odt.major != ndt.major) {
          base.mappingForm.findField(field).markInvalid();
          valid = false;
        }
      }

      mapping[ name ] = { address: address, write: write };
    }
  });

  if (valid) {
    if ($EN.eepOptions[ eep ]) {
      Ext.iterate($EN.eepOptions[ eep ], function(name) {
        options[ name ] = data[ name ];
      });
    }

    if (data.timer) {
      options.timer = data.timer.toNumber();
    }

    base.mappingForm.setValues({
      mapping: Ext.util.JSON.encode(mapping),
      options: Ext.util.JSON.encode(options),
      address: base.mappingAddress,
      id: base.mappingId
    });

    base.mappingFormPanel.body.dom.scrollTop = 0;

    base.mappingObjects = null;
    base.mappingForm.submit({ mode: 'json', waitMsg: $tr('loading') });
  }
}

$EN.modules = [];

$EN.init = function() {
  Ext.each($EN.modules, function(module) {
    module.init();
  });

  $EN.taskRunner = new Ext.util.TaskRunner();

  $EN.MainPanel.on('tabchange', function(tabPanel, activeTab) {
    var index = tabPanel.items.findIndex('id', activeTab.id), sm = $EN.modules[ index ];

    if ($EN.currentTask) {
      $EN.taskRunner.stop($EN.currentTask);
    }

    $EN.currentTask = sm.reloaderTask;

    if ($EN.currentTask) {
      $EN.taskRunner.start($EN.currentTask);
    }

    if (sm.onFocus) {
      sm.onFocus();
    }
  });

  $EN.MainPanel.setActiveTab(0);
}

Scada.Modules.push($EN);
