#!/usr/bin/haserl
Content-Type: text/javascript

Scada.EnoceanRX = {};
var $ENR = Scada.EnoceanRX;

$ENR.updateList = {};

$ENR.store = new Ext.ux.data.PagingJsonStore({
  baseParams: {},
  url: $EN.reqUrl('rx-main'),
  root: 'data',
  fields: [ 'address', 'name', 'eep', 'mapping', 'updatetime', 'options', 'source' ]
});

$ENR.pagination = new Ext.ux.PagingToolbar({
  pageSize: Config.perpage,
  store: $ENR.store,
  displayInfo: true,
  displayMsg: 'Displaying devices {0} - {1} of {2}',
  emptyMsg: 'No devices to display',
  refreshText: false, firstText: false, lastText: false, nextText: false, prevText: false,
  prependButtons: true,
  items: [
    { text: 'Showing all devices', id: 'enocean-rx-filter', enableToggle: true }, '-',
    { text: 'Clear', id: 'enocean-rx-clear', iconCls: 'icon-clear' }, '-'
  ]
});

$ENR.grid = new Ext.grid.GridPanel($def.grid({
  id: 'EnoceanRX',
  title: 'EnOcean &raquo; KNX',
  store: $ENR.store,
  border: false,
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: 'ID', width: 0.10, dataIndex: 'address' },
      { header: 'Device name', width: 0.20, dataIndex: 'name' },
      { header: 'Profile', width: 0.25, dataIndex: 'eep', renderer: $EN.translateEep },
      { header: 'Interface', width: 0.10, dataIndex: 'source' },
      { header: 'Last telegram', width: 0.10, dataIndex: 'updatetime', xtype: 'timestampcolumn' },
      { header: 'Mapping', width: 0.05, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-object-edit', dataIndex: 'setmapping' },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-delete', dataIndex: 'delete' }
    ]
  }),
  viewConfig: {
    forceFit: true,
    getRowClass: function(record) {
      var ut = record.data.updatetime.toNumber(),
          pt = $ENR.updateList[ record.data.address ] ? $ENR.updateList[ record.data.address ] : ut;
      $ENR.updateList[ record.data.address ] = ut;
      return pt == ut ? 'objects-row-normal' : 'objects-row-updated';
    }
  },
  bbar: $ENR.pagination
}));

$ENR.update = function() {
  $ENR.pagination.doRefresh();
};

$ENR.onConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    Ext.Ajax.request({
      url: $EN.reqUrl('rx-delete'),
      success: $ENR.update,
      failure: $ENR.update,
      params: Scada.encodeData({ address: this.address })
    });
  }
};

$ENR.formWindow = new Ext.Window(
  $def.window({
    title: 'Device',
    width: 600,
    height: 160,
    items: $def.form({
      id: 'enocean-rx-form', url: $EN.reqUrl('rx-save'), buttons: $def.formButtons('enocean-rx-form'),
      items: [
        { fieldLabel: 'Device name', name: 'name', allowBlank: false,	invalidText: 'Name cannot be empty' },
        $def.select({ name: 'eep', fieldLabel: 'Profile', store: $EN.eepList, width: 390, hiddenName: 'eep' }),
        { name: 'address', xtype: 'hidden' }
      ]
    })
  })
);

$ENR.mappingFormWindow = new Ext.Window(
  $def.window({
    title: 'Device mapping',
    width: 500,
    height: 160,
    items: $def.form({
      id: 'enocean-rx-mapping-form',
      autoScroll: true,
      url: $EN.reqUrl('rx-save-mapping'),
      buttons: $def.formButtons('enocean-rx-mapping-form'),
      items: []
    })
  })
);

$ENR.showForm = function(data) {
  $ENR.form.reset();
  $ENR.form.setValues(data);
  $ENR.formWindow.show().center();
};

$ENR.hideForm = function() {
  $ENR.formWindow.hide();
};

$ENR.submitForm = function() {
  if ($ENR.form.isValid()) {
    $ENR.form.submit({ mode: 'json', waitMsg: $tr('loading') });
  }
};

$ENR.hideMappingForm = function() {
  $ENR.mappingFormWindow.hide();
};

$ENR.onClearConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    Ext.Ajax.request({
      url: $EN.reqUrl('rx-clear'),
      success: $ENR.update,
      failure: $ENR.update
    });
  }
}

$ENR.init = function() {
  $ENR.reloaderTask = { run: $ENR.update, interval: 5000 };

  $ENR.formPanel = Ext.getCmp('enocean-rx-form');
  $ENR.form = $ENR.formPanel.getForm();

  Ext.getCmp('enocean-rx-form-submit').setHandler($ENR.submitForm);
  Ext.getCmp('enocean-rx-form-cancel').setHandler($ENR.hideForm);

  $ENR.form.on('actioncomplete', function(form, action) {
    if (action.type == 'submit') {
      $ENR.hideForm();
      $ENR.update();
    }
  });

  $ENR.mappingFormPanel = Ext.getCmp('enocean-rx-mapping-form');
  $ENR.mappingForm = $ENR.mappingFormPanel.getForm();

  Ext.getCmp('enocean-rx-mapping-form-submit').setHandler($EN.submitMappingForm, $ENR);
  Ext.getCmp('enocean-rx-mapping-form-cancel').setHandler($ENR.hideMappingForm);

  $ENR.mappingForm.on('actioncomplete', function(form, action) {
    if (action.type == 'submit') {
      $ENR.hideMappingForm();
      $ENR.update();

      if (Scada.Objects) {
        Scada.Objects.update();
      }
    }
  });

  $ENR.grid.on('cellclick', function(grid, rowIndex, columnIndex) {
    var row = grid.getStore().getAt(rowIndex);
    var address = row.get('address');
    var fieldName = grid.getColumnModel().getDataIndex(columnIndex);

    if (fieldName == 'delete') {
      Ext.Msg.confirm(
        'Delete device',
        'Are you sure you want to delete this device?',
        $ENR.onConfirm,
        { address: address }
      );
    }
    else if (fieldName == 'setmapping') {
      $EN.showMappingForm($ENR, row.data, 'rx');
    }
    else {
      $ENR.showForm(row.data);
    }
  });

  Ext.getCmp('enocean-rx-filter').on('toggle', function() {
    $ENR.store.setBaseParam('filter', this.pressed ? 'name' : false);
    $ENR.update();

    this.setText(this.pressed ? 'Only devices with name' : 'Showing all devices');
  });

  Ext.getCmp('enocean-rx-clear').on('click', function() {
    Ext.Msg.confirm(
      'Clear',
      'Are you sure you want to delete all entries?',
      $ENR.onClearConfirm
    );
  });
};

$EN.modules.push($ENR);
$EN.MainPanel.add($ENR.grid);
