#!/usr/bin/haserl
Content-Type: text/javascript

Scada.EnoceanTX = {};
var $ENT = Scada.EnoceanTX;

$ENT.updateList = {};
$ENT.baseIds = {};

$ENT.store = new Ext.ux.data.PagingJsonStore({
  baseParams: {},
  url: $EN.reqUrl('tx-main'),
  root: 'data',
  fields: [ 'id', 'offset', 'address', 'name', 'eep', 'mapping', 'updatetime', 'options', 'interface' ]
});

$ENT.pagination = new Ext.ux.PagingToolbar({
  pageSize: Config.perpage,
  store: $ENT.store,
  displayInfo: true,
  displayMsg: 'Displaying devices {0} - {1} of {2}',
  emptyMsg: 'No devices to display',
  refreshText: false, firstText: false, lastText: false, nextText: false, prevText: false,
  prependButtons: true,
  items: [
    { text: 'Add new device', id: 'enocean-tx-add', iconCls: 'icon-add' }, '-',
    { text: 'Clear', id: 'enocean-tx-clear', iconCls: 'icon-clear' }, '-'
  ]
});

$ENT.grid = new Ext.grid.GridPanel($def.grid({
  id: 'EnoceanTX',
  title: 'KNX &raquo; EnOcean',
  store: $ENT.store,
  border: false,
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: 'Address', width: 0.10, dataIndex: 'address' },
      { header: 'Device name', width: 0.20, dataIndex: 'name' },
      { header: 'Profile', width: 0.25, dataIndex: 'eep', renderer: $EN.translateEep },
      { header: 'Interface', width: 0.10, dataIndex: 'interface' },
      { header: 'Last telegram', width: 0.10, dataIndex: 'updatetime', xtype: 'timestampcolumn' },
      { header: 'Mapping', width: 0.05, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-object-edit', dataIndex: 'setmapping' },
      { header: 'Teach-in', width: 0.05, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-add', dataIndex: 'teachin' },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-delete', dataIndex: 'delete' }
    ]
  }),
  viewConfig: {
    forceFit: true,
    getRowClass: function(record) {
      var ut = record.data.updatetime.toNumber(),
          pt = $ENT.updateList[ record.data.id ] ? $ENT.updateList[ record.data.id ] : ut;
      $ENT.updateList[ record.data.id ] = ut;
      return pt == ut ? 'objects-row-normal' : 'objects-row-updated';
    }
  },
  bbar: $ENT.pagination
}));

$ENT.update = function() {
  $ENT.pagination.doRefresh();
};

$ENT.onFocus = $ENT.update;

$ENT.onConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    Ext.Ajax.request({
      url: $EN.reqUrl('tx-delete'),
      success: $ENT.update,
      failure: $ENT.update,
      params: Scada.encodeData({ id: this.id })
    });
  }
};

$ENT.formWindow = new Ext.Window(
  $def.window({
    title: 'Device',
    width: 600,
    height: 200,
    items: $def.form({
      id: 'enocean-tx-form', url: $EN.reqUrl('tx-save'), buttons: $def.formButtons('enocean-tx-form'),
      items: [
        $def.select({ name: 'interface', fieldLabel: 'Interface', store: [['undefined', 'undefined']], hiddenName: 'interface' }),
        { fieldLabel: 'Address offset', name: 'offset', xtype: 'spinnerfield', minValue: 0, maxValue: 127, invalidText: 'Address is already in use' },
        { fieldLabel: 'Device name', name: 'name', allowBlank: false,	invalidText: 'Name is already in use' },
        $def.select({ name: 'eep', fieldLabel: 'Profile', store: $EN.eepList, width: 390, hiddenName: 'eep', allowBlank: false }),
        { name: 'id', xtype: 'hidden' },
        { name: 'baseid', xtype: 'hidden' }
      ]
    })
  })
);

$ENT.mappingFormWindow = new Ext.Window(
  $def.window({
    title: 'Device mapping',
    width: 500,
    height: 160,
    items: $def.form({
      id: 'enocean-tx-mapping-form',
      autoScroll: true,
      url: $EN.reqUrl('tx-save-mapping'),
      buttons: $def.formButtons('enocean-tx-mapping-form'),
      items: []
    })
  })
);

$ENT.showForm = function(data) {
  var ep = $ENE.store.getRange(), values = [];

  if (!ep.length) {
    return Ext.Msg.alert('Error', 'No active interfaces found');
  }

  Ext.each(ep, function(obj) {
    values.push([
      obj.data.id,
      obj.data.id + ' (BaseID: ' + obj.data.base + ')'
    ]);

    $ENT.baseIds[ obj.data.id ] = obj.data.base;
  });

  $ENT.form.reset();
  $ENT.form.findField('interface').store.loadData(values);
  $ENT.formWindow.show().center();
  $ENT.form.setValues(data || { id: 0, offset: 0, 'interface': ep[0].id });
};

$ENT.hideForm = function() {
  $ENT.formWindow.hide();
};

$ENT.submitForm = function() {
  var values = $ENT.form.getValues();
  $ENT.form.findField('baseid').setValue($ENT.baseIds[ values['interface'] ]);

  if ($ENT.form.isValid()) {
    $ENT.form.submit({ mode: 'json', waitMsg: $tr('loading') });
  }
};

$ENT.hideMappingForm = function() {
  $ENT.mappingFormWindow.hide();
};

$ENT.onClearConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    Ext.Ajax.request({
      url: $EN.reqUrl('tx-clear'),
      success: $ENT.update,
      failure: $ENT.update
    });
  }
}

$ENT.init = function() {
  $ENT.reloaderTask = { run: $ENT.update, interval: 5000 };

  $ENT.formPanel = Ext.getCmp('enocean-tx-form');
  $ENT.form = $ENT.formPanel.getForm();

  Ext.getCmp('enocean-tx-form-submit').setHandler($ENT.submitForm);
  Ext.getCmp('enocean-tx-form-cancel').setHandler($ENT.hideForm);

  $ENT.form.on('actioncomplete', function(form, action) {
    if (action.type == 'submit') {
      $ENT.hideForm();
      $ENT.update();
    }
  });

  $ENT.mappingFormPanel = Ext.getCmp('enocean-tx-mapping-form');
  $ENT.mappingForm = $ENT.mappingFormPanel.getForm();

  Ext.getCmp('enocean-tx-mapping-form-submit').setHandler($EN.submitMappingForm, $ENT);
  Ext.getCmp('enocean-tx-mapping-form-cancel').setHandler($ENT.hideMappingForm);

  $ENT.mappingForm.on('actioncomplete', function(form, action) {
    if (action.type == 'submit') {
      $ENT.hideMappingForm();
      $ENT.update();

      if (Scada.Objects) {
        Scada.Objects.update();
      }
    }
  });

  $ENT.grid.on('cellclick', function(grid, rowIndex, columnIndex) {
    var row = grid.getStore().getAt(rowIndex);
    var fieldName = grid.getColumnModel().getDataIndex(columnIndex);

    if (fieldName == 'delete') {
      Ext.Msg.confirm(
        'Delete device',
        'Are you sure you want to delete this device?',
        $ENT.onConfirm,
        { id: row.id }
      );
    }
    else if (fieldName == 'setmapping') {
      $EN.showMappingForm($ENT, row.data, 'tx');
    }
    else if (fieldName == 'teachin') {
      Ext.Ajax.request({
        url: $EN.reqUrl('tx-teachin'),
        success: function(r) {
          result = Ext.util.JSON.decode(r.responseText);
          Ext.Msg.alert('Teach-in', result.success ? 'Telegram transmitted successfully' : 'Error transmitting telegram');
        },
        params: Scada.encodeData({ id: row.id })
      });
    }
    else {
      $ENT.showForm(row.data);
    }
  });

  Ext.getCmp('enocean-tx-add').setHandler(function() {
    $ENT.showForm();
  });

  Ext.getCmp('enocean-tx-clear').on('click', function() {
    Ext.Msg.confirm(
      'Clear',
      'Are you sure you want to delete all entries?',
      $ENT.onClearConfirm
    );
  });
};

$EN.modules.push($ENT);
$EN.MainPanel.add($ENT.grid);
