#!/usr/bin/haserl
Content-Type: text/javascript

Scada.Buildings = {};
var $B = Scada.Buildings;

$B.fields = [
  'id', 'floorid', 'name', 'description', 'building', 'number', 'background',
  'parent', 'offset', 'duplicate', 'delete', 'usermode_param', 'touch_param',
  'type'
];

$B.lpStore = new Ext.data.JsonStore({
  url: Scada.reqUrl('buildings'),
  root: 'data',
  baseParams: { filter: 'lp' },
  fields: $B.fields
});

$B.lwStore = new Ext.data.JsonStore({
  url: Scada.reqUrl('buildings'),
  root: 'data',
  baseParams: { filter: 'lw' },
  fields: $B.fields
});

$B.layoutStore = new Ext.data.JsonStore({
  url: Scada.reqUrl('buildings', 'layouts'),
  root: 'data',
  fields: [ 'id', 'name' ]
});

$B.layoutSelect = $def.select({
  fieldLabel: $tr('buildings_layout'),
  name: 'layout',
  mode: 'local',
  allowBlank: true,
  hiddenName: 'layout',
  store: $B.layoutStore,
  valueField: 'id',
  displayField: 'name'
});

$B.paramSelect = function(name) {
  name = name + '_param';

  return $def.select({
    fieldLabel: $tr('buildings_' + name),
    id: name,
    name: name,
    mode: 'local',
    allowBlank: true,
    hiddenName: name,
    store: [
      ['', $tr('buildings_vis_show') ],
      ['D', $tr('buildings_vis_default') ],
      ['H', $tr('buildings_vis_hide') ]
    ],
    valueField: 'id',
    displayField: 'name'
  });
};

$B.renderName = function(name, cell, record) {
  var data = record.data, level = 0;

  if (data.offset) {
    level = 2;
  }
  else if (data.floorid || data.parent) {
    level = 1;
  }

  cell.css += data.floorid ? 'x-grid3-cell-plan ' : 'x-grid3-cell-folder ';
  cell.css += ' scada-level-' + level;

  return name;
};

$B.renderVisibility = function(name, cell, record) {
  var data = record.data, uParam = data.usermode_param, tParam = data.touch_param, result = '';

  if (data.floorid && data.building) {
    if (uParam == 'D') {
      result = '<b>' + $tr('buildings_usermode') + '</b>';
    }
    else if (uParam == '') {
      result = $tr('buildings_usermode');
    }

    if (tParam != 'H') {
      if (result) {
        result += ', ';
      }

      if (tParam == 'D') {
        result += '<b>' + $tr('buildings_touch') + '</b>';
      }
      else {
        result += $tr('buildings_touch');
      }
    }
  }

  return result;
};

$B.viewConfig = {
  forceFit: true,
  getRowClass: function(record) {
    var data = record.data;
    return (data.floorid || data.parent) ? 'objects-row-normal' : 'objects-row-hl';
  }
};

$B.lpGrid = new Ext.grid.GridPanel($def.grid({
  store: $B.lpStore,
  border: false,
  loadMask: true,
  title: $tr('buildings_levels_plans'),
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('buildings_name'), width: 0.20, dataIndex: 'name', renderer: $B.renderName },
      { header: $tr('buildings_visibility'), width: 0.15, dataIndex: 'usermode_param', renderer: $B.renderVisibility },
      { header: $tr('buildings_description'), width: 0.35, dataIndex: 'description' },
      { header: $tr('scripting_duplicate'), width: 0.05, xtype: 'boolclscolumn', falseCls: '', trueCls: 'x-grid3-cell-duplicate', dataIndex: 'duplicate', hidden: true },
      { header: $tr('buildings_move_up'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-up', dataIndex: 'up' },
      { header: $tr('buildings_move_down'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-down', dataIndex: 'down' },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-add', trueCls: '', dataIndex: 'floorid', hidden: true },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: '', trueCls: 'x-grid3-cell-delete', dataIndex: 'delete', hidden: true }
    ]
  }),
  viewConfig: $B.viewConfig,
  bbar: [
    { text: $tr('buildings_add'), iconCls: 'icon-add', hidden: true , handler: function() { $B.showBuildingForm(); } }
  ]
}));

$B.lwGrid = new Ext.grid.GridPanel($def.grid({
  store: $B.lwStore,
  border: false,
  loadMask: true,
  title: $tr('buildings_layouts_widgets'),
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('buildings_name'), width: 0.20, dataIndex: 'name', renderer: $B.renderName },
      { header: $tr('buildings_description'), width: 0.60, dataIndex: 'description' },
      { header: $tr('scripting_duplicate'), width: 0.05, xtype: 'boolclscolumn', falseCls: '', trueCls: 'x-grid3-cell-duplicate', dataIndex: 'duplicate' },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'x-grid3-cell-add', trueCls: '', dataIndex: 'floorid', hidden: true },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: '', trueCls: 'x-grid3-cell-delete', dataIndex: 'delete', hidden: true }
    ]
  }),
  viewConfig: $B.viewConfig,
  bbar: [
    { text: $tr('buildings_add_layout'), iconCls: 'icon-add', hidden: true, handler: function() { $B.showFloorForm({ bid: 'layout', bname: $tr('buildings_layout') }); } },
    '-',
    { text: $tr('buildings_add_widget'), iconCls: 'icon-add', hidden: true, handler: function() { $B.showFloorForm({ bid: 'widget', bname: $tr('buildings_widget') }); } }
  ]
}));

$B.bFormWindow = new Ext.Window(
  $def.window({
    title: $tr('buildings_bform_title'), width: 600, height: 380,
    items: $def.form({
      id: 'buildings-bform',
      url: Scada.reqUrl('buildings', 'building-save'),
      buttons: $def.formButtons('buildings-bform'),
      items: [
        { fieldLabel: $tr('buildings_pname'), name: 'pname', xtype: 'displayfield' },
        { fieldLabel: $tr('buildings_bname'), name: 'name', allowBlank: false },
        { fieldLabel: $tr('buildings_bdescription'), name: 'description', xtype: 'textarea', width: 405, height: 200 },
        { name: 'parent', xtype: 'hidden' },
        { name: 'id', xtype: 'hidden' }
      ]
    })
  })
);

$B.fFormWindow = new Ext.Window(
  $def.window({
    width: 470, height: 350,
    items: $def.form({
      id: 'buildings-fform',
      url: Scada.reqUrl('buildings', 'floor-save'),
      buttons: $def.formButtons('buildings-fform'),
      items: [
        { fieldLabel: $tr('buildings_parent'), name: 'bname', xtype: 'displayfield' },
        { fieldLabel: $tr('buildings_name'), name: 'name', allowBlank: false },
        $B.layoutSelect,
        $B.paramSelect('usermode'),
        $B.paramSelect('touch'),
        { fieldLabel: $tr('buildings_background'), id: 'buildings-bg-select', xtype: 'trigger', name: 'background', editable: false },
        { fieldLabel: $tr('buildings_bgcolor'), xtype: 'colorfield', name: 'bgcolor', width: 80, allowBlank: true },
        { fieldLabel: $tr('buildings_bgrepeat'), name: 'bgrepeat', xtype: 'checkbox' },
        { fieldLabel: $tr('buildings_fadminonly'), name: 'adminonly', xtype: 'checkbox' },
        { name: 'bid', xtype: 'hidden' },
        { name: 'id', xtype: 'hidden' }
      ]
    })
  })
);

$B.MainPanel = new Ext.TabPanel({
  border: false,
  region: 'center',
  margins: '5 0 0 0',
  items: [ $B.lpGrid, $B.lwGrid ]
});

$B.container = new Ext.Container({
  id: 'Buildings',
  title: $tr('buildings_title'),
  layout: 'border',
  items: $B.MainPanel
});

$B.update = function() {
  $B.lpStore.reload();
  $B.lwStore.reload();
  $B.layoutStore.reload();
};

$B.showBuildingForm = function(id, parent) {
  $B.bForm.reset();
  $B.bFormWindow.show().center();

  $B.bForm.findField('pname').setVisible(!!parent);

  if (id) {
    $B.bForm.load({
      url: Scada.reqUrl('buildings', 'building-form'),
      params: Scada.encodeData({ id: id }),
      waitMsg: $tr('loading')
    });
  }
  else if (parent) {
    $B.bForm.findField('parent').setValue(parent.bid);
    $B.bForm.findField('pname').setValue(parent.bname);
  }
};

$B.hideBuildingForm = function() {
  $B.bFormWindow.hide();
};

$B.submitBuildingForm = function() {
  $B.bForm.submit({ mode: 'json', waitMsg: $tr('loading') });
};

$B.showFloorForm = function(data) {
  var layout = $B.fForm.findField('layout'),
    uParam = $B.fForm.findField('usermode_param'),
    tParam = $B.fForm.findField('touch_param'),
    title, bid = data.bid.toNumber();

  if (data.type) {
    title = data.type;
  }
  else {
    title = (data.bid == 'layout') ? 'layout' : (bid ? 'plan' : 'widget');
  }

  $B.fFormWindow.setTitle($tr('buildings_' + title));

  $B.fForm.reset();
  $B.fFormWindow.show().center();

  layout.setVisible(bid);
  uParam.setVisible(bid);
  tParam.setVisible(bid);

  if (bid) {
    layout.setValue(0);
    uParam.setValue('');
    tParam.setValue('');
  }

  if (data.id) {
    $B.fForm.load({
      url: Scada.reqUrl('buildings', 'floor-form'),
      params: Scada.encodeData({ id: data.id }),
      waitMsg: $tr('loading')
    });
  }
  else {
    data.bgcolor = '#FFFFFF';
    $B.fForm.setValues(data);
  }

  $B.fForm.findField('adminonly').setVisible(bid);
};

$B.hideFloorForm = function() {
  $B.fFormWindow.hide();
};

$B.submitFloorForm = function() {
  $B.fForm.submit({ mode: 'json', waitMsg: $tr('loading') });
};

$B.onDeleteConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    Ext.Ajax.request({
      url: Scada.reqUrl('buildings', 'delete'),
      success: $B.update,
      failure: $B.update,
      params: Scada.encodeData(this)
    });
  }
};

$B.lvl2Window = new Ext.Window($def.window({
  title: $tr('buildings_lvl2_title'),
  border: false,
  layout: 'vbox',
  layoutConfig: {
    pack: 'center',
    padding: 5
  },
  width: 222,
  height: 100,
  items: [
    {
      xtype: 'button',
      text: $tr('buildings_add_lvl2'),
      margins: '0 0 10 0',
      width: 200,
      handler: function() {
        $B.showBuildingForm(0, $B.lvl2Window.params);
        $B.lvl2Window.hide();
      }
    },
    {
      xtype: 'button',
      text: $tr('buildings_add_lvl3'),
      width: 200,
      handler: function() {
        $B.showFloorForm($B.lvl2Window.params);
        $B.lvl2Window.hide();
      }
    }
  ]
}));

$B.gridCellClick = function(grid, rowIndex, columnIndex) {
  var row, fieldName, mode, id, editable;

  row = grid.getStore().getAt(rowIndex);
  fieldName = grid.getColumnModel().getDataIndex(columnIndex);

  mode = row.get('floorid') ? 'floor' : 'building';
  editable = row.get('delete');
  id = row.get(mode == 'building' ? 'id' : 'floorid');

  if (fieldName == 'floorid' || !editable) {
    if (mode == 'building') {
      var parent = row.get('parent'), params = { bid: id, bname: row.get('name') };

      if (parent || !editable) {
        $B.showFloorForm(params);
      }
      else {
        $B.lvl2Window.params = params;
        $B.lvl2Window.show();
      }
    }
  }
  else if (fieldName == 'duplicate') {
    grid.loadMask.show();

    Ext.Ajax.request({
      url: Scada.reqUrl('buildings', 'duplicate'),
      success: $B.update,
      failure: $B.update,
      params: Scada.encodeData({ id: id, mode: mode })
    });
  }
  else if (fieldName == 'delete') {
    Ext.Msg.confirm(
      $tr('buildings_delete_title_' + mode),
      $tr('buildings_delete_confirm_' + mode),
      $B.onDeleteConfirm,
      { id: id, mode: mode }
    );
  }
  else if (fieldName == 'up' || fieldName == 'down') {
    Ext.Ajax.request({
      url: Scada.reqUrl('buildings', 'updown'),
      success: $B.update,
      failure: $B.update,
      params: Scada.encodeData({ id: id, up: fieldName == 'up', mode: mode })
    });
  }
  else if (mode == 'building') {
    $B.showBuildingForm(id);
  }
  else {
    $B.showFloorForm({ id: id, bid: row.get('id'), type: row.get('type') });
  }
};

$B.init = function() {
  $B.lpGrid.on('cellclick', $B.gridCellClick);
  $B.lwGrid.on('cellclick', $B.gridCellClick);

  $B.bForm = Ext.getCmp('buildings-bform').getForm();
  $B.bForm.on('actioncomplete', function(form, action) {
    if (action.type == 'submit') {
      $B.hideBuildingForm();
      $B.update();
    }
  });

  Ext.getCmp('buildings-bform-submit').setHandler($B.submitBuildingForm);
  Ext.getCmp('buildings-bform-cancel').setHandler($B.hideBuildingForm);

  $B.fForm = Ext.getCmp('buildings-fform').getForm();

  $B.fForm.on('actioncomplete', function(form, action) {
    if (action.type == 'submit') {
      $B.hideFloorForm();
      $B.update();
    }
  });

  Scada.initImgSelect($B);
  Scada.bindImgSelect($B, 'buildings-bg-select');

  Ext.getCmp('buildings-fform-submit').setHandler($B.submitFloorForm);
  Ext.getCmp('buildings-fform-cancel').setHandler($B.hideFloorForm);

  $B.MainPanel.setActiveTab(0);
};

$B.onFocus = $B.update;
Scada.Modules.push($B);
