#!/usr/bin/haserl
Content-Type: text/javascript

Ext.chart.Chart.CHART_URL = '/scada/ext/resources/charts.swf';

Scada.Vis = {};
var $V = Scada.Vis;

$V.center = Config.viscenter.toNumber() > 0;
$V.editorTypes = [];

$V.iconStore = new Ext.data.JsonStore({
  fields: ['icon'],
  root: 'images',
  url: Scada.reqUrl('vis', 'graphics')
});

$V.imgStore = new Ext.data.JsonStore({
  fields: ['image'],
  root: 'images',
  url: Scada.reqUrl('vis', 'graphics'),
  baseParams: { mode: 'img' }
});

$V.visIconStore = new Ext.data.ArrayStore({
  fields: ['icon', 'name']
});

$V.graphIconStore = new Ext.data.ArrayStore({
  fields: ['icon', 'name']
});

$V.iconSelects = [];

$V.createIconSelect = function(mode, type, override) {
  var suffix = type ? ('_' + type) : '', id, config;

  id = 'scada-vis-icon-select-' + mode + suffix.replace(/_/g, '-');
  config = $extend({
    id: id,
    name: 'icon' + suffix,
    xtype: 'trigger',
    width: 160,
    hidden: true,
    editable: false,
    allowBlank: false,
    fieldLabel: $tr('vis_icon' + suffix)
  }, override || {}, true);

  $V.iconSelects.push(id);
  return config;
};

$V.iconSelectUpdateCount = function(count) {
  var text = count ? (' (' + count + ')') : '';
  Ext.getCmp('scada-vis-icons-additional').setText($tr('vis_icons_additional') + text);
};

$V.iconSelectSubmit = function(fw) {
  var values = fw.form.clearInvalid().getValues(), result = [], store = {}, valid = true;

  Ext.iterate(values, function(key, value) {
    var chunks = key.split('_'), name = chunks[0], id = chunks[1];

    if (!store[ id ]) {
      store[ id ] = {};
    }

    store[ id ][ name ] = name == 'icon' ? value : value.toNumber();
  });

  Ext.iterate(store, function(id, values) {
    if (isNaN(values.min) || isNaN(values.max) || values.max < values.min) {
      valid = false;
      fw.form.findField('max_' + id).markInvalid($tr('vis_icon_max_error'));
    }
    else {
      result.push(values);
    }
  });

  if (valid) {
    Ext.getCmp('scada-vis-icons-additional-store').setValue(Ext.util.JSON.encode(result));
    $V.iconSelectUpdateCount(result.length);
    fw.hide();
  }
};

$V.iconSelectRestore = function() {
  var value = Ext.getCmp('scada-vis-icons-additional-store').getValue(), items;
  try {
    items = Ext.util.JSON.decode(value);
    if (typeof items == 'object') {
      Ext.each(items, $V.addAdditionalIcon);
    }
  }
  catch (e) {}
};

$V.iconSelectWindow = new ExtFormWindow({
  id: 'scada-vis-icon-select-window',
  window: {
    title: $tr('vis_icons_additional'),
    width: 600,
    height: 400,
  },
  form: {
    autoScroll: true,
    items: []
  },
  events: { onSubmit: $V.iconSelectSubmit }
});

$V.textStylesSubmit = function() {
  var values = $V.textStylesWindow.form.getValues();
  $extend(values, { bold: '', underline: '', italic: '', backdrop: '' });
  $V.editorForms.object.setValues(values);
  $V.textStylesWindow.hide();
};

$V.textStylesWindow = new ExtFormWindow({
  id: 'scada-vis-text-style-window',
  window: {
    title: $tr('vis_text_styles'),
    width: 360,
    height: 225,
  },
  form: {
    labelWidth: 150,
    defaults: {
      width: 150
    },
    items: [
      { xtype: 'spinnerfield', fieldLabel: $tr('vis_label_font_size'), name: 'size', allowBlank: false, value: 12, minValue: 10, maxValue: 100, allowDecimals: false },
      { xtype: 'checkboxgroup', fieldLabel: $tr('vis_text_styles'), columns: 3, itemCls: 'x-check-group-alt', items: [
        { boxLabel: '<b>B</b>', name: 'bold', value: 1 },
        { boxLabel: '<i>I</i>', name: 'italic', value: 1 },
        { boxLabel: '<u>U</u>', name: 'underline', value: 1 },
      ]},
      { xtype: 'combo', triggerAction: 'all', fieldLabel: $tr('vis_font'), name: 'font', store: [ 'Arial', 'Tahoma', 'Times New Roman', 'Verdana' ] },
      { xtype: 'colorfield', fieldLabel: $tr('vis_color'), name: 'color', value: '#000000' },
      { xtype: 'checkbox', fieldLabel: $tr('vis_backdrop'), name: 'backdrop', value: 1 }
    ]
  },
  events: { onSubmit: $V.textStylesSubmit }
});

$V.showTextStyles = function() {
  var values, result = {};

  $V.textStylesWindow.show();
  $V.textStylesWindow.form.reset();

  values = $V.editorForms.object.getValues();
  Ext.iterate(values, function(key, value) {
    if (value) {
      result[ key ] = value;
    }
  });
  $V.textStylesWindow.form.setValues(result);
};

$V.removeAdditionalIconRow = function(cmp) {
  if (cmp.items) {
    cmp.items.each(function(item) {
      $V.iconSelectWindow.form.remove(item, true);
    });
  }

  $V.iconSelectWindow.form.remove(cmp, true);
  $V.iconSelectWindow.formPanel.remove(cmp, true);
};

$V.deleteAdditionalIcon = function() {
  var id = this.id.replace('_delete', ''), cmp = Ext.getCmp(id);
  $V.removeAdditionalIconRow(cmp);
};

$V.additionalIconId = 1;

$V.addAdditionalIcon = function(values) {
  var id = $V.additionalIconId, items = $V.iconSelectWindow.formPanel.items, index = items.length - 1, bounds, dec, dt, min, max, sel;

  bounds = Scada.dtGetBounds($V.iconSelectWindowObj.datatype);

  if (values && !values.id) {
    values = values;
  }
  else {
    values = {};
    if (index > 0) {
      try {
        items = items.items[ index - 1 ].items.items;
        min = items[ 1 ].getValue().toNumber();
        max = items[ 3 ].getValue().toNumber();

        if (min !== null && max !== null) {
          values.min = (max).constrain(bounds[0], bounds[1]);
          values.max = (max + (max - min)).constrain(bounds[0], bounds[1]);
        }
      }
      catch (e) {}
    }
  }

  dt = $V.iconSelectWindowObj.datatype.major;
  dec = dt == Scada.dt.float16 || dt == Scada.dt.float32;

  sel = $V.createIconSelect('additional', id, { hidden: false, value: values.icon });

  $V.iconSelectWindow.formPanel.insert(index, {
    id: 'vis_icons_additional_row_' + id,
    xtype: 'compositefield',
    hideLabel: true,
    width: 540,
    items: [
      { xtype: 'displayfield', value: $tr('vis_min_value') },
      { xtype: 'spinnerfield', minValue: bounds[0], maxValue: bounds[1], allowBlank: false, allowDecimals: dec, name: 'min_' + id, width: 80, value: values.min },
      { xtype: 'displayfield', value: $tr('vis_max_value') },
      { xtype: 'spinnerfield', minValue: bounds[0], maxValue: bounds[1], allowBlank: false, allowDecimals: dec, name: 'max_' + id, width: 80, value: values.max },
      { xtype: 'displayfield', value: $tr('vis_icon') },
      sel,
      { xtype: 'button', iconCls: 'icon-delete', width: 16, id: 'vis_icons_additional_row_' + id + '_delete', handler: $V.deleteAdditionalIcon }
    ]
  });

  Scada.bindImgSelect($V, sel.id, 'icon');
  $V.iconSelectWindow.formPanel.doLayout();

  $V.additionalIconId++;
};

$V.showIconSelect = function() {
  var o, so, obj;

  o = $V.comboBoxes.object.getValue();
  so = $V.comboBoxes.statusobject.getValue();

  obj = $V.objectStore.getObject(so || o);
  if (!obj || !Scada.dtIsNumeric(obj.datatype.major)) {
    return;
  }

  $V.iconSelectWindowObj = obj;

  $V.iconSelectWindow.formPanel.items.each($V.removeAdditionalIconRow);

  $V.iconSelectWindow.formPanel.add({
    id: 'scada-vis-icon-add',
    xtype: 'button',
    text: $tr('vis_icon_add'),
    handler: $V.addAdditionalIcon,
    iconCls: 'icon-add',
    width: 'auto'
  });

  $V.iconSelectRestore();

  $V.iconSelectWindow.show();
  $V.iconSelectWindow.formPanel.doLayout();
};

$V.setFloorLinkForm = function() {
  var form = $V.editorForms.floorlink, show;
  show = form.findField('displaymode').getValue() == 'value';

  Ext.getCmp('scada-vis-icon-select-floorlink').setVisible(!show).setDisabled(show);
  Ext.getCmp('scada-vis-icon-select-floorlink-active').setVisible(!show).setDisabled(show);

  Ext.each(form.items.items.slice(-4), function(item) {
    item.setVisible(show).setDisabled(!show);
  });
};

$V.setObjectForm = function() {
  var o, so, obj, mode, form, bool, numeric, show, count, icons, widget;

  form = $V.editorForms.object;

  o = $V.comboBoxes.object.getValue();
  so = $V.comboBoxes.statusobject.getValue();
  mode = form.findField('displaymode').getValue();
  obj = $V.objectStore.getObject(so || o);
  widget = ($V.comboBoxes.objectwidget.getValue() || 0).toNumber();

  bool = obj ? obj.boolean : false;
  numeric = obj ? Scada.dtIsNumeric(obj.datatype.major) : false;

  show = obj && (mode == 'icon' || mode == 'icon-value');
  Ext.getCmp('scada-vis-icon-select-object-default').setVisible(show && !bool).setDisabled(!show || bool);
  Ext.getCmp('scada-vis-icon-select-object-on').setVisible(show && bool).setDisabled(!show || !bool);
  Ext.getCmp('scada-vis-icon-select-object-off').setVisible(show && bool).setDisabled(!show || !bool);
  Ext.getCmp('scada-vis-icons-additional').setVisible(show && numeric);
  Ext.getCmp('scada-vis-object-text-styles').setVisible(mode != 'icon');

  form.findField('readonly').setVisible(!widget);
  form.findField('fixedvalue').setVisible(!widget);
  form.findField('update').setVisible(!widget);
  form.findField('showcontrol').setVisible(!widget);

  try {
    icons = Ext.getCmp('scada-vis-icons-additional-store').getValue();
    icons = Ext.util.JSON.decode(icons);
    count = icons ? icons.length : 0;
    $V.iconSelectUpdateCount(count);
  }
  catch(e) {}
};

$V.setImageForm = function() {
  var source = Ext.getCmp('scada-vis-image-source').getValue();
  Ext.getCmp('vis-src-local').setVisible(source == 'local').setDisabled(source != 'local');
  Ext.getCmp('vis-src-remote').setVisible(source == 'remote').setDisabled(source != 'remote');
};

$V.setIframeForm = function() {
  var source = Ext.getCmp('scada-vis-iframe-source').getValue();
  Ext.getCmp('vis-iframe-url').setVisible(source == 'url').setDisabled(source != 'url');
};

$V.showVisParams = function(btn) {
  var id = $V.comboBoxes[ btn.mode ].getValue(), item;
  if (id) {
    item = $O.store.getById(id);

    if (item) {
      $O.showVisParams(item.data);
    }
  }
};

$V.createEditorPanel = function(id, items, buttons) {
  $V.editorTypes.push(id);

  return {
    id: 'scada-vis-editor-' + id + '-panel',
    title: $tr('vis_widget_' + id.replace('-', '_')),
    border: false,
    xtype: 'form',
    layout: 'form',
    autoScroll: true,
    labelWidth: 120,
    padding: 10,
    defaults: { width: 160 },
    items: items,
    buttons: buttons
  };
};

$V.floorEditorConfig = [
  $V.createEditorPanel('object',
    [
      { xtype: 'compositefield', items: [
        $def.select({ fieldLabel: $tr('vis_mainobject'), id: 'scada-vis-editor-object', name: 'object', allowBlank: false, hiddenName: 'object', editable: true,
          mode: 'local', valueField: 'id', displayField: 'text', store: $O.extObjStore, width: 133, listWidth: 180
        }),
        { xtype: 'button', iconCls: 'icon-visparams', width: 16, handler: $V.showVisParams, tooltip: $tr('objects_vis_params'), tooltipType: 'title', mode: 'object' }
      ]},
      $def.select({ fieldLabel: $tr('vis_statusobject'), id: 'scada-vis-editor-statusobject', name: 'statusobject', allowBlank: true, hiddenName: 'statusobject', editable: true,
        mode: 'local', valueField: 'id', displayField: 'text', store: new Ext.data.ArrayStore({ fields: [ 'id', 'text' ], data: [] }), listWidth: 180
      }),
      { xtype: 'textfield', fieldLabel: $tr('vis_name'), name: 'name', allowBlank: true },
      { xtype: 'checkbox', fieldLabel: $tr('vis_readonly'), name: 'readonly', value: 1 },
      { xtype: 'checkbox', fieldLabel: $tr('vis_notouch'), name: 'notouch', value: 1 },
      { xtype: 'hidden', name: 'sortorder', value: 0 },
      { xtype: 'hidden', name: 'width' }, { xtype: 'hidden', name: 'height' },
      { xtype: 'checkbox', fieldLabel: $tr('vis_nobg'), name: 'nobg', checked: true, value: 1 },
      { xtype: 'textfield', fieldLabel: $tr('vis_fixedvalue'), name: 'fixedvalue', allowBlank: true },
      { xtype: 'checkbox', fieldLabel: $tr('vis_update'), name: 'update', value: 1, boxLabel: $tr('vis_update_hint') },
      { xtype: 'textfield', fieldLabel: $tr('vis_pincode'), name: 'pincode', minLength: 3, maxLength: 8, allowBlank: true, maskRe: /[0-9]/ },
      $def.select({ fieldLabel: $tr('vis_object_widget'), id: 'scada-vis-editor-object-widget', name: 'floorid', allowBlank: false, hiddenName: 'widget',
        mode: 'local', valueField: 'id', displayField: 'text', store: new Ext.data.ArrayStore({ fields: [ 'id', 'text' ], data: [] }), value: '', listWidth: 180
      }),
      $def.select({ name: 'displaymode', hiddenName: 'displaymode', fieldLabel: $tr('vis_displaymode'), value: 'icon-value', id: 'scada-vis-object-display-mode',
        store: [['icon-value',  $tr('vis_displaymode_icon_value')], ['icon',  $tr('vis_displaymode_icon')], ['value',  $tr('vis_displaymode_value')]]
      }),
      $V.createIconSelect('object', 'default'),
      $V.createIconSelect('object', 'on'),
      $V.createIconSelect('object', 'off'),
      { xtype: 'checkbox', fieldLabel: $tr('vis_showcontrol'), boxLabel: $tr('vis_showcontrol_hint'), name: 'showcontrol', value: 1 },
      { xtype: 'button', text: $tr('vis_icons_additional'), id: 'scada-vis-icons-additional', handler: $V.showIconSelect, hidden: true },
      { xtype: 'button', text: $tr('vis_text_styles'), id: 'scada-vis-object-text-styles', handler: $V.showTextStyles, hidden: true },
      { xtype: 'hidden', id: 'scada-vis-icons-additional-store', name: 'icons_add' },
      { xtype: 'hidden', name: 'size' }, { xtype: 'hidden', name: 'bold' }, { xtype: 'hidden', name: 'italic' }, { xtype: 'hidden', name: 'backdrop' },
      { xtype: 'hidden', name: 'underline' }, { xtype: 'hidden', name: 'font' }, { xtype: 'hidden', name: 'color' }
    ],
    [
      { text: $tr('delete'), id: 'scada-vis-editor-object-delete', hidden: true },
      { text: $tr('vis_button_dupe'), id: 'scada-vis-editor-object-dupe', hidden: true },
      { text: $tr('vis_button_add'), id: 'scada-vis-editor-object-add' },
      { text: $tr('vis_button_reset'), id: 'scada-vis-editor-object-reset' }
    ]
  ),
  $V.createEditorPanel('floorlink',
    [
      $def.select({ fieldLabel: $tr('vis_floor'), id: 'scada-vis-editor-floor', name: 'floorid', allowBlank: false, hiddenName: 'floorid',
        mode: 'local', valueField: 'id', displayField: 'text', store: new Ext.data.ArrayStore({ fields: [ 'id', 'text' ], data: [] }), listWidth: 180
      }),
      { xtype: 'textfield', fieldLabel: $tr('vis_name'), name: 'name', allowBlank: true },
      { xtype: 'checkbox', fieldLabel: $tr('vis_notouch'), name: 'notouch', value: 1 },
      { xtype: 'hidden', name: 'sortorder', value: 0 },
      { xtype: 'checkbox', fieldLabel: $tr('vis_nobg'), name: 'nobg', checked: true, value: 1 },
      { xtype: 'hidden', name: 'width' }, { xtype: 'hidden', name: 'height' },
      $def.select({ name: 'displaymode', hiddenName: 'displaymode', fieldLabel: $tr('vis_displaymode'), value: 'icon', id: 'scada-vis-link-display-mode',
        store: [['icon', $tr('vis_displaymode_icon')], ['value', $tr('vis_displaymode_value')]]
      }),
      $V.createIconSelect('floorlink', '', { hidden: false }),
      $V.createIconSelect('floorlink', 'active', { hidden: false, allowBlank: true }),
      { xtype: 'spinnerfield', fieldLabel: $tr('vis_label_font_size'), name: 'size', allowBlank: false, value: 14, minValue: 10, maxValue: 100, allowDecimals: false },
      { xtype: 'checkboxgroup', fieldLabel: $tr('vis_text_styles'), columns: 3, itemCls: 'x-check-group-alt', items: [
        { boxLabel: '<b>B</b>', name: 'bold', value: 1 },
        { boxLabel: '<i>I</i>', name: 'italic', value: 1 },
        { boxLabel: '<u>U</u>', name: 'underline', value: 1 },
      ]},
      { xtype: 'combo', triggerAction: 'all', fieldLabel: $tr('vis_font'), name: 'font', store: [ 'Arial', 'Tahoma', 'Times New Roman', 'Verdana' ] },
      { xtype: 'colorfield', fieldLabel: $tr('vis_color'), name: 'color', value: '#000000' }
    ],
    [
      { text: $tr('delete'), id: 'scada-vis-editor-floorlink-delete', hidden: true },
      { text: $tr('vis_button_dupe'), id: 'scada-vis-editor-floorlink-dupe', hidden: true },
      { text: $tr('vis_button_add'), id: 'scada-vis-editor-floorlink-add' },
      { text: $tr('vis_button_reset'), id: 'scada-vis-editor-floorlink-reset' }
    ]
  ),
  $V.createEditorPanel('camera',
    [
      { xtype: 'textfield', fieldLabel: $tr('vis_src'), name: 'src', allowBlank: false },
      { xtype: 'spinnerfield', fieldLabel: $tr('vis_width'), name: 'win_width', allowBlank: false, minValue: 10, maxValue: 1024, value: 640, allowDecimals: false },
      { xtype: 'spinnerfield', fieldLabel: $tr('vis_height'), name: 'win_height', allowBlank: false, minValue: 10, maxValue: 1024, value: 480, allowDecimals: false },
      { xtype: 'textfield', fieldLabel: $tr('vis_name'), name: 'name', allowBlank: true },
      $V.createIconSelect('camera', '', { hidden: false, value: 'camera.png' }),
      { xtype: 'checkbox', fieldLabel: $tr('vis_autoopen'), name: 'autoopen', value: 1 },
      { xtype: 'checkbox', fieldLabel: $tr('vis_nobg'), name: 'nobg', checked: true, value: 1 },
      { xtype: 'hidden', name: 'width' }, { xtype: 'hidden', name: 'height' },
      { xtype: 'hidden', name: 'sortorder', value: 0 }
    ],
    [
      { text: $tr('delete'), id: 'scada-vis-editor-camera-delete', hidden: true },
      { text: $tr('vis_button_dupe'), id: 'scada-vis-editor-camera-dupe', hidden: true },
      { text: $tr('vis_button_add'), id: 'scada-vis-editor-camera-add' },
      { text: $tr('vis_button_reset'), id: 'scada-vis-editor-camera-reset' }
    ]
  ),
  $V.createEditorPanel('graph',
    [
      $def.select({ fieldLabel: $tr('vis_dataobject'), id: 'scada-vis-editor-graph-object', name: 'object', allowBlank: false, hiddenName: 'object', editable: true,
        mode: 'local', valueField: 'id', displayField: 'text', store: $O.extNumObjStore, listWidth: 180
      }),
      { xtype: 'textfield', fieldLabel: $tr('vis_name'), name: 'name', allowBlank: true },
      $V.createIconSelect('graph', '', { hidden: false }),
      { xtype: 'spinnerfield', fieldLabel: $tr('vis_width'), name: 'win_width', allowBlank: false, minValue: 10, maxValue: 1024, value: 640, allowDecimals: false },
      { xtype: 'spinnerfield', fieldLabel: $tr('vis_height'), name: 'win_height', allowBlank: false, minValue: 10, maxValue: 1024, value: 480, allowDecimals: false },
      { xtype: 'spinnerfield', fieldLabel: $tr('vis_limit'), name: 'limit', allowBlank: false, minValue: 5, maxValue: 200, value: 10, allowDecimals: false },
      { xtype: 'checkbox', fieldLabel: $tr('vis_autoopen'), name: 'autoopen', value: 1 },
      { xtype: 'checkbox', fieldLabel: $tr('vis_nobg'), name: 'nobg', checked: true, value: 1 },
      { xtype: 'hidden', name: 'width' }, { xtype: 'hidden', name: 'height' }
    ],
    [
      { text: $tr('delete'), id: 'scada-vis-editor-graph-delete', hidden: true },
      { text: $tr('vis_button_dupe'), id: 'scada-vis-editor-graph-dupe', hidden: true },
      { text: $tr('vis_button_add'), id: 'scada-vis-editor-graph-add' },
      { text: $tr('vis_button_reset'), id: 'scada-vis-editor-graph-reset' }
    ]
  ),
  $V.createEditorPanel('label',
    [
      { xtype: 'textfield', fieldLabel: $tr('vis_label_text'), name: 'name', allowBlank: false },
      { xtype: 'spinnerfield', fieldLabel: $tr('vis_label_font_size'), name: 'size', allowBlank: false, value: 14, minValue: 10, maxValue: 100, allowDecimals: false },
      { xtype: 'checkboxgroup', fieldLabel: $tr('vis_text_styles'), columns: 3, itemCls: 'x-check-group-alt', items: [
        { boxLabel: '<b>B</b>', name: 'bold', value: 1 },
        { boxLabel: '<i>I</i>', name: 'italic', value: 1 },
        { boxLabel: '<u>U</u>', name: 'underline', value: 1 },
      ]},
      { xtype: 'combo', triggerAction: 'all', fieldLabel: $tr('vis_font'), name: 'font', store: [ 'Arial', 'Tahoma', 'Times New Roman', 'Verdana' ] },
      { xtype: 'colorfield', fieldLabel: $tr('vis_color'), name: 'color', value: '#000000' }
    ],
    [
      { text: $tr('delete'), id: 'scada-vis-editor-label-delete', hidden: true },
      { text: $tr('vis_button_dupe'), id: 'scada-vis-editor-label-dupe', hidden: true },
      { text: $tr('vis_button_add'), id: 'scada-vis-editor-label-add' },
      { text: $tr('vis_button_reset'), id: 'scada-vis-editor-label-reset' }
    ]
  ),
  $V.createEditorPanel('image',
    [
      $def.select({ name: 'source', hiddenName: 'source', fieldLabel: $tr('vis_img_source'), value: 'local', id: 'scada-vis-image-source',
        store: [['local',  $tr('vis_img_source_local')], ['remote',  $tr('vis_img_source_remote')]]
      }),
      { xtype: 'textfield', fieldLabel: $tr('vis_src_remote'), name: 'src_remote', allowBlank: false, hidden: true, id: 'vis-src-remote' },
      { xtype: 'trigger', fieldLabel: $tr('vis_select_image'), name: 'src_local', editable: false, hidden: true, id: 'vis-src-local' },
      { xtype: 'spinnerfield', fieldLabel: $tr('vis_width'), name: 'width', allowBlank: true, minValue: 10, maxValue: 2048, allowDecimals: false },
      { xtype: 'spinnerfield', fieldLabel: $tr('vis_height'), name: 'height', allowBlank: true, minValue: 10, maxValue: 2048, allowDecimals: false },
      { xtype: 'textfield', fieldLabel: $tr('vis_link'), name: 'link', allowBlank: true }
    ],
    [
      { text: $tr('delete'), id: 'scada-vis-editor-image-delete' },
      { text: $tr('vis_button_add'), id: 'scada-vis-editor-image-add' },
      { text: $tr('vis_button_dupe'), id: 'scada-vis-editor-image-dupe', hidden: true },
      { text: $tr('vis_button_reset'), id: 'scada-vis-editor-image-reset' }
    ]
  ),
  $V.createEditorPanel('gauge',
    [
      { xtype: 'compositefield', items: [
        $def.select({ fieldLabel: $tr('vis_dataobject'), id: 'scada-vis-editor-gauge-object', name: 'object', allowBlank: false, hiddenName: 'object', editable: true,
          mode: 'local', valueField: 'id', displayField: 'text', store: $O.extNumObjStore, width: 133, listWidth: 180
        }),
        { xtype: 'button', iconCls: 'icon-visparams', width: 16, handler: $V.showVisParams, tooltip: $tr('objects_vis_params'), tooltipType: 'title', mode: 'gauge' }
      ]},
      { xtype: 'spinnerfield', fieldLabel: $tr('vis_size'), name: 'size', allowBlank: true, minValue: 100, maxValue: 500, value: 200, allowDecimals: false },
      { xtype: 'textfield', fieldLabel: $tr('vis_name'), name: 'name', allowBlank: true },
      { xtype: 'checkbox', fieldLabel: $tr('vis_readonly'), name: 'readonly', value: 1 }
    ],
    [
      { text: $tr('delete'), id: 'scada-vis-editor-gauge-delete' },
      { text: $tr('vis_button_dupe'), id: 'scada-vis-editor-gauge-dupe', hidden: true },
      { text: $tr('vis_button_add'), id: 'scada-vis-editor-gauge-add' },
      { text: $tr('vis_button_reset'), id: 'scada-vis-editor-gauge-reset' }
    ]
  ),
  $V.createEditorPanel('iframe',
    [
      $def.select({ name: 'source', hiddenName: 'source', fieldLabel: $tr('vis_iframe_source'), value: 'url', id: 'scada-vis-iframe-source',
        store: [['url',  $tr('vis_iframe_source_url')], ['schedulers',  $tr('vis_iframe_source_schedulers')], ['trends',  $tr('vis_iframe_source_trends')]]
      }),
      { xtype: 'textfield', fieldLabel: $tr('vis_iframe_source_url'), name: 'url', allowBlank: false, id: 'vis-iframe-url' },
      { xtype: 'spinnerfield', fieldLabel: $tr('vis_width'), name: 'width', allowBlank: false, value: 480, minValue: 10, maxValue: 2048, allowDecimals: false },
      { xtype: 'spinnerfield', fieldLabel: $tr('vis_height'), name: 'height', allowBlank: false, value: 320, minValue: 10, maxValue: 2048, allowDecimals: false },
      { xtype: 'textfield', fieldLabel: $tr('vis_name'), name: 'name', allowBlank: true },
      { xtype: 'checkbox', fieldLabel: $tr('vis_notouch'), name: 'notouch', value: 1 },
      { xtype: 'hidden', name: 'sortorder', value: 0 },
    ],
    [
      { text: $tr('delete'), id: 'scada-vis-editor-iframe-delete' },
      { text: $tr('vis_button_dupe'), id: 'scada-vis-editor-iframe-dupe', hidden: true },
      { text: $tr('vis_button_add'), id: 'scada-vis-editor-iframe-add' },
      { text: $tr('vis_button_reset'), id: 'scada-vis-editor-iframe-reset' }
    ]
  )
];

$V.container = new Ext.Container({
  id: 'Vis', layout: 'border', title: $tr('vis_title'), hideMode: 'offsets',
  items: [
    { xtype: 'panel', margins: '10 10 10 10', cmargins: '10 10 10 10', title: $tr('vis_buildings'), region: 'west', collapsible: true, layout: 'vbox',
      width: 250, id: 'scada-vis-sidebar', layoutConfig: { align: 'stretch' },
      items: [
        $def.tree({ id: 'scada-vis-tree', dataUrl: Scada.reqUrl('vis', 'buildings'), border: false, flex: 1, margins: 0 }),
        { xtype: 'form', id: 'scada-vis-plan-size', border: false, hidden: true, padding: 10, labelWidth: 120, items:
          [
            { id: 'scada-vis-plan-width', fieldLabel: $tr('vis_plan_width'), xtype: 'spinnerfield', minValue: 0, maxValue: 2000, width: 90, enableKeyEvents: true, value: 1024, allowDecimals: false },
            { id: 'scada-vis-plan-height', fieldLabel: $tr('vis_plan_height'), xtype: 'spinnerfield', minValue: 0, maxValue: 2000, width: 90, enableKeyEvents: true, value: 768, allowDecimals: false }
          ]
        },
        { xtype: 'form', id: 'scada-vis-widget-loc', border: false, hidden: true, padding: 10, labelWidth: 120, items:
          [
            { id: 'scada-vis-loc-x', fieldLabel: $tr('vis_loc_x'), xtype: 'spinnerfield', minValue: 0, maxValue: 2000, width: 90, enableKeyEvents: true, allowBlank: false, allowDecimals: false },
            { id: 'scada-vis-loc-y', fieldLabel: $tr('vis_loc_y'), xtype: 'spinnerfield', minValue: 0, maxValue: 2000, width: 90, enableKeyEvents: true, allowBlank: false, allowDecimals: false }
          ]
        },
        { xtype: 'button', text: $tr('vis_touch_order_edit'), id: 'scada-vis-touch-edit', height: 30, margins: '10 10 10 10', width: 290, hidden: true }
      ]
    },
    { xtype: 'panel', margins: '10 10 10 0', id: 'scada-vis-wrap', region: 'center', autoScroll: true },
    { xtype: 'panel', id: 'scada-vis-editor', region: 'east', title: $tr('vis_editor'),
      margins: '10 10 10 0', cmargins: '10 10 10 0', collapsible: true, animCollapse: false, width: 340, collapsed: false,
      layout: 'border',
      items: [
        { xtype: 'panel', region: 'center', id: 'scada-vis-editor-panel', border: false, layout: 'accordion', items: $V.floorEditorConfig },
        { xtype: 'button', text: $tr('vis_unlock'), id: 'scada-vis-editor-control', region: 'south', height: 30, margins: '10 10 10 10', width: 290, disabled: true }
      ]
    }
  ]
});

$V.toBoolean = function(v) {
  var n = (v && v.toNumber) ? v.toNumber() : 0;
  return v === true || (!isNaN(n) && n);
};

$V.objectStore = {
  objects: {},
  update: function(data) {
    for (var i = data.length - 1; i >= 0; i--) {
      var item = data[ i ];
      this.register(item.data);
    }
  },
  empty: function() {
    this.objects = {};
  },
  register: function(data) {
    if (!data.id || data.datadec === undefined) {
      return;
    }

    var id = data.id.toNumber();
    var o = this.objects[ id ];
    var params = (Ext.util.JSON.decode(data.visparams || '') || {});

    if (o) {
      if (o.name != data.name) {
        o.name = data.name;
        this.notify(null, o, 'name');
      }

      if (!$compare(o.value, data.datadec)) {
        o.value = data.datadec;
        this.notify(null, o, 'value');
      }

      if (o.rawdatatype != data.datatype) {
        o.rawdatatype = data.datatype;
        o.datatype = Scada.normalizeDt(data.datatype);
      }

      o.enums = data.enums;
      o.params = params;
    }
    else {
      var dt = Scada.normalizeDt(data.datatype);
      this.objects[ id ] = {
        rawaddress: data.address.toNumber(),
        address: Scada.decodeGroupAddress(data.address),
        name: data.name,
        units: data.units,
        datatype: dt,
        rawdatatype: data.datatype,
        boolean: (dt.major == 1),
        value: data.datadec,
        params: params,
        enums: data.enums,
        listeners: {}
      };
    }
  },
  notify: function(listenerId, o, type) {
    var listener;

    if (listenerId) {
      listener = o.listeners[ listenerId ];
      this.callListener(listener, o, type);
    }
    else {
      for (listenerId in o.listeners) {
        if (o.listeners.hasOwnProperty(listenerId)) {
          listener = o.listeners[ listenerId ];
          this.callListener(listener, o, type);
        }
      }
    }
  },
  callListener: function(listener, o, type) {
    if (listener && typeof listener.fn == 'function') {
      listener.fn.call(listener.bind, o, type);
    }
  },
  addListener: function(listenerId, objectId, v, bind) {
    var o = this.objects[ objectId ];
    if (o) {
      if (typeof v == 'function') {
        o.listeners[ listenerId ] = { fn: v, bind: bind };
        this.notify(listenerId, o, 'init');
      }
      else {
        delete o.listeners[ listenerId ];
      }
    }
  },
  removeListener: function(listenerId, objectId) {
    this.addListener(listenerId, objectId, null);
  },
  getObject: function(id) {
    return this.objects[ id.toNumber() ];
  }
};

$V.widgetPrio = [
  8, // object
  8, // object
  7, // planlink
  0, // skip
  6, // camera
  5, // graph
  3, // label
  2, // image
  4, // gauge
  1, // iframe
];

$V.floorController = {
  widgetId: 1,
  widgets: {},
  mapping: {
    0: 'object',
    1: 'object',
    2: 'floorlink',
    3: 'html',
    4: 'camera',
    5: 'graph',
    6: 'label',
    7: 'image',
    8: 'gauge',
    9: 'iframe'
  },
  panels: {
    'object': 0,
    'floorlink': 1,
    'camera': 2,
    'graph': 3,
    'label': 4,
    'image': 5,
    'gauge': 6,
    'iframe': 7
  },
  load: function(data) {
    this.setBackground($V.floor, $V.layoutCfg);

    $V.floorObjects = data.main;

    this.registerItems(data.main, $V.Content);
    this.registerItems(data.layout, $V.Layout);

    $V.ContentGrid.hide();
    $V.PlanSizes.hide();

    $V.TouchOrderEdit.setVisible($V.floor.type != 'layout');
    $V.ContentBg.removeClass('scada-vis-panel-edit');
    $V.Sidebar.doLayout();
    $V.Content.doLayout();
  },
  registerItems: function(data, container) {
    var i;
    for (i = data.length - 1; i >= 0; i--) {
      this.register(data[ i ], container);
    }
  },
  unlock: function() {
    var widget;
    for (var w in this.widgets) {
      if (this.widgets.hasOwnProperty(w)) {
        widget = this.widgets[ w ];

        if (widget.container == $V.Content) {
          widget.unlock();
        }
        else {
          widget.disable();
        }
      }
    }

    $V.Tree.disable();
    $V.ContentGrid.show();
    $V.PlanSizes.show();
    $V.TouchOrderEdit.hide();
    $V.Sidebar.doLayout();

    $V.ContentBg.addClass('scada-vis-panel-edit');
    $V.Layout.addClass('scada-vis-fade');

    if ($V.planSize && $V.planSize.width && $V.planSize.height) {
      $V.PlanWidthInput.setValue($V.planSize.width);
      $V.PlanHeightInput.setValue($V.planSize.height);
    }
    else {
      $V.PlanWidthInput.setValue(1024);
      $V.PlanHeightInput.setValue(768);
    }

    $V.setPlanGuides();
  },
  lock: function() {
    this.stopEdit();

    for (var w in this.widgets) {
      if (this.widgets.hasOwnProperty(w)) {
        this.widgets[ w ].lock();
      }
    }

    $V.Tree.enable();
  },
  register: function(config, container, unlock) {
    var t = this.mapping[ config.type.toNumber() ];

    if (t) {
      if (this.edit) {
        if (!this.dupe && t == this.edit.type) {
          var e = this.edit.elem, c = e.config;

          config.locx = c.locx;
          config.locy = c.locy;

          e.markDeleted(true);
        }

        this.stopEdit();
      }

      var v = new $V[ t + 'Widget' ](config, container);
      if (v.config) {
        this.widgets[ this.widgetId ] = v;
        this.widgetId++;
      }

      if (unlock) {
        v.unlock();
      }

      return v;
    }
  },
  clean: function() {
    for (var w in this.widgets) {
      if (this.widgets.hasOwnProperty(w)) {
        if (typeof this.widgets[ w ].dispose == 'function') {
          this.widgets[ w ].dispose();
        }
        delete this.widgets[ w ];
      }
    }
  },
  setBackground: function(contentCfg, layoutCfg) {
    this.doSetBackground($V.ContentBg.el, contentCfg);
    this.doSetBackground($V.LayoutBg.el, layoutCfg || {});
  },
  doSetBackground: function(el, config) {
    var bg = config.background ? ('url("/scada/resources/img/' + config.background + '")') : '';
    config.bgcolor = config.bgcolor || '#FFFFFF';

    el.setStyle('background-image', bg);
    el.setStyle('background-color', config.bgcolor == '#FFFFFF' ? 'transparent' : config.bgcolor);
    el.setStyle('background-repeat', (config.bgrepeat ? 'repeat' : 'no-repeat'));
  },
  serialize: function() {
    var data = { 'update': [], 'delete': [], 'insert': [] }, w, d, key, widget;

    for (w in this.widgets) {
      if (this.widgets.hasOwnProperty(w)) {
        widget = this.widgets[ w ];

        if (widget && widget.container == $V.Content) {
          d = widget.serialize();

          if (d && !d.disposed) {
            if (d.deleted) {
              key = 'delete';
            }
            else if (d.insert) {
              key = 'insert';
            }
            else {
              key = 'update';
            }

            data[ key ].push(d);
          }
        }
      }
    }

    data.id = $V.floorId;
    data.size = $V.getPlanSize();
    return data;
  },
  startEdit: function(elem) {
    this.stopEdit();

    var cfg = elem.config, ct = cfg.type.toNumber(), t = this.mapping[ ct ];

    if (t) {
      $V.editorSubmits[ t ].setText($tr('vis_button_edit'));
      $V.editorDupes[ t ].show();
      $V.editorDeletes[ t ].show();
      $V.EditorPanel.getLayout().setActiveItem(this.panels[ t ]);

      var values = elem.getFormData();
      $V.editorForms[ t ].setValues(values);

      this.edit = { type: t, elem: elem };
      $V.setObjectForm();
      $V.setFloorLinkForm();
      $V.setImageForm();
      $V.setIframeForm();

      elem.form = $V.editorForms[ t ];
      $V.setWidgetLocForm(elem);

      if (elem.buttonEl) {
        elem.buttonEl.addClass('x-resizable-over').addClass('scada-widget-active');
      }
    }
  },
  stopEdit: function(type) {
    var elem;
    $V.setWidgetLocForm();

    if (!this.edit) {
      return;
    }

    if (type && this.edit.type != type) {
      return;
    }

    elem = this.edit.elem;
    elem.form = false;
    if (elem.buttonEl) {
      elem.buttonEl.removeClass('x-resizable-over').removeClass('scada-widget-active');
    }

    $V.editorSubmits[ this.edit.type ].setText($tr('vis_button_add'));
    $V.editorForms[ this.edit.type ].reset();

    Ext.iterate($V.editorDupes, function(type, btn) {
      btn.hide();
    });

    Ext.iterate($V.editorDeletes, function(type, btn) {
      btn.hide();
    });

    this.edit = null;
    this.dupe = false;

    $V.setObjectForm();
    $V.setFloorLinkForm();
    $V.setImageForm();
    $V.setIframeForm();
  },
  save: function() {
    this.stopEdit();

    var data = this.serialize();
    if (data.size) {
      Ext.Ajax.request({
        url: Scada.reqUrl('buildings', 'floor-plan-save'),
        success: $V.onSaveFloor,
        failure: $V.onSaveFloor,
        params: Scada.encodeData(data)
      });
    }
    else {
      Ext.Msg.alert(
        $tr('vis_plan_size_error_title'),
        $tr('vis_plan_size_error_text')
      );
    }
  }
};

$V.widget = {
  createButton: function(noIcon, clean) {
    var cls = 'scada-widget ' + (noIcon ? 'scada-widget-no-icon' : 'scada-widget-icon'), el, tr, prio;
    if ($V.toBoolean(this.config.nobg)) {
      cls = cls + ' x-scada-widget-nobg';
    }

    this.buttonEl = new Ext.Button({
      border: false,
      iconAlign: 'top',
      cls: cls + ($V.toBoolean(this.config.readonly) ? ' scada-widget-icon-ro' : ''),
      x: this.config.locx,
      y: this.config.locy
    });

    if (this.clickHandler) {
      this.buttonEl.on('click', this.clickHandler, this);
    }

    this.buttonEl.on('mouseover', this.mouseover, this);
    this.buttonEl.on('mouseout', this.mouseout, this);
    this.buttonEl.on('afterrender', function(btn) {
      btn.el.removeClass('x-btn-noicon');
    });

    this.container.add(this.buttonEl);
    this.container.doLayout();

    prio = this.config.displaymode == 'value' ? 1 : 0;
    this.buttonEl.el.setStyle('z-index', $V.widgetPrio[ this.config.type ] * 10 + prio);

    if (clean) {
      el = this.buttonEl.el;
      tr = el.query('tr');
      Ext.get(tr[ 2 ]).remove();
      Ext.get(tr[ 0 ]).remove();
      Ext.get(el.query('.x-btn-ml')).remove();
      Ext.get(el.query('.x-btn-mr')).remove();
    }
  },
  mouseover: function() {
    if ($V.deleteButton.dragging) {
      return;
    }

    if (this.unlocked) {
      var pos = this.buttonEl.getPosition(true);
      var width = this.buttonEl.getWidth();

      $V.deleteButton.widget = this;
      $V.deleteButton.setPosition(pos[ 0 ] + width - 17, pos[ 1 ] + 1);

      $V.deleteButton.overIcon = true;
    }
  },
  mouseout: function() {
    if (this.unlocked) {
      $V.deleteButton.overIcon = false;
      $V.hideDeleteButton.defer(200);
    }
  },
  setIcon: function(icon, override, width, height) {
    if (this.buttonEl && icon) {
      var el;
      icon = override ? icon : ('/scada/resources/icons/' + icon);

      if (this.svg) {
        this.buttonEl.btnEl.update('<object type="image/svg+xml"></object>');
        el = this.buttonEl.btnEl.child('object');
        el.dom.setAttribute('data', icon);
      }
      else {
        this.buttonEl.btnEl.update('<img>');
        el = this.buttonEl.btnEl.child('img');
        el.dom.setAttribute('src', icon);
      }

      if (height && height.toNumber()) {
        el.setStyle('height', height + 'px');
      }

      if (width && width.toNumber()) {
        el.setStyle('width', width + 'px');
      }
    }
  },
  addListener: function(object) {
    $V.objectStore.addListener(this.config.id, (object || this.config.object), this.listener, this);
  },
  removeListener: function() {
    $V.objectStore.removeListener(this.config.id, this.config.object);
  },
  destroy: function() {
    var that = this;

    Ext.each([ 'buttonEl', 'windowEl', 'graphEl' ], function(el) {
      if (that[ el ]) {
        that[ el ].destroy();
        delete that[ el ];
      }
    });
  },
  dispose: function() {
    this.destroy();
  },
  unlock: function() {
    if (this.onLock) {
      this.onLock();
    }

    if (this.buttonEl) {
      if (!this.dd) {
        var self = this;

        this.dd = new Ext.dd.DD(this.buttonEl.id, 'scada-vis');
        this.dd.startDrag = function() {
          this.constrainTo($V.Content.el);

          $V.activeWidget = self;
          $V.hideDeleteButtonForced();
          $V.deleteButton.dragging = true;

          $V.setWidgetLocForm(self);
        };
        this.dd.endDrag = function() {
          self.dragged = true;
          self.updateLocation();

          $V.deleteButton.dragging = false;
          $V.showDeleteButton();

          $V.setWidgetLocForm(self);
        };
      }
      else {
        this.dd.unlock();
      }
    }

    this.unlocked = true;
  },
  disable: function() {
    if (this.clickHandler) {
      this.buttonEl.removeAllListeners();
    }
  },
  lock: function() {
    if (this.dd) {
      this.dd.lock();
    }

    this.unlocked = false;
  },
  markDeleted: function(hide) {
    this.config.deleted = true;

    if (this.config.insert) {
      this.destroy();
      this.config.disposed = true;
    }
    else if (this.buttonEl) {
      if (hide) {
        this.buttonEl.hide();
      }
      else {
        this.buttonEl.disable();
      }
    }
  },
  disable: function() {
    if (this.buttonEl) {
      this.buttonEl.disable();
    }
  },
  serialize: function() {
    return this.config;
  },
  startEdit: function() {
    $V.floorController.startEdit(this);
  },
  updateLocation: function() {
    if (this.buttonEl) {
      var pos = this.buttonEl.el.getStyles('top', 'left');
      this.config.locx = pos.left.toNumber();
      this.config.locy = pos.top.toNumber();
    }
  },
  getFormData: function() {
    return this.config;
  },
  applyTextStyles: function(text, params, cls) {
    var btn = this.buttonEl, btnEl = this.valueLabel || btn.btnEl;

    if (text) {
      btn.setText(text);
    }

    if (cls) {
      btn.addClass(cls);
    }

    btnEl.setStyle('font-size', (params.size || 12).toString() + 'px');
    btnEl.setStyle('color', (params.color || '#000'));

    if (params.bold) {
      btnEl.setStyle('font-weight', 'bold');
    }
    if (params.italic) {
      btnEl.setStyle('font-style', 'italic');
    }
    if (params.underline) {
      btnEl.setStyle('text-decoration', 'underline');
    }
    if (params.font) {
      btnEl.setStyle('font-family', params.font);
    }
    if (params.backdrop) {
      btn.addClass('scada-widget-backdrop');
      btnEl.addClass('value');
    }
  }
};

$V.showSetValueForm = function(o, c, b, e) {
  if (o.datatype && o.datatype.major) {
    o.datadec = o.value;

    $O.showSetValueForm({
      row: o,
      datatype: o.rawdatatype,
      decodeddatatype: $tr('objects_datatype_' + o.rawdatatype),
      address: o.address,
      decodedaddress: o.address,
      name: o.name
    });
  }
  else {
    Ext.Msg.alert(
      $tr('objects_setvalue_error'),
      $tr('objects_setvalue_error_text')
    );
  }
};

$V.imgResize = function(dom, rs, params) {
  rs.resizeTo(params.width || dom.naturalWidth || dom.width, params.height || dom.naturalHeight || dom.height);
};

$V.onResize = function(me, params, w, h, img) {
  if (img) {
    img.setWidth(w);
    img.setHeight(h);
  }

  me.config.width = w;
  me.config.height = h;

  params.width = w;
  params.height = h;

  if (me.form) {
    me.form.setValues({ width: w, height: h });
  }

  me.config.params = Ext.util.JSON.encode(params);
};

$V.objectWidget = function(config, container) {
  var o = $V.objectStore.getObject(config.object), s, displaymode;

  if (config.statusobject) {
    s = $V.objectStore.getObject(config.statusobject);

    if (config.statusobject != config.object) {
      this.mainobject = o;
    }
  }
  else {
    config.statusobject = config.object;
    s = o;
  }

  if (!o || !s) {
    return false;
  }

  $extend(this, $V.widget);

  this.container = container;
  this.config = config;
  this.config.boolean = s.boolean;
  this.config.mainObjBoolean = o.boolean;

  this.clickHandler = function(button, e) {
    if (this.unlocked) {
      if (this.dragged) {
        this.dragged = false;
      }
      else {
        this.startEdit();
      }
    }
    else if (this.config.widget) {
      Ext.Msg.alert(
        $tr('vis_object_widget'),
        $tr('vis_object_widget_error')
      );
    }
    else if (!$V.toBoolean(this.config.readonly)) {
      var o = $V.objectStore.getObject(this.config.object);
      var s = $V.objectStore.getObject(this.config.statusobject);

      if (!s || !o) {
        return;
      }

      if (this.config.fixedvalue) {
        Ext.Ajax.request({
          url: Scada.reqUrl('objects', 'setvalue'),
          params: Scada.encodeData({
            address: o.address,
            datatype: o.rawdatatype,
            value: this.config.fixedvalue,
            type: 'text'
          })
        });
      }
      else if (this.config.mainObjBoolean) {
        Ext.Ajax.request({
          url: Scada.reqUrl('objects', 'setvalue'),
          params: Scada.encodeData({
            address: o.address,
            datatype: 1,
            value: (s.value ? 'false' : 'true')
          })
        });
      }
      else {
        o.statobj = s;
        $V.showSetValueForm(o, this.config, button, e);
      }
    }
  };

  this.onLock = function() {
    if (this.config.displaymode == 'value') {
      return;
    }

    var btn = this.buttonEl.btnEl, img = btn.child('img'), rs, me = this;

    rs = new Ext.Resizable(btn, { wrap: true });
    rs.on('resize', function(el, w, h) {
      $V.onResize(me, params, w, h, img);
    });

    if (img) {
      img.on('load', function() {
        $V.imgResize(img.dom, rs, params);
      });
    }
  };

  var params, hideIcon;
  try {
    params = Ext.util.JSON.decode(this.config.params);
  }
  catch (e) {}

  if (params) {
    $extend(this.config, params);
  }

  displaymode = this.config.displaymode;
  hideIcon = displaymode == 'value';

  this.createButton(hideIcon);
  this.buttonEl.baseIcon = hideIcon ? null : this.config.icon_default;

  if (displaymode == 'icon-value') {
    this.valueLabel = this.buttonEl.btnEl.insertSibling({ tag: 'div' }, 'after');
  }
  else if (displaymode == 'value') {
    this.valueLabel = this.buttonEl.btnEl;
  }

  if (displaymode != 'value' && params.width && params.height) {
    this.buttonEl.btnEl.setSize(params.width, params.height);
  }

  if (displaymode != 'icon') {
    this.applyTextStyles(null, params);
  }

  this.updateIcon = function(value, dt) {
    var icon, i, j, add;

    if (dt == Scada.dt.bool) {
      icon = value ? this.config.icon_on : this.config.icon_off;
    }
    else {
      icon = this.config.icon_default;

      if (this.config.icons_add) {
        for (i = 0, j = this.config.icons_add.length; i < j; i++) {
          add = this.config.icons_add[ i ];

          if (add.min <= value && value <= add.max) {
            icon = add.icon;
            break;
          }
        }
      }
    }

    this.setIcon(icon);
  };

  this.getFormData = function() {
    var c = $extend({}, this.config);

    if (c.object == c.statusobject) {
      c.statusobject = 0;
    }

    if (c.icons_add) {
      c.icons_add = Ext.util.JSON.encode(c.icons_add);
    }

    if (!c.widget) {
      c.widget = '';
    }

    return c;
  };

  this.listener = function(object, type) {
    if (type == 'init' || type == 'value') {
      if (this.config.displaymode != 'value') {
        this.updateIcon(object.value, object.datatype.major);
      }

      if (this.valueLabel) {
        var v = Scada.decodeValue(object.value, object.rawdatatype, object.units, object.enums);
        this.valueLabel.update(v.toString());
      }
    }
  };

  this.addListener(config.statusobject);

  this.dispose = function() {
    this.removeListener();
    this.destroy();
  };

  return this;
};

$V.floorlinkWidget = function(config, container) {
  var floorText, params;

  $extend(this, $V.widget);
  this.container = container;
  this.config = config;

  if (config.object == 'trends') {
    this.url = '/cgi-bin/scada-vis/trends.cgi';
    floorText = $tr('trends_title');
  }
  else if (config.object == 'schedulers') {
    this.url = '/cgi-bin/scada-vis/schedulers.cgi';
    floorText = $tr('schedulers_title');
  }
  else if ($V.floors[ config.object ]) {
    this.floorObject = $V.floors[ config.object ];
    floorText = this.floorObject.text;
  }
  else {
    return false;
  }

  this.clickHandler = function() {
    if (this.unlocked) {
      if (this.dragged) {
        this.dragged = false;
      }
      else {
        this.startEdit();
      }
    }
    else if (this.url) {
      window.open(this.url);
    }
    else {
      $V.showFloor(this.floorObject);
    }
  };

  try {
    params = Ext.util.JSON.decode(this.config.params);
  }
  catch (e) {}

  if (!params) {
    params = { icon: this.config.params };
  }

  $extend(this.config, params);

  this.createButton();

  this.onLock = function() {
    if (!this.config.icon) {
      return;
    }

    var btn = this.buttonEl.btnEl, img = btn.child('img'), rs, me = this;

    rs = new Ext.Resizable(btn, { wrap: true });
    rs.on('resize', function(el, w, h) {
      $V.onResize(me, params, w, h, img);
    });

    if (img) {
      img.on('load', function() {
        $V.imgResize(img.dom, rs, params);
      });
    }
  };

  var icon = params.icon, title = this.config.name || $tr('vis_goto', floorText);

  if (icon) {
    this.config.icon = icon;

    if (params.width && params.height) {
      this.buttonEl.btnEl.setSize(params.width, params.height);
    }

    if (params.icon_active && this.floorObject && this.floorObject.floorid == $V.floorId) {
      icon = params.icon_active;
    }
    this.setIcon(icon);
  }
  else {
    this.applyTextStyles(title, params, 'x-scada-text-label');
  }

  this.getFormData = function() {
    var c = $clone(this.config);
    c.floorid = c.object;

    if (!c.displaymode) {
      c.displaymode = c.icon ? 'icon' : 'value';
    }

    return c;
  };

  return this;
};

$V.getModalConfig = function(config, params) {
  config.autoopen = !!params.autoopen;
  config.icon = params.icon;
  Ext.each(['width', 'height', 'win_width', 'win_height'], function(key) {
    var value = params[ key ];
    config[ key ] = value ? value.toNumber() : 32;
  });
};

$V.cameraWidget = function(config, container) {
  var params = Ext.util.JSON.decode(config.params);

  if (!params) {
    return;
  }

  $extend(this, $V.widget);

  this.container = container;
  this.config = config;

  config.src = params.src;
  $V.getModalConfig(config, params);

  this.setImage = function() {
    var show = this.windowEl.isVisible(), src = show ? config.src : '/scada/ext/images/s.gif';
    Ext.DomHelper.overwrite(this.windowEl.body, {
      tag: 'img',
      width: config.win_width,
      height: config.win_height,
      src: src
    });
  };

  this.clickHandler = function() {
    if (this.unlocked) {
      if (this.dragged) {
        this.dragged = false;
      }
      else {
        this.startEdit();
      }
    }
    else {
      if (this.windowEl) {
        this.windowEl.setVisible(!this.windowEl.isVisible());
      }
      else {
        this.windowEl = new Ext.Window(
          $def.window({
            modal: false,
            draggable: true,
            title: $tr('vis_camera'),
            x: this.config.locx.toNumber() + (config.width || 32) + 6,
            y: this.config.locy,
            width: config.win_width + 12,
            height: config.win_height + 30,
            bodyBorder: false,
            border: false
          })
        );

        this.container.add(this.windowEl);
        this.container.doLayout();

        this.windowEl.on('show', this.setImage, this, true);
        this.windowEl.on('hide', this.setImage, this, false);

        this.windowEl.show();
      }
    }
  };

  this.createButton();
  this.setIcon(this.config.icon || 'camera.png');
  if (config.width && config.height) {
    this.buttonEl.btnEl.setSize(config.width, config.height);
  }

  this.onLock = function() {
    if (this.windowEl) {
      this.windowEl.hide();
    }

    var btn = this.buttonEl.btnEl, img = btn.child('img'), rs, me = this;

    rs = new Ext.Resizable(btn, { wrap: true });
    rs.on('resize', function(el, w, h) {
      $V.onResize(me, params, w, h, img);
    });

    if (img) {
      img.on('load', function() {
        $V.imgResize(img.dom, rs, params);
      });
    }
  };

  if (this.config.autoopen) {
    this.clickHandler();
  }

  return this;
};

$V.graphWidget = function(config, container) {
  var params = Ext.util.JSON.decode(config.params);

  if (!params) {
    return;
  }

  $extend(this, $V.widget);

  this.container = container;
  this.config = config;

  $V.getModalConfig(config, params);
  config.limit = params.limit ? params.limit.toNumber() : 10;

  this.object = $V.objectStore.getObject(config.object);
  if (!this.object || !Scada.dtIsNumeric(this.object.datatype.major)) {
    return;
  }

  this.clickHandler = function() {
    if (this.unlocked) {
      if (this.dragged) {
        this.dragged = false;
      }
      else {
        this.startEdit();
      }
    }
    else {
      if (this.windowEl) {
        this.windowEl.setVisible(!this.windowEl.isVisible());
      }
      else {
        this.store = new Ext.data.JsonStore({
          autoDestroy: true,
          autoLoad: true,
          url: Scada.reqUrl('vis', 'graph'),
          fields: [ 'logtime', 'datadec' ],
          root: 'data',
          sortInfo: { field: 'logtime', direction: 'ASC' },
          baseParams: Scada.encodeData({ address: this.object.rawaddress, limit: this.config.limit })
        });

        this.windowEl = new Ext.Window(
          $def.window({
            modal: false,
            draggable: true,
            title: this.object.name,
            x: this.config.locx.toNumber() + (config.width || 32) + 6,
            y: this.config.locy,
            width: this.config.win_width + 12,
            height: this.config.win_height + 30,
            bodyBorder: false,
            border: false
          })
        );

        this.container.add(this.windowEl);
        this.container.doLayout();

        this.windowEl.show();

        this.graphEl = new Ext.chart.LineChart({
          renderTo: this.windowEl.body,
          width: this.config.win_width,
          height: this.config.win_height,
          store: this.store,
          xField: 'logtime',
          yField: 'datadec',
          tipRenderer: function(chart, record) {
            return record.data.datadec;
          },
          xAxis: new Ext.chart.CategoryAxis({
            labelRenderer: function(timestamp) {
              var dt = Date.parseDate(timestamp, 'U');
              return dt ? dt.format('H:i:s') : '';
            }
          })
        });
      }
    }
  };

  this.createButton();
  this.setIcon(this.config.icon);
  if (config.width && config.height) {
    this.buttonEl.btnEl.setSize(config.width, config.height);
  }

  this.listener = function(object, type) {
    if (type == 'value' && this.store) {
      this.store.reload();
    }

    if (type == 'init' && this.config.autoopen) {
      this.clickHandler();
    }
  };

  this.onLock = function() {
    if (this.windowEl) {
      this.windowEl.hide();
    }

    var btn = this.buttonEl.btnEl, img = btn.child('img'), rs, me = this;

    rs = new Ext.Resizable(btn, { wrap: true });
    rs.on('resize', function(el, w, h) {
      $V.onResize(me, params, w, h, img);
    });

    if (img) {
      img.on('load', function() {
        $V.imgResize(img.dom, rs, params);
      });
    }
  };

  this.addListener(this.config.object);

  return this;
};

$V.labelWidget = function(config, container) {
  var params = Ext.util.JSON.decode(config.params);
  $extend(this, $V.widget);

  this.container = container;
  this.config = config;
  $extend(this.config, params);

  this.clickHandler = function() {
    if (this.unlocked) {
      if (this.dragged) {
        this.dragged = false;
      }
      else {
        this.startEdit();
      }
    }
  };

  this.createButton();
  this.applyTextStyles(this.config.name, params, 'x-scada-widget-label');

  return this;
};

$V.imageWidget = function(config, container) {
  var source, src, params;

  $extend(this, $V.widget);
  this.container = container;
  this.config = config;

  params = Ext.util.JSON.decode(this.config.params);
  $extend(this.config, params);

  this.clickHandler = function() {
    if (this.unlocked) {
      if (this.dragged) {
        this.dragged = false;
      }
      else {
        this.startEdit();
      }
    }
    else if (this.config.link) {
      window.location = this.config.link;
    }
  };

  this.onLock = function() {
    var btn = this.buttonEl.btnEl, img = btn.child(this.svg ? 'object' : 'img'), rs, me = this;

    rs = new Ext.Resizable(btn, { wrap: true });
    rs.on('resize', function(el, w, h) {
      $V.onResize(me, params, w, h, img);
    });

    if (img) {
      img.on('load', function() {
        $V.imgResize(img.dom, rs, params);
      });
    }
  };

  src = params.src;
  source = params.source == 'local' ? 'local' : 'remote';
  this.config[ 'src_' + source ] = src;

  if (source == 'local') {
    src = '/scada/resources/img/' + src;

    this.svg = !!src.match(/\.svg$/);
  }

  this.config.nobg = 1;
  this.createButton(false, true);
  this.setIcon(src, true, params.width, params.height);

  if (!params.link) {
    this.buttonEl.addClass('scada-widget-image-no-link');
  }

  this.buttonEl.addClass('scada-widget-image');

  return this;
};

$V.gaugeWidget = function(config, container) {
  var params = Ext.util.JSON.decode(config.params), id, dtm, decimals, bounds;
  if (!params || !params.size) {
    return;
  }

  $extend(this, $V.widget);

  this.object = $V.objectStore.getObject(config.object);
  if (!this.object || !Scada.dtIsNumeric(this.object.datatype.major)) {
    return;
  }

  this.container = container;
  this.config = config;
  $extend(this.config, params);

  this.clickHandler = function(button, e) {
    if (this.unlocked) {
      if (this.dragged) {
        this.dragged = false;
      }
      else {
        this.startEdit();
      }
    }
    else if (!$V.toBoolean(this.config.readonly)) {
      var o = $V.objectStore.getObject(this.config.object);
      $V.showSetValueForm(o, button, e);
    }
  };

  this.config.nobg = 1;
  this.createButton();

  id = Ext.id();
  dtm = this.object.datatype.major;
  decimals = (dtm == Scada.dt.float16 || dtm == Scada.dt.float32) ? 2 : 0;
  bounds = Scada.dtGetBounds(this.object.rawdatatype, this.object.params);

  this.buttonEl.btnEl.update('<canvas id="' + id + '"></canvas>');

  this.gauge = new Gauge({
    minValue: bounds[ 0 ],
    maxValue: bounds[ 1 ],
    decimals: decimals,
    width: params.size,
    height: params.size,
    units: (this.config.name || this.object.name || this.object.address),
    renderTo: id
  });

  this.gauge.draw();
  this.gauge.setValue(this.object.value);

  this.listener = function(object, type) {
    if (type == 'value' && this.gauge) {
      this.gauge.setValue(object.value);
    }
  };

  this.addListener(this.config.object);

  return this;
};

$V.iframeWidget = function(config, container) {
  var src, frame, params;

  $extend(this, $V.widget);
  this.container = container;
  this.config = config;

  params = Ext.util.JSON.decode(this.config.params);
  $extend(this.config, params);

  this.clickHandler = function() {
    if (this.unlocked) {
      if (this.dragged) {
        this.dragged = false;
      }
      else {
        this.startEdit();
      }
    }
  };

  this.onLock = function() {
    var btn = this.buttonEl.btnEl, frame, rs, me = this;
    btn.update('<div class="scada-widget-iframe-inner"></div>');

    frame = this.buttonEl.btnEl.child('div');
    frame.setStyle('width', params.width + 'px');
    frame.setStyle('height', params.height + 'px');

    rs = new Ext.Resizable(btn, { wrap: true });
    rs.on('resize', function(el, w, h) {
      frame.setStyle('width', w + 'px');
      frame.setStyle('height', h + 'px');

      $V.onResize(me, params, w, h);
    });
  };

  if (params.source == 'trends') {
    src = '/cgi-bin/scada-vis/trends.cgi';
  }
  else if (params.source == 'schedulers') {
    src = '/cgi-bin/scada-vis/schedulers.cgi';
  }
  else {
    src = params.url;
  }

  this.config.nobg = 1;
  this.createButton(false, true);
  this.buttonEl.btnEl.update('<iframe frameborder="0" border="0">');
  frame = this.buttonEl.btnEl.child('iframe');
  frame.set({
    src: src,
    width: params.width,
    height: params.height
  });

  this.buttonEl.addClass('scada-widget-iframe');
  return this;
};


$V.floors = {};

$V.onSaveFloor = function() {
  $V.EditorControl.setText($tr('vis_unlock'));
  $V.EditorControl.enable();

  $V.showFloor($V.floor, true);
};

$V.onShowFloor = function() {
  $V.Tree.enable();
  $V.Content.enable();
  $V.Editor.enable();
  $V.EditorControl.enable();

  $V.hideDeleteButtonForced();
};

$V.setPlanSize = function(panel, width, height) {
  panel.el.setStyle('width', (width || 1024) + 'px');
  panel.el.setStyle('height', (height || 768) + 'px');
};

$V.setPlanPosition = function(panel, top, left) {
  panel.el.setStyle('top', (top || 0) + 'px');
  panel.el.setStyle('left', (left || 0) + 'px');
};

$V.repositionPlans = function(width, height) {
  var pWidth = width || $V.planSize.width || 1024,
      pHeight = height || $V.planSize.height || 768,
      lWidth = $V.layoutCfg ? $V.layoutSize.width || 1024 : 0,
      lHeight = $V.layoutCfg ? $V.layoutSize.height || 768 : 0,
      wDelta = Math.round((lWidth - pWidth) / 2),
      hDelta = Math.round((lHeight - pHeight) / 2),
      top = Math.max(hDelta, 0), left = Math.max(wDelta, 0);

  $V.setPlanPosition($V.Content, top, left);
  $V.setPlanPosition($V.ContentBg, top, left);
  $V.setPlanPosition($V.ContentGrid, top, left);

  top = $V.layoutCfg ? Math.abs(Math.min(hDelta, 0)) : 0;
  left = $V.layoutCfg ? Math.abs(Math.min(wDelta, 0)) : 0;

  $V.setPlanPosition($V.Layout, top, left);
  $V.setPlanPosition($V.LayoutBg, top, left);
};

$V.doShowFloor = function(result) {
  var data = Ext.util.JSON.decode(result.responseText);

  $V.onShowFloor();

  $V.floorController.load(data);
  $V.planSize = data.size || {};
  $V.layoutSize = data.layoutsize || { width: 10, height: 10 };

  $V.setPlanSize($V.Content, $V.planSize.width, $V.planSize.height);
  $V.setPlanSize($V.ContentBg, $V.planSize.width, $V.planSize.height);
  $V.setPlanSize($V.ContentGrid, $V.planSize.width, $V.planSize.height);

  $V.setPlanSize($V.Layout, $V.layoutSize.width, $V.layoutSize.height);
  $V.setPlanSize($V.LayoutBg, $V.layoutSize.width, $V.layoutSize.height);

  $V.repositionPlans();
};

$V.showFloor = function(floor, forced) {
  var layout, node;

  if (floor && floor.floorid && ($V.floorId != floor.floorid || forced)) {
    $V.floorId = floor.floorid;
    $V.floor = floor;

    node = $V.Tree.getNodeById(floor.id);
    if (node) {
      node.select();
    }

    $V.layoutCfg = false;
    layout = floor.layout.toNumber();

    if (layout) {
      node = $V.Tree.root.findChild('floorid', layout, true);
      if (node) {
        $V.layoutCfg = node.attributes;
      }
    }

    $V.floorController.clean();
    $V.Tree.disable();
    $V.Content.disable();
    $V.Editor.disable();

    Ext.Ajax.request({
      url: Scada.reqUrl('vis', 'floor'),
      success: $V.doShowFloor,
      failure: $V.onShowFloor,
      params: Scada.encodeData({ id: $V.floor.floorid })
    });
  }
};

$V.sortFloor = function(a, b) {
  return parseInt(a[ 2 ], 10) || 0 - parseInt(b[ 2 ], 10) || 0;
};

$V.hideDeleteButton = function() {
  if (!$V.deleteButton.overIcon && !$V.deleteButton.overSelf) {
    $V.deleteButton.setPosition(-9999, -9999);
  }
};

$V.showDeleteButton = function() {
  if ($V.deleteButton.widget) {
    $V.deleteButton.widget.mouseover();
  }
};

$V.hideDeleteButtonForced = function() {
  $V.deleteButton.setPosition(-9999, -9999);
};

$V.deleteButton = new Ext.BoxComponent({ cls: 'scada-widget-delete' });

$V.onFocus = function() {
  $V.Tree.root.reload();
};

$V.isChecked = function(v) {
  return v == 'on' || v == 1;
};

$V.getVisParams = function(values, extend) {
  var result = {
    size: (values.size ? values.size.toNumber() : ''),
    color: values.color,
    font: values.font,
    bold: $V.isChecked(values.bold) ? 1 : 0,
    italic: $V.isChecked(values.italic) ? 1 : 0,
    underline: $V.isChecked(values.underline) ? 1 : 0
  }, w, h;

  if (values.width) {
    w = values.width.toNumber();

    if (w > 0) {
      result.width = w;
    }
  }

  if (values.height) {
    h = values.height.toNumber();

    if (h > 0) {
      result.height = h;
    }
  }

  $extend(result, extend || {});
  return Ext.util.JSON.encode(result);
};

$V.onBlur = function() {
  $V.unclocked = false;
  $V.EditorPanel.disable();
  $V.EditorControl.disable();
  $V.EditorControl.setText($tr('vis_unlock'));

  for (var f in $V.editorForms) {
    if ($V.editorForms.hasOwnProperty(f)) {
      $V.editorForms[ f ].reset();
    }
  }

  $V.setObjectForm();
  $V.setFloorLinkForm();
  $V.setImageForm();
  $V.setIframeForm();
};

$V.remove = function(store, record) {
  var id = record.id;

  if ($V.objectStore.objects[ id ]) {
    delete $V.objectStore.objects[ id ];
  }

  $V.reload(store);
};

$V.clear = function(store) {
  $V.objectStore.objects = {};
  $V.reload(store);
};

$V.reload = function(store) {
  $V.objectStore.update(store.data.items);
};

$V.setPlanGuides = function() {
  var width, height;

  if (!$V.PlanWidthInput.isValid() || !$V.PlanHeightInput.isValid()) {
    return;
  }

  width = $V.PlanWidthInput.getValue();
  height = $V.PlanHeightInput.getValue();

  $V.setPlanSize($V.Content, width, height);
  $V.setPlanSize($V.ContentBg, width, height);
  $V.setPlanSize($V.ContentGrid, width, height);

  if ($V.center) {
    $V.repositionPlans(width, height);
  }
};

$V.getPlanSize = function() {
  if (!$V.PlanWidthInput.isValid() || !$V.PlanHeightInput.isValid()) {
    return;
  }

  return {
    width: $V.PlanWidthInput.getValue(),
    height: $V.PlanHeightInput.getValue()
  };
};

$V.updateWidgetLocation = function() {
  var el;

  if (!$V.WidgetLocX.isValid() || !$V.WidgetLocY.isValid() || !$V.activeWidget) {
    return;
  }

  el = $V.activeWidget.buttonEl.el;
  el.setStyle('left', $V.WidgetLocX.getValue() + 'px');
  el.setStyle('top', $V.WidgetLocY.getValue() + 'px');
  $V.activeWidget.updateLocation();
};

$V.setWidgetLocForm = function(widget) {
  var pos;

  $V.WidgetLoc.setVisible(!!widget);
  $V.Sidebar.doLayout();

  if (widget) {
    pos = widget.buttonEl.el.getStyles('top', 'left');

    $V.WidgetLocX.setValue(pos.left.toNumber());
    $V.WidgetLocY.setValue(pos.top.toNumber());
  }

  $V.activeWidget = widget;
};

$V.touchOrderShow = function() {
  var root, res = [];

  Ext.each($V.floorObjects, function(item) {
    var type = $V.floorController.mapping[ item.type ], obj, name = '';

    if (item.notouch) {
      return;
    }

    if (type == 'object') {
      obj = $O.extObjStore.getExact('id', item.object.toNumber());
      if (obj) {
        name = obj.text;
      }
    }
    else if (type == 'floorlink') {
      obj = item.object.toNumber();

      if (obj) {
        obj = $V.comboBoxes.floor.store.getExact('id', obj);
        if (obj) {
          name = obj.text;
        }
      }
      else {
        name = $tr(item.object + '_title');
      }
    }
    else if (type != 'camera' && type != 'iframe') {
      return;
    }

    if (item.name) {
      if (name) {
        name = item.name + ' (' + name + ')';
      }
      else {
        name = item.name;
      }
    }

    if (name) {
      name = ': ' + name;
    }

    res.push({
      id: item.id,
      text: $tr('vis_widget_' + type) + name,
      cls: 'icon-' + type,
      leaf: true,
      draggable: true,
      sortorder: item.sortorder ? item.sortorder.toNumber() : 0
    });
  });

  if (!res.length) {
    return Ext.Msg.alert(
      $tr('error'),
      $tr('vis_touch_order_error')
    );
  }

  res.sort(function(a, b) {
    return a.sortorder.toNumber() - b.sortorder.toNumber();
  });

  root = new Ext.tree.AsyncTreeNode({
    draggable: false,
    children: res
  });

  $V.TouchOrderWin.show();
  $V.TouchOrderTree.setRootNode(root);
};

$V.touchOrderHide = function() {
  $V.TouchOrderWin.hide();
};

$V.touchOrderSave = function() {
  var nodes = $V.TouchOrderTree.root.childNodes, data = {};

  Ext.each(nodes, function(node, order) {
    data[ node.attributes.id ] = order + 1;
  });

  $V.touchOrderHide();

  Ext.Ajax.request({
    url: Scada.reqUrl('vis', 'order-save'),
    success: $V.onSaveFloor,
    failure: $V.onSaveFloor,
    params: Scada.encodeData(data)
  });
};

$V.TouchOrderWin = new Ext.Window($def.window({
  title: $tr('vis_touch_order'),
  width: 320,
  height: 480,
  draggable: true,
  resizable: true,
  bbar: [
    '->',
    { text: $tr('save'), handler: $V.touchOrderSave }, ' ',
    { text: $tr('cancel'), handler: $V.touchOrderHide }
  ],
  items: $def.tree({
    id: 'scada-vis-touch-tree',
    root: [],
    border: false,
    enableDD: true
  })
}));

$V.init = function() {
  var wrap = Ext.getCmp('scada-vis-wrap'), cls = $V.center ? 'scada-vis-panel-center' : 'scada-vis-panel';

  $V.Tree = Ext.getCmp('scada-vis-tree');

  $V.Content = new Ext.Container({ id: 'scada-vis-content', cls: cls });
  $V.Layout = new Ext.Container({ id: 'scada-vis-layout', cls: cls });
  $V.ContentBg = new Ext.Container({ id: 'scada-vis-content-bg', cls: cls });
  $V.LayoutBg = new Ext.Container({ id: 'scada-vis-layout-bg', cls: cls });

  $V.ContentGrid = new Ext.Container({ id: 'scada-vis-grid', cls: 'x-hide-display' });
  $V.Sidebar = Ext.getCmp('scada-vis-sidebar');

  $V.PlanWidthInput = Ext.getCmp('scada-vis-plan-width');
  $V.PlanHeightInput = Ext.getCmp('scada-vis-plan-height');

  $V.PlanWidthInput.on('spin', $V.setPlanGuides);
  $V.PlanWidthInput.on('keyup', $V.setPlanGuides);
  $V.PlanHeightInput.on('spin', $V.setPlanGuides);
  $V.PlanHeightInput.on('keyup', $V.setPlanGuides);

  $V.WidgetLocX = Ext.getCmp('scada-vis-loc-x');
  $V.WidgetLocY = Ext.getCmp('scada-vis-loc-y');

  $V.WidgetLocX.on('spin', $V.updateWidgetLocation);
  $V.WidgetLocX.on('keyup', $V.updateWidgetLocation);
  $V.WidgetLocY.on('spin', $V.updateWidgetLocation);
  $V.WidgetLocY.on('keyup', $V.updateWidgetLocation);

  $V.PlanSizes = Ext.getCmp('scada-vis-plan-size');
  $V.WidgetLoc = Ext.getCmp('scada-vis-widget-loc');

  wrap.add($V.LayoutBg);
  wrap.add($V.ContentBg);
  wrap.add($V.Layout);
  wrap.add($V.Content);
  wrap.add($V.ContentGrid);

  $V.Content.add($V.deleteButton);

  $V.deleteButton.on('afterrender', function() {
    $V.deleteButton.el.on('click', function() {
      if ($V.deleteButton.widget) {
        $V.deleteButton.widget.markDeleted();

        $V.deleteButton.overSelf = false;
        $V.hideDeleteButton();
      }
    });

    $V.deleteButton.el.on('mouseover', function() {
      $V.deleteButton.overSelf = true;
    });

    $V.deleteButton.el.on('mouseout', function() {
      $V.deleteButton.overSelf = false;
      $V.hideDeleteButton.defer(200);
    });
  });

  $V.comboBoxes = {
    object: Ext.getCmp('scada-vis-editor-object'),
    objectwidget: Ext.getCmp('scada-vis-editor-object-widget'),
    statusobject: Ext.getCmp('scada-vis-editor-statusobject'),
    floor: Ext.getCmp('scada-vis-editor-floor'),
    graphobject: Ext.getCmp('scada-vis-editor-graph-object'),
    gauge: Ext.getCmp('scada-vis-editor-gauge-object')
  };

  $V.Editor = Ext.getCmp('scada-vis-editor');
  $V.EditorPanel = Ext.getCmp('scada-vis-editor-panel');
  $V.EditorControl = Ext.getCmp('scada-vis-editor-control');

  $V.EditorPanel.disable();

  $V.EditorControl.on('click', function() {
    if ($V.unclocked) {
      $V.unclocked = false;
      $V.EditorPanel.disable();
      $V.EditorControl.disable();
      $V.floorController.save();
      $V.Layout.removeClass('scada-vis-fade');
    }
    else {
      $V.unclocked = true;
      $V.EditorControl.setText($tr('vis_save_and_lock'));
      $V.floorController.unlock();
      $V.EditorPanel.enable();
    }
  });

  $V.Tree.on('click', function(node) {
    $V.showFloor(node.attributes);
  });

  $V.Tree.loader.on('load', function(tree, rootNode) {
    $V.floors = {};
    var floorData = [], widgets = [];

    function addFloor(node) {
      var attr = node.attributes;
      if (attr && attr.floorid) {
        $V.floors[ attr.floorid ] = attr;

        if (attr.type == 'widget') {
          widgets.push([ attr.floorid, attr.text, attr.sortorder ]);
        }
        else if (!attr.type) {
          floorData.push([ attr.floorid, attr.text, attr.sortorder ]);
        }
      }

      node.eachChild(addFloor);
    }

    addFloor(rootNode);

    floorData.sort($V.sortFloor);
    floorData.unshift([ 'schedulers', $tr('schedulers_title') ]);
    floorData.unshift([ 'trends', $tr('trends_title') ]);
    $V.comboBoxes.floor.store.loadData(floorData);
    $V.comboBoxes.floor.reset();

    widgets.sort($V.sortFloor);
    widgets.unshift([ '', $tr('vis_object_widget_none') ]);
    $V.comboBoxes.objectwidget.store.loadData(widgets);
    $V.comboBoxes.objectwidget.reset();

    if ($V.floor) {
      $V.floor = $V.floors[ $V.floor.floorid ];
      $V.showFloor($V.floor, true);
    }
  });

  $O.store.on('load', $V.reload);
  $O.store.on('remove', $V.remove);
  $O.store.on('clear', $V.clear);

  $O.extObjStore.on('load', function() {
    var data = [ [ 0, $tr('vis_statusobject_no') ] ];

    Ext.each($O.extObjStore.getRange(), function(item) {
      data.push([ item.data.id, item.data.text ]);
    });

    $V.comboBoxes.statusobject.store.loadData(data);
    $V.comboBoxes.statusobject.reset();
  });

  $V.iconStore.on('load', function(store) {
    var data = [];
    data.push(['', $tr('vis_icon_empty')]);

    Ext.each(store.reader.jsonData.images, function(item) {
      data.push([ item.icon, item.icon ]);
    });

    $V.visIconStore.loadData(data);

    data.shift();
    $V.graphIconStore.loadData(data);
  });

  $V.iconStore.load();
  $V.imgStore.load();

  $V.editorForms = {};
  $V.editorSubmits = {};
  $V.editorDupes = {};
  $V.editorResets = {};
  $V.editorDeletes = {};

  var prefix = 'scada-vis-editor-';
  Ext.each($V.editorTypes, function(type) {
    var p = prefix + type + '-';
    $V.editorForms[ type ] = Ext.getCmp(p + 'panel').getForm();
    $V.editorSubmits[ type ] = Ext.getCmp(p + 'add');
    $V.editorDupes[ type ] = Ext.getCmp(p + 'dupe');
    $V.editorResets[ type ] = Ext.getCmp(p + 'reset');
    $V.editorDeletes[ type ] = Ext.getCmp(p + 'delete');
  });

  $V.editorSubmits.object.on('click', function() {
    if (!$V.editorForms.object.isValid()) {
      return;
    }

    var values = $V.editorForms.object.getValues(),
        oid = values.object.toNumber(),
        sid = values.statusobject ? values.statusobject.toNumber() : null,
        object = $V.objectStore.getObject(sid || oid), config, icons,
        widget = values.widget.toNumber();

    try {
      icons = Ext.util.JSON.decode(values.icons_add);
    }
    catch (e) {}

    config = {
      floor: $V.floorId,
      object: oid,
      statusobject: sid,
      type: object.boolean ? 0 : 1,
      locx: 10,
      locy: 10,
      params: $V.getVisParams(values, {
        icon_on: values.icon_on,
        icon_off: values.icon_off,
        icon_default: values.icon_default,
        icons_add: (icons && icons.length ? icons : []),
        displaymode: values.displaymode,
        showcontrol: values.showcontrol ? 1 : 0,
        fixedvalue: widget ? '' : values.fixedvalue,
        update: widget ? false : (typeof values.update != 'undefined'),
        pincode: values.pincode,
        widget: widget,
        backdrop: $V.isChecked(values.backdrop) ? 1 : 0
      }),
      name: values.name,
      readonly: values.readonly ? 1 : 0,
      notouch: values.notouch ? 1 : 0,
      nobg: values.nobg ? 1 : 0,
      sortorder: values.sortorder.toNumber(),
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
    Ext.getCmp('scada-vis-icons-additional-store').setValue('');
    $V.setObjectForm();
  });

  $V.editorSubmits.floorlink.on('click', function() {
    if (!$V.editorForms.floorlink.isValid()) {
      return;
    }

    var values = $V.editorForms.floorlink.getValues(), config;
    config = {
      floor: $V.floorId,
      object: values.floorid,
      type: 2,
      locx: 10,
      locy: 10,
      params: $V.getVisParams(values, {
        displaymode: values.displaymode,
        icon: values.icon,
        icon_active: values.icon_active
      }),
      name: values.name,
      notouch: values.notouch ? 1 : 0,
      nobg: values.nobg ? 1 : 0,
      sortorder: values.sortorder.toNumber(),
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
    $V.setFloorLinkForm();
  });

  $V.editorSubmits.camera.on('click', function() {
    if (!$V.editorForms.camera.isValid()) {
      return;
    }

    var values = $V.editorForms.camera.getValues(), config;
    config = {
      floor: $V.floorId,
      object: 0,
      type: 4,
      locx: 10,
      locy: 10,
      params: Ext.util.JSON.encode({
        autoopen: values.autoopen ? 1 : 0,
        width: values.width,
        win_width: values.win_width,
        height: values.height,
        win_height: values.win_height,
        src: values.src,
        icon: values.icon
      }),
      name: values.name,
      nobg: values.nobg ? 1 : 0,
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
  });

  $V.editorSubmits.graph.on('click', function() {
    if (!$V.editorForms.graph.isValid()) {
      return;
    }

    var values = $V.editorForms.graph.getValues(), config;
    config = {
      floor: $V.floorId,
      object: values.object.toNumber(),
      type: 5,
      locx: 10,
      locy: 10,
      params: Ext.util.JSON.encode({
        autoopen: values.autoopen ? 1 : 0,
        width: values.width,
        win_width: values.win_width,
        height: values.height,
        win_height: values.win_height,
        limit: values.limit.toNumber(),
        icon: values.icon
      }),
      name: values.name,
      nobg: values.nobg ? 1 : 0,
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
  });

  $V.editorSubmits.label.on('click', function() {
    if (!$V.editorForms.label.isValid()) {
      return;
    }

    var values = $V.editorForms.label.getValues(), config;

    config = {
      floor: $V.floorId,
      type: 6,
      locx: 10,
      locy: 10,
      params: $V.getVisParams(values),
      name: values.name,
      nobg: 1,
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
  });

  $V.editorSubmits.image.on('click', function() {
    if (!$V.editorForms.image.isValid()) {
      return;
    }

    var values = $V.editorForms.image.getValues(), config;

    config = {
      floor: $V.floorId,
      type: 7,
      locx: 10,
      locy: 10,
      params: Ext.util.JSON.encode({
        source: values.source,
        src: (values.source == 'local' ? values.src_local : values.src_remote),
        width: values.width,
        height: values.height,
        link: values.link
      }),
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
  });

  $V.editorSubmits.gauge.on('click', function() {
    if (!$V.editorForms.gauge.isValid()) {
      return;
    }

    var values = $V.editorForms.gauge.getValues(), oid = values.object.toNumber(), config;
    config = {
      floor: $V.floorId,
      object: oid,
      type: 8,
      locx: 10,
      locy: 10,
      params: Ext.util.JSON.encode({
        size: values.size
      }),
      readonly: values.readonly ? 1 : 0,
      name: values.name,
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
  });

  $V.editorSubmits.iframe.on('click', function() {
    if (!$V.editorForms.iframe.isValid()) {
      return;
    }

    var values = $V.editorForms.iframe.getValues(), config;

    config = {
      floor: $V.floorId,
      type: 9,
      locx: 10,
      locy: 10,
      params: Ext.util.JSON.encode({
        source: values.source,
        url: values.url,
        width: values.width,
        height: values.height
      }),
      notouch: values.notouch ? 1 : 0,
      name: values.name,
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
  });

  Ext.iterate($V.editorDupes, function(type, button) {
    button.on('click', function() {
      $V.floorController.dupe = true;
      $V.editorSubmits[ type ].fireEvent('click');
    });
  });

  Ext.iterate($V.editorDeletes, function(type, button) {
    button.on('click', function() {
      var e = $V.floorController.edit;

      if (e) {
        e.elem.markDeleted();
      }

      $V.editorResets[ type ].fireEvent('click');
    });
  });

  Ext.iterate($V.editorResets, function(type, button) {
    button.on('click', function() {
      $V.editorForms[ type ].reset();
      $V.floorController.stopEdit(type);
      $V.setObjectForm();
      $V.setFloorLinkForm();
      $V.setImageForm();
      $V.setIframeForm();
    });
  });

  $V.comboBoxes.object.on('select', $V.setObjectForm);
  $V.comboBoxes.statusobject.on('select', $V.setObjectForm);
  $V.comboBoxes.objectwidget.on('select', $V.setObjectForm);

  Ext.getCmp('scada-vis-object-display-mode').on('select', $V.setObjectForm);
  Ext.getCmp('scada-vis-link-display-mode').on('select', $V.setFloorLinkForm);
  Ext.getCmp('scada-vis-image-source').on('select', $V.setImageForm);
  Ext.getCmp('scada-vis-iframe-source').on('select', $V.setIframeForm);

  $V.setFloorLinkForm();
  $V.setImageForm();

  Scada.initImgSelect($V);
  Scada.initImgSelect($V, 'icon');

  Scada.bindImgSelect($V, 'vis-src-local');
  Ext.each($V.iconSelects, function(id) {
    Scada.bindImgSelect($V, id, 'icon');
  });

  Ext.iterate($V.comboBoxes, function(index, combo) {
    combo.on('afterrender', function(combo) {
      combo.reset();
    });
  });

  $V.TouchOrderTree = Ext.getCmp('scada-vis-touch-tree');
  $V.TouchOrderEdit = Ext.getCmp('scada-vis-touch-edit');
  $V.TouchOrderEdit.on('click', $V.touchOrderShow);
};

Scada.Modules.push($V);
