#!/usr/bin/haserl
Content-Type: text/javascript

Scada.VisGraphics = {};
var $VG = Scada.VisGraphics;

$VG.iconGrid = new Ext.DataView({
  cls: 'scada-icon-grid',
  store: Scada.Vis.iconStore,
  tpl: new Ext.XTemplate(
    '<tpl for=".">',
      '<div class="scada-icon-wrap scada-image-deletable">',
          '<div class="scada-icon">',
            '<div class="scada-icon-delete" data-id="{icon}" data-mode="icon"></div>',
            '<img src="/scada/resources/icons/{icon}" title="{icon}">',
          '</div>',
        '<div class="scada-icon-title">{icon}</div>',
      '</div>',
    '</tpl>',
    '<div class="x-clear"></div>'
  )
});

$VG.imgGrid = new Ext.DataView({
  cls: 'scada-icon-grid',
  store: Scada.Vis.imgStore,
  tpl: new Ext.XTemplate(
    '<tpl for=".">',
      '<div class="scada-icon-wrap scada-image-deletable">',
          '<div class="scada-icon">',
            '<div class="scada-icon-delete" data-id="{image}" data-mode="image"></div>',
            '<img src="/scada/resources/img/{image}" title="{image}">',
          '</div>',
        '<div class="scada-icon-title">{image}</div>',
      '</div>',
    '</tpl>',
    '<div class="x-clear"></div>'
  )
});

$VG.deleteBtn = {
  text: $tr('visgraphics_delete_selected'),
  iconCls: 'icon-clear',
  handler: function() {
    var el = $VG.MainPanel.getActiveTab().el, els, mode, files = [];
    els = el.query('.scada-icon-selected .scada-icon-delete');

    if (els.length) {
      Ext.each(els, function(el) {
        files.push(el.getAttribute('data-id'));
        mode = el.getAttribute('data-mode');
      });

      $VG.onDelete({
        files: files,
        mode: mode
      });
    }
  }
};

$VG.iconPanel = new Ext.Panel({
  title: $tr('visgraphics_icons'),
  autoScroll: true,
  items: $VG.iconGrid,
  bbar: [
    { text: $tr('visgraphics_add_icon'), iconCls: 'icon-add', id: 'vis-graphics-add-icon-button' },
    '-',
    $VG.deleteBtn
  ]
});

$VG.imgPanel = new Ext.Panel({
  title: $tr('visgraphics_img'),
  autoScroll: true,
  items: $VG.imgGrid,
  bbar: [
    { text: $tr('visgraphics_add_img'), iconCls: 'icon-add', id: 'vis-graphics-add-img-button' },
    '-',
    $VG.deleteBtn
  ]
});

$VG.cssPanel = new Ext.Panel({
  title: $tr('visgraphics_css_edit')
});

$VG.MainPanel = new Ext.TabPanel({
  border: false,
  region: 'center',
  margins: '5 0 0 0',
  items: [ $VG.iconPanel, $VG.imgPanel, $VG.cssPanel ]
});

$VG.deleteGraphics = function(files, mode) {
  Ext.Ajax.request({
    url: Scada.reqUrl('visgraphics', 'delete'),
    success: $VG.update,
    failure: $VG.update,
    params: Scada.encodeData({
      files: files,
      mode: mode
    })
  });
};

$VG.update = function() {
  $VG.addItemEvents();

  Scada.Vis.iconStore.reload();
  Scada.Vis.imgStore.reload();

  $VG.MainPanel.setDisabled(false);
};

$VG.uploadFW = new ExtFormWindow({
  id: 'vis-graphics-upload',
  window: { title: $tr('visgraphics_add'), width: 460, height: 210 },
  form: {
    url: Scada.reqUrl('visgraphics', 'save'),
    fileUpload: true,
    items: [
      { name: 'name', fieldLabel: $tr('visgraphics_name'), maskRe: /[\w\-]/ },
      $def.fileUpload({ name: 'file', fieldLabel: $tr('visgraphics_file') }),
      { name: 'mode', xtype: 'hidden' },
      $def.formInfoPanel('visgraphics_help')
    ]
  },
  events: { onFormSubmitComplete: $VG.update, onFormSubmitFailed: $VG.update }
});

$VG.onDeleteConfirm = function(res) {
  $VG.clearSelection();
  if (res == 'yes') {
    $VG.MainPanel.setDisabled(true);
    Ext.Ajax.request({
      url: Scada.reqUrl('visgraphics', 'delete'),
      success: $VG.update,
      failure: $VG.update,
      params: Scada.encodeData(this)
    });
  }
};

$VG.onDelete = function(data) {
  $VG.clearSelection();
  Ext.Msg.confirm(
    $tr('visgraphics_delete_title'),
    $tr('visgraphics_delete_confirm'),
    $VG.onDeleteConfirm,
    data
  );
};

$VG.onDeleteClick = function() {
  $VG.onDelete({
    files: this.getAttribute('data-id'),
    mode: this.getAttribute('data-mode')
  });
};

$VG.attachImageHover = function(target) {
  var tmp = new Image(), src = target.src;

  tmp.onload = function() {
    if (this.width > 70 || this.height > 70) {
      Ext.QuickTips.register({
        target: target,
        cls: 'x-html-tip',
        text: '<img src="' + this.src + '">'
      });
    }
  }

  tmp.src = src;
};

$VG.clearSelection = function() {
  var els;

  if ($VG.MainPanel.el) {
    els = $VG.MainPanel.el.query('.scada-icon-selected');
    Ext.get(els).removeClass('scada-icon-selected')
  }
};

$VG.onElOver = function() {
  var img = this.query('img')[0], tip = $VG.previewEl;
  this.addClass('scada-icon-hover');

  if (img.naturalWidth > 70 || img.naturalHeight > 70) {
    tip.update('<img src="' + img.src + '">');
    tip.show();
    tip.alignTo(this, 'tr?');
  }
};

$VG.onElOut = function() {
  this.removeClass('scada-icon-hover');
  $VG.previewEl.hide()
};

$VG.onElClick = function() {
  this.toggleClass('scada-icon-selected');
};

$VG.addItemEvents = function() {
  var items = Ext.query('.scada-image-deletable');
  $VG.clearSelection();

  Ext.each(items, function(item) {
    var el = Ext.get(item).removeAllListeners(), delBtn = el.query('.scada-icon-delete')[0];

    el.on('mouseover', $VG.onElOver);
    el.on('mouseout', $VG.onElOut);
    el.on('click', $VG.onElClick);

    Ext.get(delBtn).on('click', $VG.onDeleteClick);
  });
};

$VG.container = new Ext.Container({
  id: 'VisGraphics',
  title: $tr('visgraphics_title'),
  layout: 'border',
  items: $VG.MainPanel
});

$VG.init = function() {
  Scada.Vis.iconStore.on('load', $VG.addItemEvents);
  Scada.Vis.imgStore.on('load', $VG.addItemEvents);

  Ext.getCmp('vis-graphics-add-icon-button').on('click', function() {
    $VG.uploadFW.show();
    $VG.uploadFW.formPanel.form.findField('mode').setValue('icon');
  });

  Ext.getCmp('vis-graphics-add-img-button').on('click', function() {
    $VG.uploadFW.show();
    $VG.uploadFW.formPanel.form.findField('mode').setValue('image');
  });

  $VG.MainPanel.on('beforetabchange', function(prev, curr) {
    return curr != $VG.cssPanel;
  });

  $VG.MainPanel.on('afterrender', function() {
    el = $VG.MainPanel.el.query('li')[ 2 ];

    Ext.get(el).on('click', function() {
      window.open('/cgi-bin/scada/editor.cgi?id=css');
    });
  });

  $VG.MainPanel.on('tabchange', $VG.addItemEvents);
  $VG.MainPanel.setActiveTab(0);

  $VG.previewEl = Ext.DomHelper.append(Ext.getBody(), { tag: 'div', cls: 'scada-preview' }, true);
};

$VG.onFocus = $VG.update;
Scada.Modules.push($VG);
