#!/usr/bin/haserl
Content-Type: text/javascript

Scada.Utilities = {};
var $U = Scada.Utilities;

$U.languageStore = new Ext.data.JsonStore({
  fields: ['code', 'name'],
  root: 'languages',
  url: Scada.reqUrl('general', 'languages')
});

$U.neighbourStore = new Ext.data.JsonStore({
  fields: ['ip', 'name'],
  root: 'neighbours',
  url: Scada.reqUrl('general', 'neighbours')
});

$U.buttons = [];

$U.buttons.push({
  id: 'utilities-btn-esf-import',
  text: $tr('utilities_esf_import'),
  iconCls: 'scada-utilities-btn-esf-import',
  xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
});

$U.buttons.push({
  id: 'utilities-btn-neighbour-import',
  text: $tr('utilities_neighbour_import'),
  iconCls: 'scada-utilities-btn-esf-import',
  cls: 'hidden',
  hidden: true,
  xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
});

$U.buttons.push({
  id: 'utilities-btn-reset',
  text: $tr('utilities_reset'),
  iconCls: 'scada-utilities-btn-objects-reset',
  hidden: true,
  xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
});

$U.buttons.push({
  id: 'utilities-btn-factory-reset',
  text: $tr('utilities_factory_reset'),
  iconCls: 'scada-utilities-btn-objects-reset',
  hidden: true,
  xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
});

$U.buttons.push({
  id: 'utilities-btn-datetime',
  text: $tr('utilities_time_settings'),
  iconCls: 'scada-utilities-btn-datetime',
  xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
});

$U.buttons.push({
  id: 'utilities-btn-update',
  text: $tr('utilities_update'),
  iconCls: 'scada-utilities-btn-update',
  xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
});

$U.buttons.push({
  id: 'utilities-btn-backup',
  text: $tr('utilities_backup'),
  iconCls: 'scada-utilities-btn-backup',
  xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
});

$U.buttons.push({
  id: 'utilities-btn-restore',
  text: $tr('utilities_restore'),
  iconCls: 'scada-utilities-btn-restore',
  xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
});

$U.buttons.push({
  id: 'utilities-btn-config',
  text: $tr('utilities_config'),
  iconCls: 'scada-utilities-btn-config',
  xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
});

$U.buttons.push({
  id: 'utilities-btn-system',
  text: $tr('utilities_system'),
  iconCls: 'scada-utilities-btn-config',
  handler: function() { window.open('/cgi-bin/flashsys/index.cgi') },
  xtype: 'splitbutton', scale: 'large', iconAlign: 'bottom', minWidth: 120, menu: {
    items: [
      { text: $tr('utilities_config_knx'), iconCls: 'icon-list-view', handler: function() { window.open('/cgi-bin/flashsys/index.cgi#network-knx') } },
      { text: $tr('utilities_config_login'), iconCls: 'icon-list-view', handler: function() { window.open('/cgi-bin/flashsys/index.cgi#system-login') } }
    ]
  }
});

$U.pageReload = function() {
  window.location.reload();
};

$U.resetCB = function() {
  Scada.Objects.update(true);
};

$U.resetFormWindow = new ExtFormWindow({
  id: 'utilities-reset',
  window: {
    title: $tr('utilities_reset'),
    width: 300,
    height: 260
  },
  form: {
    url: Scada.reqUrl('general', 'reset'),
    items: [
      { name: 'objects', fieldLabel: $tr('utilities_reset_objects'), xtype: 'checkbox', width: 30 },
      { name: 'objectlogs', fieldLabel: $tr('utilities_reset_objectlogs'), xtype: 'checkbox', width: 30 },
      { name: 'alerts', fieldLabel: $tr('utilities_reset_alerts'), xtype: 'checkbox', width: 30 },
      { name: 'logs', fieldLabel: $tr('utilities_reset_logs'), xtype: 'checkbox', width: 30 },
      { name: 'errors', fieldLabel: $tr('utilities_reset_errors'), xtype: 'checkbox', width: 30 },
      { name: 'storage', fieldLabel: $tr('utilities_reset_storage'), xtype: 'checkbox', width: 30 }
    ]
  },
  events: { onFormSubmitComplete: $U.resetCB, onFormSubmitFailed: $U.resetCB }
});

$U.importCB = function(form, action) {
  var r = action.result, i, d;

  if (r.imported && r.discarded) {
    i = r.imported.join(', ') || $tr('utilities_import_none');
    d = r.discarded.join(', ') || $tr('utilities_import_none');

    Ext.MessageBox.alert($tr('utilities_import_result_title'), $tr('utilities_import_result', i, d));
  }
  else if (r.error) {
    Ext.MessageBox.alert($tr('utilities_import_result_title'), $tr('utilities_import_error', r.error));
  }

  Scada.Objects.update(true);
};

$U.importFormWindow = new ExtFormWindow({
  id: 'utilities-esf-import',
  window: {
    title: $tr('utilities_esf_import'),
    width: 460,
    height: 180
  },
  form: {
    url: Scada.reqUrl('objects', 'import'),
    fileUpload: true,
    items: [
      $def.fileUpload({ fieldLabel: $tr('utilities_import') }),
      $def.formInfoPanel('utilities_import_help')
    ]
  },
  events: { onFormSubmitComplete: $U.importCB, onFormSubmitFailed: $U.importCB }
});

$U.importNeighbourFormWindow = new ExtFormWindow({
  id: 'utilities-neighbours-import',
  window: {
    title: $tr('utilities_neighbour_import'),
    width: 460,
    height: 180
  },
  form: {
    url: Scada.reqUrl('objects', 'import-neighbours'),
    items: [
      $def.select({
        fieldLabel: $tr('utilities_import_neighbour'), name: 'ip', hiddenName: 'ip', allowBlank: false,
        store: $U.neighbourStore, valueField: 'ip', displayField: 'name', mode: 'local'
      }),
      { name: 'password', fieldLabel: $tr('utilities_import_password'), allowBlank: false }
    ]
  },
  events: { onFormSubmitComplete: $U.importCB, onFormSubmitFailed: $U.importCB }
});

$U.timezones = new Ext.data.ArrayStore({
  fields: ['value'],
  idIndex: 0,
  data: <? lua /lib/genohm-scada/web/general/timezones.lua ?>
});

$U.useCurrentDateTime = function() {
  var d = new Date(), form = $U.datetimeFW.form;
  form.findField('date').setValue(d);
  form.findField('timeh').setValue(d.getHours());
  form.findField('timem').setValue(d.getMinutes());
  form.findField('times').setValue(d.getSeconds());
};

$U.spacer = { columnWidth: 0.03, border: false, html: '&nbsp;' };

$U.datetimeFW = new ExtFormWindow({
  id: 'utilities-datetime',
  window: {
    title: $tr('utilities_time_settings'),
    width: 460,
    height: 210
  },
  form: {
    url: Scada.reqUrl('general', 'datetime-save'),
    loadUrl: Scada.reqUrl('general', 'datetime-form'),
    items: [
      { name: 'current', xtype: 'displayfield', fieldLabel: $tr('utilities_dt_current') },
      {
        xtype: 'panel', layout: 'column', border: false, fieldLabel: $tr('utilities_time'), items: [
          { xtype: 'spinnerfield', name: 'timeh', zeroPad: true, padLength: 2, minValue: 0, maxValue: 23, columnWidth: 0.28 },
          $U.spacer,
          { xtype: 'spinnerfield', name: 'timem', zeroPad: true, padLength: 2, minValue: 0, maxValue: 59, columnWidth: 0.28 },
          $U.spacer,
          { xtype: 'spinnerfield', name: 'times', zeroPad: true, padLength: 2, minValue: 0, maxValue: 59, columnWidth: 0.28 },
          $U.spacer,
          { xtype: 'button', iconCls: 'icon-accept', text: $tr('utilities_use_current'), handler: $U.useCurrentDateTime }
        ]
      },
      { name: 'date', xtype: 'datefield', fieldLabel: $tr('utilities_date'), format: 'd.m.Y', showToday: false },
      $def.select({
        fieldLabel: $tr('utilities_timezone'), name: 'timezone', hiddenName: 'timezone',
        store: $U.timezones, valueField: 'value', displayField: 'value', mode: 'local'
      })
    ]
  }
});

$U.configFW = new ExtFormWindow({
  id: 'utilities-config',
  window: {
    title: $tr('utilities_config'),
    width: 490,
    height: 520
  },
  form: {
    url: Scada.reqUrl('general', 'config-save'),
    loadUrl: Scada.reqUrl('general', 'config-form'),
    labelWidth: 180,
    items: [
      $def.select({
        name: 'language', hiddenName: 'language', fieldLabel: $tr('utilities_language'),
        mode: 'local', store: $U.languageStore, valueField: 'code', displayField: 'name'
      }),
      { name: 'perpage', fieldLabel: $tr('utilities_perpage'), xtype: 'spinnerfield', minValue: 10, maxValue: 50 },
      { name: 'autorange', fieldLabel: $tr('utilities_autorange'), invalidText: $tr('objects_error_address'), allowBlank: false, vtype: 'knxga' },
      $def.select({
        name: 'autoadd', hiddenName: 'autoadd', fieldLabel: $tr('utilities_autoadd'),
        store: [['1',  $tr('utilities_autoadd_enable')], ['0',  $tr('utilities_autoadd_disable')]]
      }),
      { name: 'logobjects', fieldLabel: $tr('utilities_logobjects'), xtype: 'spinnerfield', minValue: 100, maxValue: 50000 },
      $def.select({
        name: 'logpolicy', hiddenName: 'logpolicy', fieldLabel: $tr('utilities_logpolicy'),
        store: [['drop',  $tr('utilities_logpolicy_drop')], ['accept',  $tr('utilities_logpolicy_accept')]]
      }),
      { name: 'logalerts', fieldLabel: $tr('utilities_logalerts'), xtype: 'spinnerfield', minValue: 50, maxValue: 5000 },
      { name: 'loglogs', fieldLabel: $tr('utilities_loglogs'), xtype: 'spinnerfield', minValue: 50, maxValue: 5000 },
      { name: 'logerrors', fieldLabel: $tr('utilities_logerrors'), xtype: 'spinnerfield', minValue: 50, maxValue: 5000 },
      $def.select({
        name: 'fullscreen', hiddenName: 'fullscreen', fieldLabel: $tr('utilities_sidebar'),
        store: [['0',  $tr('utilities_sidebar_0')], ['2',  $tr('utilities_sidebar_2')], ['1',  $tr('utilities_sidebar_1')]]
      }),
      $def.select({
        name: 'viscenter', hiddenName: 'viscenter', fieldLabel: $tr('utilities_viscenter'),
        store: [['0',  $tr('utilities_viscenter_disable')], ['1',  $tr('utilities_viscenter_enable')], ['2',  $tr('utilities_viscenter_enable_auto')]]
      }),
      { name: 'useralerts', xtype: 'checkbox', fieldLabel: $tr('utilities_useralerts') },
      $def.formInfoPanel('utilities_config_help')
    ]
  },
  events: {
    onFormSubmitComplete: $U.pageReload
  }
});

$U.updateCB = function(form, action) {
  var r = action.result || {};
  var m = r.success ? 'ok' : ('error_' + (r.error || 'file'));

  if (r.succes) {
    this.hide();
  }

  Ext.Msg.alert(
    $tr('utilities_update'),
    $tr('utilities_update_' + m)
  );
};

$U.updateFW = new ExtFormWindow({
  id: 'utilities-update',
  window: { title: $tr('utilities_update'), width: 460, height: 180 },
  form: {
    url: Scada.reqUrl('general', 'update'),
    fileUpload: true,
    items: [
      $def.fileUpload({ id: 'utilities-update-file', fieldLabel: $tr('utilities_update_file') }),
      $def.formInfoPanel('utilities_update_help')
    ]
  },
  events: { onFormSubmitComplete: $U.updateCB, onFormSubmitFailed: $U.updateCB }
});

$U.restoreCB = function(form, action) {
  var r = action.result || {}, m = r.success ? 'ok' : 'error';

  if (r.succes) {
    this.hide();
  }

  Ext.Msg.alert(
    $tr('utilities_restore'),
    $tr('utilities_restore_' + m)
  );
};

$U.restoreFW = new ExtFormWindow({
  id: 'utilities-restore',
  window: { title: $tr('utilities_restore'), width: 460, height: 180 },
  form: {
    url: Scada.genUrl('utilities-restore'),
    fileUpload: true,
    items: [
      $def.fileUpload({ id: 'utilities-restore-file', fieldLabel: $tr('utilities_restore_file') }),
      $def.formInfoPanel('utilities_restore_help')
    ]
  },
  events: { onFormSubmitComplete: $U.restoreCB, onFormSubmitFailed: $U.restoreCB }
});

$U.container = new Ext.Container({
  id: 'Utilities',
  title: $tr('utilities_title'),
  layout: 'border',
  items: {
    region: 'center',
    layout: 'column',
    ctCls: 'x-utilities-menu',
    border: false,
    margins: '10 10 10 10',
    items: $U.buttons
  }
});

$U.onResetComplete = function() {
  if (Scada.currentModule.update) {
    Scada.currentModule.update(true);
  }

  if (this.resetType == 'objects') {
    Scada.Objects.update(true);
  }

  Scada.Viewport.setDisabled(false);
};

$U.onResetConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    var data = {};
    data[ this.resetType ] = true;

    Scada.Viewport.setDisabled(true);

    Ext.Ajax.request({
      url: Scada.reqUrl('general', 'reset'),
      success: $U.onResetComplete,
      failure: $U.onResetComplete,
      params: Scada.encodeData(data)
    });
  }
};

$U.reset = function() {
  if (this.resetType) {
    Ext.Msg.confirm(
      $tr('utilities_reset_button'),
      $tr('utilities_reset_confirm'),
      $U.onResetConfirm,
      this
    );
  }
};

$U.onFactoryResetComplete = function() {
  Scada.Viewport.setDisabled(false);

  Ext.Msg.alert(
    $tr('utilities_factory_reset'),
    $tr('utilities_factory_reset_alert')
  );
}

$U.onFactoryResetConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    Scada.Viewport.setDisabled(true);

    Ext.Ajax.request({
      url: Scada.genUrl('utilities-reset'),
      success: $U.onFactoryResetComplete,
      failure: $U.onFactoryResetComplete
    });
  }
};

$U.factoryReset = function() {
  Ext.Msg.confirm(
    $tr('utilities_factory_reset'),
    $tr('utilities_factory_reset_confirm'),
    $U.onFactoryResetConfirm
  );
};

$U.onBusStatusConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    Scada.Viewport.setDisabled(true);

    Ext.Ajax.request({
      url: Scada.reqUrl('general', 'bus-set-ip'),
      success: $U.pageReload,
      failure: $U.pageReload
    });
  }
};

$U.onBusStatusComplete = function(result) {
  if (!Ext.decode(result.responseText)) {
    Ext.Msg.confirm(
      $tr('utilities_bus_status_title'),
      $tr('utilities_bus_status_text'),
      $U.onBusStatusConfirm
    );
  }
};

$U.onUsageLoad = function(res) {
  var data = Ext.decode(res.responseText), text;
  text = $tr('usage', data[ 0 ], data[ 1 ]);
  Ext.get('scada-usage').update(text);
};

$U.usageLoad = function() {
  Ext.Ajax.request({
    url: Scada.reqUrl('general', 'usage'),
    success: $U.onUsageLoad
  });
};

$U.init = function() {
  var runner;

  Ext.getCmp('utilities-btn-reset').on('click', $U.resetFormWindow.show, $U.resetFormWindow);
  Ext.getCmp('utilities-btn-factory-reset').on('click', $U.factoryReset );
  Ext.getCmp('utilities-btn-esf-import').on('click', $U.importFormWindow.show, $U.importFormWindow);
  Ext.getCmp('utilities-btn-neighbour-import').on('click', $U.importNeighbourFormWindow.show, $U.importNeighbourFormWindow);
  Ext.getCmp('utilities-btn-datetime').on('click', $U.datetimeFW.show, $U.datetimeFW);
  Ext.getCmp('utilities-btn-update').on('click', $U.updateFW.show, $U.updateFW);
  Ext.getCmp('utilities-btn-config').on('click', $U.configFW.show, $U.configFW);
  Ext.getCmp('utilities-btn-restore').on('click', $U.restoreFW.show, $U.restoreFW);

  Ext.getCmp('utilities-btn-backup').on('click', function() {
    var url = Scada.genUrl('utilities-backup');
    window.open(url);
  });

  Ext.each([ 'objects', 'objectlogs', 'alerts', 'errors', 'logs' ], function(resetType) {
    var cmp = Ext.getCmp('reset-' + resetType);
    if (cmp) {
      cmp.resetType = resetType;
      cmp.setHandler($U.reset, cmp);
    }
  });

  $U.languageStore.on('load', function(store, records) {
    if (records.length < 2) {
      return;
    }

    var select = Ext.get('language-select');

    Ext.each(records, function(record, index) {
      var data = record.data, code = data.code || '';
      select.dom.options[ index ] = new Option(data.name, code, code == Config.language, code == Config.language);
    });

    select.on('change', function() {
      var language = select.getValue();

      if (language != Config.language) {
        Ext.Ajax.request({
          url: Scada.reqUrl('general', 'config-save'),
          success: $U.pageReload,
          failure: $U.pageReload,
          params: Scada.encodeData({ language: language })
        });
      }
    });

    Ext.get('language-select-wrap').removeClass('hidden');
  });

  $U.neighbourStore.on('load', function(store, records) {
    if (!records.length) {
      return;
    }

    var select = Ext.get('neighbour-select');

    Ext.each(records, function(record, index) {
      var data = record.data;
      select.dom.options[ index + 1 ] = new Option(data.name, data.ip);
    });

    select.on('change', function() {
      var ip = select.getValue();
      if (ip) {
        window.location = 'http://' + ip + '/cgi-bin/scada/index.cgi';
      }
    });

    Ext.get('neighbour-select-wrap').removeClass('hidden');
    Ext.getCmp('utilities-btn-neighbour-import').removeClass('hidden');
  });

  $U.languageStore.reload();
  $U.neighbourStore.reload();

  Ext.Ajax.request({
    url: Scada.reqUrl('general', 'bus-status'),
    success: $U.onBusStatusComplete
  });

  runner = new Ext.util.TaskRunner();
  runner.start({ run: $U.usageLoad, interval: 10000 });
};

Scada.Modules.push($U);
