#!/usr/bin/haserl
Content-Type: text/javascript

Scada.Logs = {};
$L = Scada.Logs;

$L.store = new Ext.data.JsonStore({
  url: Scada.reqUrl('logs'),
  root: 'data',
  fields: [ 'id', 'scriptname', 'log', 'logtime' ]
});

$L.logStore = new Ext.data.JsonStore({
  url: Scada.reqUrl('logs'),
  root: 'data',
  fields: [ 'id', 'scriptname', 'log', 'logtime' ],
  baseParams: { limit: 20, start: 0 }
});

$L.pagination = new Ext.PagingToolbar({
  pageSize: Config.perpage,
  store: $L.store,
  displayInfo: true,
  displayMsg: $tr('logs_pages_hint'),
  emptyMsg: $tr('logs_pages_empty'),
  refreshText: false, firstText: false, lastText: false, nextText: false, prevText: false,
  prependButtons: true,
  items: [
    { text: $tr('utilities_reset_button'), id: 'reset-logs', iconCls: 'icon-clear' }, '-',
    { text: $tr('logs_window_show'), iconCls: 'icon-accept', handler: function() { $L.logWindow.show() } }, '-'
  ]
});

$L.grid = new Ext.grid.GridPanel($def.grid({
  id: 'logs-grid',
  store: $L.store,
  bbar: $L.pagination,

  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('logs_time'), width: .15, dataIndex: 'logtime', xtype: 'timestampcolumn' },
      { header: $tr('scripting_name'), width: .15, dataIndex: 'scriptname', renderer: Ext.util.Format.htmlEncode },
      { header: $tr('logs_text'), width: .70, dataIndex: 'log', renderer: Ext.util.Format.htmlEncode }
    ]
  })
}));

$L.window = new Ext.Window($def.window({
  width: 600,
  height: 400
}));

$L.container = new Ext.Container({
  id: 'Logs',
  title: $tr('logs_title'),
  layout: 'border',
  items: $L.grid
});

$L.logUpdate = function() {
  $L.logStore.reload();
};

$L.update = function() {
  $L.pagination.doRefresh();
};

$L.maxLogId = 0;

$L.logWindow = new Ext.Window({
  title: $tr('logs_window'),
  width: 600,
  height: 400,
  closable: true,
  animCollapse: false,
  collapsible: true,
  closeAction: 'hide',
  plain: true,
  layout: 'fit',
  shadow: false,
  html: '<div class="scada-log-code" id="scada-log-wrap"><pre id="scada-log-content"></pre></div>',
  border: false,
  bbar: [
    { xtype: 'checkbox', id: 'scada-log-autoscroll', boxLabel: $tr('logs_autoscroll'), checked: true },
    '->',
    { iconCls: 'icon-clear', text: $tr('utilities_reset_button'), handler: function() { Ext.get('scada-log-content').dom.innerHTML = ''; } }
  ]
});

$L.init = function() {
  $L.logTask = { run: $L.logUpdate, interval: 3000 };
  $L.logRunner = new Ext.util.TaskRunner();

  $L.reloaderTask = { run: $L.update, interval: 6000 };

  $L.grid.on('cellclick', function(grid, rowIndex) {
    var row = grid.getStore().getAt(rowIndex);

    var log = row.get('log'), time = row.get('logtime');
    if (log) {
      var dt = Date.parseDate(time, 'U', true);
      if (dt) {
        Scada.Logs.window.setTitle($tr('logs_window_title') + dt.format(Scada.DateTimeLongFormat));
      }

      $L.window.show().center();
      $L.window.body.update('<div class="scada-log-code"><pre>' + Ext.util.Format.htmlEncode(log) + '</pre></div>');
    }
  });

  $L.logStore.on('load', function(store, data) {
    var html = [], content = Ext.get('scada-log-content'), max = 0;

    Ext.each(data, function(item) {
      var data = item.data, dt = Date.parseDate(data.logtime, 'U', true);

      if (data.id > $L.maxLogId) {
        html.push('<b>' + data.scriptname + '</b> ' + dt.format(Scada.DateTimeLongFormat) + '\n');
        html.push(data.log, '<hr>');
      }

      max = Math.max(data.id, max);
    });

    $L.maxLogId = Math.max($L.maxLogId, max);
    if ($L.maxLogId > max) {
      $L.maxLogId = 0;
    }

    if (content && content.dom && html.length) {
      content.dom.innerHTML = html.join('') + content.dom.innerHTML;
      if (Ext.getCmp('scada-log-autoscroll').checked) {
        Ext.get('scada-log-wrap').dom.scrollTop = 0;
      }
    }
  });

  $L.logWindow.on('show', function() {
    $L.logRunner.start($L.logTask);
  });

  $L.logWindow.on('hide', function() {
    $L.logRunner.stop($L.logTask);
  });
}

Scada.Modules.push($L);
