#!/usr/bin/haserl
Content-Type: text/javascript; charset=utf-8

Scada.Editor = {};
var $E = Scada.Editor;
$E.css = $S.data.id == 'css';

$E.editorPanel = new Ext.Panel({
  title: '&nbsp;',
  region: 'center',
  margins: $E.css ? '10 10 0 10' : '10 10 0 0'
});

$E.editorPanel.on('afterrender', function() {
  if (!$E.css) {
    CodeMirror.commands.autocomplete = function(cm) {
      CodeMirror.showHint(cm, CodeMirror.luaHint);
    }
  }

  $E.codeMirror = CodeMirror($E.editorPanel.body, {
    lineNumbers: true,
    mode: $E.css ? 'text/css' : 'text/x-lua',
    tabMode: 'shift',
    tabSize: 2,
    highlightSelectionMatches: true,
    extraKeys: { 'Ctrl-Space': 'autocomplete', 'Ctrl-Enter': 'autocomplete' }
  });

  $E.codeMirror.setValue($E.script.script || '');
  $E.resize();
});

$E.saveButton = new Ext.Button({ text: 'Save' });
$E.saveCloseButton = new Ext.Button({ text: 'Save and close' });
$E.closeButton = new Ext.Button({ text: 'Close' });

$E.fbar = [ Ext.isMac ? $tr('scripting_editor_help_mac') : $tr('scripting_editor_help'), '->', $E.saveButton, $E.saveCloseButton, $E.closeButton ];

if (!$E.css) {
  $E.sidePanel = [];

  $E.sidePanel.push($def.tree({
    border: false,
    title: $tr('scripting_helpers'),
    id: 'scripting-helpers',
    dataUrl: '/scada/resources/helpers.js'
  }));

  $E.sidePanel.push($def.tree({
    border: false,
    title: $tr('scripting_group_addresses'),
    id: 'scripting-group-addresses',
    root: {
      expanded: true,
      children: <? lua /lib/genohm-scada/web/scripting/group-addresses.lua ?>
    }
  }));

  $E.sidePanel.push($def.tree({
    border: false,
    title: $tr('scripting_objects'),
    id: 'scripting-objects',
    root: {
      expanded: true,
      children: <? lua /lib/genohm-scada/web/scripting/objects.lua ?>
    }
  }));

  $E.sidePanel.push($def.tree({
    border: false,
    title: $tr('scripting_tags'),
    id: 'scripting-tags',
    root: {
      expanded: true,
      children: <? lua /lib/genohm-scada/web/scripting/tags.lua ?>
    }
  }));

  $E.sidePanel.push($def.tree({
    border: false,
    title: $tr('scripting_datatypes'),
    id: 'scripting-datatypes',
    root: {
      expanded: true,
      children: [
        { text: $tr('objects_datatype_1_h'), cls: 'file', leaf: true, value: 'dt.bool' },
        { text: $tr('objects_datatype_2_h'), cls: 'file', leaf: true, value: 'dt.bit2' },
        { text: $tr('objects_datatype_3_h'), cls: 'file', leaf: true, value: 'dt.bit4' },
        { text: $tr('objects_datatype_4'), cls: 'file', leaf: true, value: 'dt.char' },
        { text: $tr('objects_datatype_5_h'), cls: 'file', leaf: true, value: 'dt.uint8' },
        { text: $tr('objects_datatype_5001_h'), cls: 'file', leaf: true, value: 'dt.scale' },
        { text: $tr('objects_datatype_5003_h'), cls: 'file', leaf: true, value: 'dt.angle' },
        { text: $tr('objects_datatype_6_h'), cls: 'file', leaf: true, value: 'dt.int8' },
        { text: $tr('objects_datatype_7_h'), cls: 'file', leaf: true, value: 'dt.uint16' },
        { text: $tr('objects_datatype_8_h'), cls: 'file', leaf: true, value: 'dt.int16' },
        { text: $tr('objects_datatype_9'), cls: 'file', leaf: true, value: 'dt.float16' },
        { text: $tr('objects_datatype_254'), cls: 'file', leaf: true, value: 'dt.uint24' },
        { text: $tr('objects_datatype_10'), cls: 'file', leaf: true, value: 'dt.time' },
        { text: $tr('objects_datatype_11'), cls: 'file', leaf: true, value: 'dt.date' },
        { text: $tr('objects_datatype_12'), cls: 'file', leaf: true, value: 'dt.uint32' },
        { text: $tr('objects_datatype_13'), cls: 'file', leaf: true, value: 'dt.int32' },
        { text: $tr('objects_datatype_14'), cls: 'file', leaf: true, value: 'dt.float32' },
        { text: $tr('objects_datatype_15'), cls: 'file', leaf: true, value: 'dt.access' },
        { text: $tr('objects_datatype_16'), cls: 'file', leaf: true, value: 'dt.string' }
      ]
    }
  }));

  $E.reloadParent = function() {
    var win = window.opener, store;
    if (win && $E.script.type) {
      store = $E.script.type[ 0 ] + 'Store';

      try {
        win.$S[ store ].reload();
      }
      catch (e) {}
    }
  };

  if ($S.data.id.toNumber()) {
    $E.toggleScriptBtnUpdate = function(status) {
      status = status.toNumber();
      $E.toggleScriptBtn.setText($tr(status ? 'enabled' : 'disabled'));
      $E.toggleScriptBtn.setIconClass(status ? 'icon-accept' : 'icon-delete');
    };

    $E.toggleScriptResult = function(res) {
      var res = res ? res.responseText : '';

      if (res == '0' || res == '1') {
        $E.toggleScriptBtnUpdate(res);
      }

      Scada.Viewport.enable();
      $E.reloadParent();
    };

    $E.toggleScriptStatus = function() {
      Scada.Viewport.disable();

      Ext.Ajax.request({
        url: Scada.reqUrl('scripting', 'status'),
        success: $E.toggleScriptResult,
        failure: $E.toggleScriptResult,
        params: Scada.encodeData({ id: $S.data.id })
      });
    };

    $E.toggleScriptBtn = new Ext.Button({
      handler: $E.toggleScriptStatus
    });

    $E.fbar.splice(2, 0, $E.toggleScriptBtn);
    $E.toggleScriptBtnUpdate($S.data.active);
  }

  Ext.each(['error', 'log'], function(mode) {
    var store, textfield, module, win, task, runner, gmax = 0;

    textfield = mode == 'log' ? 'log' : 'errortext';
    module = mode == 'log' ? 'logs' : 'errorlog';
    store = new Ext.data.JsonStore({
      url: Scada.reqUrl(module),
      root: 'data',
      fields: [ 'id', 'scriptname', textfield, mode + 'time' ],
      baseParams: { limit: 10, start: 0 }
    });

    win = new Ext.Window({
      title: $tr(mode + 's_window'),
      width: 600,
      height: 400,
      closable: true,
      animCollapse: false,
      collapsible: true,
      closeAction: 'hide',
      plain: true,
      layout: 'fit',
      shadow: false,
      html: '<div class="scada-log-code" id="scada-' + mode + '-wrap"><pre id="scada-' + mode + '-content"></pre></div>',
      border: false,
      bbar: [
        { xtype: 'checkbox', id: 'scada-' + mode + '-autoscroll', boxLabel: $tr(mode + 's_autoscroll'), checked: true },
        '->',
        { iconCls: 'icon-clear', text: $tr('utilities_reset_button'), handler: function() { Ext.get('scada-' + mode + '-content').dom.innerHTML = ''; } }
      ]
    });

    task = {
      run: function() {
        store.reload();
      },
      interval: 3000
    };

    runner = new Ext.util.TaskRunner();

    win.on('show', function() {
      runner.start(task);
    });

    win.on('hide', function() {
      runner.stop(task);
    });

    store.on('load', function(store, data) {
      var html = [], content = Ext.get('scada-' + mode + '-content'), cmax = 0;

      Ext.each(data, function(item) {
        var data = item.data, dt = Date.parseDate(data[ mode + 'time' ], 'U', true);
        if (data.id > gmax) {
          html.push('<b>' + data.scriptname + '</b> ' + dt.format(Scada.DateTimeLongFormat) + '\n');

          if (mode == 'log') {
            html.push(data.log);
          }
          else {
            html.push(Scada.renderError(data.errortext));
          }

          html.push('<hr>');
        }

        cmax = Math.max(data.id, cmax);
      });

      gmax = Math.max(gmax, cmax);
      if (gmax > cmax) {
        gmax = 0;
      }

      if (content && content.dom && html.length) {
        content.dom.innerHTML = html.join('') + content.dom.innerHTML;
        if (Ext.getCmp('scada-' + mode + '-autoscroll').checked) {
          Ext.get('scada-' + mode + '-wrap').dom.scrollTop = 0;
        }
      }
    });

    $E.fbar.unshift({
      text: $tr(mode + 's_window_show'),
      handler: function() {
        win.show();
      }
    });
  });
}

$E.containerItems = [
  $E.editorPanel,
  { name: 'id', xtype: 'hidden' },
  { name: 'script', xtype: 'hidden' },
  { name: 'scriptonly', xtype: 'hidden', value: true }
];

if ($E.css) {
  $E.reload = function() {
    window.location.reload();
  };

  $E.cssUploadFW = new ExtFormWindow({
    id: 'vis-css-upload',
    window: { title: $tr('visgraphics_css_upload'), width: 460, height: 180 },
    form: {
      url: Scada.genUrl('visgraphics-css'),
      fileUpload: true,
      items: [
        $def.fileUpload({ name: 'file', fieldLabel: $tr('visgraphics_file'), allowBlank: true }),
        $def.formInfoPanel('visgraphics_css_help')
      ]
    },
    events: { onFormSubmitComplete: $E.reload, onFormSubmitFailed: $E.reload }
  });

  $E.fbar.unshift({
    text: $tr('visgraphics_css_upload'),
    iconCls: 'icon-add',
    handler: function() {
      $E.cssUploadFW.show();
    }
  });
}
else {
  $E.containerItems.unshift({
    id: 'scripting-accordion',
    xtype: 'panel',
    layout: 'accordion',
    region: 'west',
    margins: '10 10 0 10',
    width: 270,
    items: $E.sidePanel
  });
}

$E.container = new Ext.FormPanel({
  url: Scada.reqUrl('scripting', 'save'),
  frame: false,
  border: false,
  ctCls: 'scada-fullscreen-form',
  layout: 'border',
  footerCfg: { region: 'south' },
  items: $E.containerItems,
  buttonAlign: 'left',
  fbar: $E.fbar
});

$E.submitForm = function() {
  $E.form.setValues({
    id: $E.script.id,
    script: $E.codeMirror.getValue()
  });

  $E.form.submit({
    mode: 'json',
    exclude: 'script',
    waitMsg: $tr('loading')
  });
};

$E.formActionComplete = function(form, action) {
  if (action.type == 'submit') {
    if ($E.close) {
      window.close();
    }
  }
};

$E.formActionFailed = function(form, action) {
  var error;
  $E.close = false;

  if (action.type == 'submit' && action.result.errors && action.result.errors.script) {
    error = action.result.errors.script;
    Ext.Msg.alert($tr('scripting_error_title'), $tr('scripting_error_script', error.substring(22) || ''));
  }
};

$E.resize = function() {
  if ($E.codeMirror) {
    setTimeout(function() {
      $E.codeMirror.refresh();
    }, 250);
  }
}

$E.checkSelection = function(script, start, end) {
  var s = '|>', e = '<|', lines = script.split('\n'), l, lstart, lend;

  if (script.indexOf(s) >= 0 && script.indexOf(e) >= 0) {
    for (l = 0; l < lines.length; l++) {
      var line = lines[ l ], ch;

      ch = line.indexOf(s);
      if (ch >= 0) {
        lstart = { line: l, ch: ch };
        line = line.substring(0, ch) + line.substring(ch + 2);
        lines[ l ] = line;
      }

      ch = line.indexOf(e);
      if (ch >= 0) {
        lend = { line: l, ch: ch };
        line = line.substring(0, ch) + line.substring(ch + 2);
        lines[ l ] = line;
      }
    }

    if (lstart && lend) {
      script = lines.join('\n');

      var sch = start.ch;

      start.line = start.line + lstart.line;
      if (lstart.line > 0) {
        sch = 0;
      }

      start.ch = sch + lstart.ch;

      end.line = start.line + (lend.line - lstart.line);
      if (lend.line > 0) {
        sch = 0;
      }

      end.ch = sch + lend.ch;
    }
  }
  else {
    start.line = start.line + lines.length;
    start.ch = 0;

    end.line = start.line;
    end.ch = 0;
  }

  return { script: script, start: start, end: end };
};

$E.addObjectName = function(script) {
  var id, obj;

  if (script.type == 'event' && window.opener) {
    id = Scada.encodeGroupAddress(script.params);
    obj = window.opener.$O.extObjStore.getExact('id', id);

    if (obj && obj.text) {
      script.name += ' (' + obj.text + ')';
    }
  }
};

$E.init = function() {
  var title = '', id, obj;
  if (!$S || !$S.success) {
    return;
  }

  $E.script = $S.data;

  $E.form = $E.container.getForm();
  $E.form.on('actioncomplete', $E.formActionComplete);
  $E.form.on('actionfailed', $E.formActionFailed);

  if ($E.script.type) {
    title = $tr('scripting_' + $E.script.type) + ': ';
    $E.addObjectName($E.script);
  }

  title += $E.script.name;
  $E.editorPanel.setTitle(title);
  document.title = title + ' - ' + document.title;

  $E.saveButton.on('click', $E.submitForm);
  $E.saveCloseButton.on('click', function() {
    $E.close = true;
    $E.submitForm();
  });
  $E.closeButton.on('click', function() {
    window.close();
  });

  if (!$E.css) {
    Ext.getCmp('scripting-helpers').on('click', function(node) {
      var attr = node.attributes;
      var cm = $E.codeMirror;

      if (attr && attr.leaf && attr.script && cm) {
        var start = cm.getCursor(true), end = cm.getCursor(false);
        var d = $E.checkSelection(attr.script, start, end);

        cm.replaceSelection(d.script + "\n");
        cm.setSelection(d.start, d.end);

        setTimeout(function() {
          cm.focus();
        }, 10);
      }
      else {
        node.toggle();
      }
    });
    Ext.getCmp('scripting-helpers').on('load', $E.resize);

    $E.editorAddNodeText = function(node) {
      var attr = node.attributes;

      if (attr && attr.leaf && (attr.text || attr.value)) {
        var value = (attr.value || attr.text).replace('"', '\\"', 'g');
        if (!attr.value) {
          value = '"' + value + '"';
        }
        $E.codeMirror.replaceSelection(value);
      }
    };

    Ext.getCmp('scripting-group-addresses').on('click', $E.editorAddNodeText);
    Ext.getCmp('scripting-objects').on('click', $E.editorAddNodeText);
    Ext.getCmp('scripting-tags').on('click', $E.editorAddNodeText);
    Ext.getCmp('scripting-datatypes').on('click', $E.editorAddNodeText);

    var scripts = <? lua /lib/genohm-scada/web/scripting/main.lua ?>, store, data = [];

    store = {
      'event': [],
      'resident': [],
      'scheduled': [],
      'user': []
    };

    Ext.each(scripts.data, function(item) {
      if (store[ item.type ]) {
        $E.addObjectName(item);

        store[ item.type ].push({
          id: item.id,
          text: item.name,
          leaf: true
        });
      }
    });

    Ext.iterate(store, function(type, items) {
      if (items.length) {
        data.push({
          text: $tr('scripting_' + type),
          cls: 'folder',
          expanded: false,
          children: items
        });
      }
    });

    data.push({
      id: 'userlib',
      text: $tr('scripting_tools_userlib'),
      leaf: true
    });

    data.push({
      id: 'initscript',
      text: $tr('scripting_tools_initscript'),
      leaf: true
    });

    Ext.getCmp('scripting-accordion').add($def.tree({
      border: false,
      title: $tr('scripting_tree_title'),
      id: 'scripting-list',
      root: {
        expanded: true,
        children: data
      }
    }));

    Ext.getCmp('scripting-list').on('click', function(node) {
      if (node.leaf) {
        window.location = window.location.pathname + '?id=' + node.attributes.id;
      }
      else {
        node.toggle();
      }
    });

    $E.reloadParent();
  }

  window.onresize = $E.resize;
};

Scada.Modules.push($E);
