var Scada = { Modules: [] };

Scada.init = function() {
  var tip, langSelect, neighbourSelect;

  Scada.MainPanel = new Ext.TabPanel({
    ctCls: 'scada-main-panel',
    border: false,
    flex: 1
  });

  langSelect = [
    '<div id="language-select-wrap" class="topbar-select-wrap hidden">',
      '<label for="language-select">' + $tr('language') + '</label>',
      '<select id="language-select"></select>',
    '</div>'
  ].join('');

  neighbourSelect = [
    '<div id="neighbour-select-wrap" class="topbar-select-wrap hidden">',
      '<label for="neighbour-select">' + $tr('neighbour') + '</label>',
      '<select id="neighbour-select"><option value="">' + $tr('neighbour_select') + '</option></select>',
    '</div>'
  ].join('');

  Scada.Viewport = new Ext.Viewport({
    layout: 'vbox',
    layoutConfig: {
      align : 'stretch'
    },
    items: [
      {
        html: '<h1 class="x-panel-header" id="scada-header"><b>' + document.title + '</b> <a id="start-link" href="/">' + $tr('start_page') + '</a>' + langSelect + neighbourSelect + '</h1>',
        height: 26,
        border: false,
        margins: '0 0 5 0'
      },
      Scada.MainPanel,
      {
        html: '<h1 class="x-panel-header" id="scada-footer"><span id="scada-version">' + $tr('version', LMVersion) + '</span><span id="scada-usage">&nbsp;</span></h1>',
        height: 26,
        border: false
      }
    ]
  });

  Ext.QuickTips.init();
  tip = Ext.QuickTips.getQuickTip();
  Ext.apply(tip, {
    maxWidth: 500,
    minWidth: 100,
    showDelay: 100,
    dismissDelay: 0,
    trackMouse: false
  });
  tip.render();
  tip.el.disableShadow();

  Ext.Ajax.timeout = 90 * 1000;
  Ext.form.Field.prototype.msgTarget = 'side';

  Scada.initModules();

  if (Scada.warn) {
    Ext.Msg.show({
       title: $tr('password_warn_title'),
       msg: $tr('password_warn_msg'),
       buttons: Ext.Msg.OK,
       icon: Ext.MessageBox.WARNING
    });
  }
};

Scada.reqUrl = function(module, action) {
  return '/cgi-bin/' + (Scada.baseMode || 'scada') + '/general/request.cgi?_module=' + module + '&_action=' + (action || 'main');
};

Scada.genUrl = function(action) {
  return '/cgi-bin/' + (Scada.baseMode || 'scada') + '/general/' + action + '.cgi';
};

Scada.encodeData = function(data) {
  var fn = typeof Ext === "undefined" ? JSON.stringify : Ext.util.JSON.encode;
  return { data: fn(data) };
};

Scada.decodeJson = function(str) {
  return typeof Ext === "undefined" ? JSON.parse(str) : Ext.util.JSON.decode(str);
};

Scada.initModules = function() {
  var i, j, module;

  for (i = 0, j = Scada.Modules.length; i < j; i++) {
    module = Scada.Modules[i];
    if (module.container) {
      try {
        if (typeof module.index != 'undefined') {
          Scada.MainPanel.insert(module.index, module.container);
        }
        else {
          Scada.MainPanel.add(module.container);
        }
      }
      catch (e) {}
    }

    if (typeof module.init == 'function') {
      try {
        module.init();
      }
      catch (e) {}
    }
  }

  Scada.taskRunner = new Ext.util.TaskRunner();

  Scada.MainPanel.on('tabchange', function(x, module) {
    var sm = Scada[ module.id ];

    if (Scada.currentModule && Scada.currentModule.onBlur) {
      Scada.currentModule.onBlur();
    }

    if (sm) {
      if (Scada.currentTask) {
        Scada.taskRunner.stop(Scada.currentTask);
      }

      Scada.currentTask = sm.reloaderTask;

      if (Scada.currentTask) {
        Scada.taskRunner.start(Scada.currentTask);
      }
      else if (sm.onFocus) {
        sm.onFocus();
      }

      Scada.currentModule = sm;
    }
  });

  Scada.MainPanel.setActiveTab(Scada.activeTab || 0);

  Scada.MainPanel.doLayout();
  Scada.Viewport.doLayout();
};

Scada.decodeIndAddress = function(ind) {
  var i = ind.toNumber();
  return [ (i >> 12) & 0x0F, (i >> 8) & 0x0F, i & 0xFF ].join('.');
};

Scada.decodeGroupAddress = function(group) {
  var g = group.toNumber();
  return [ (g >> 11) & 0x0F, (g >> 8) & 0x07, g & 0xFF ].join('/');
};

Scada.encodeIndAddress = function(ind) {
  var i = ind.split('.'), x, y, z;

  if (i.length == 3) {
    x = i[0].toNumber();
    y = i[1].toNumber();
    z = i[2].toNumber();

    if (isNaN(x) || isNaN(y) || isNaN(z)) {
      return false;
    }
    else {
      return ((x & 0x0F) << 12) | ((y & 0x0F) << 8) | (z & 0xFF);
    }
  }
  else {
    return false;
  }
};

Scada.encodeGroupAddress = function(group) {
  var g = group.split('/'), x, y, z;

  if (g.length == 2) {
    x = g[0].toNumber();
    y = g[1].toNumber();

    if (isNaN(x) || isNaN(y)) {
      return false;
    }
    else {
      return ((x & 0x0F) << 11) | (z & 0x7FF);
    }
  }
  else if (g.length == 3) {
    x = g[0].toNumber();
    y = g[1].toNumber();
    z = g[2].toNumber();

    if (isNaN(x) || isNaN(y) || isNaN(z)) {
      return false;
    }
    else {
      return ((x & 0x0F) << 11) | ((y & 0x07) << 8) | (z & 0xFF);
    }
  }
  else {
    return false;
  }
};

Scada.dt = {
  'bool': 1, 'bit2': 2, 'bit4': 3, 'char': 4, 'uint8': 5, 'int8': 6, 'uint16': 7, 'int16': 8,
  'float16': 9, 'time': 10, 'date': 11, 'uint32': 12, 'int32': 13, 'float32': 14, 'access': 15, 'string': 16,
  'uint24': 254
};

Scada.minorDt = { 'scale': 1, 'angle': 3, 'temperature': 1, 'dimstep': 7, 'hex': 2 };

Scada.decodeEnums = function(value, enums) {
  if (!enums) {
    return;
  }

  if (typeof enums == 'string') {
    enums = Scada.decodeJson(enums);
  }

  if (typeof enums != 'object') {
    return;
  }

  if (typeof value == 'boolean') {
    value = value ? 1 : 0;
  }
  else {
    value = parseInt(value, 10);
  }

  if (isNaN(value)) {
    return;
  }

  return enums[ value ] || enums['default'];
};

Scada.decodeValue = function(value, rawdt, units, enums) {
  var vt = typeof value, dt = Scada.normalizeDt(rawdt), result, enumvalue, i;

  if (Scada.dtIsNumeric(dt.major) && !value) {
    value = 0;
  }

  enumvalue = Scada.decodeEnums(value, enums);
  if (typeof enumvalue != 'undefined') {
    return enumvalue;
  }

  switch (dt.major) {
    case Scada.dt.bool:
      if (vt == 'string') {
        value = value.toNumber();
      }
      result = $tr('dt_1_' + dt.minor + '_' + (value ? 1 : 0));
    break;
    case Scada.dt.bit2:
      result = (value > 1 ? $tr('dt_control') : $tr('dt_no_control')) + ', ' + (value & 1);
    break;
    case Scada.dt.bit4:
      var status = value & 0x07;

      if (dt.minor == Scada.minorDt.dimstep) {
        if (status) {
          units = units || '%';
          result = $tr((value & 0x08) ? 'dt_up' : 'dt_down') + ' ' + ([100, 50, 25, 12, 6, 3, 1])[ status - 1 ];
        }
        else {
          result = $tr('dt_stop');
        }
      }
      else {
        status = status > 0 ? $tr('dt_step') + ' ' + status : $tr('dt_break');
        result = ((value & 0x08) ? $tr('dt_control') : $tr('dt_no_control')) + ', ' + status;
      }
    break;
    case Scada.dt.uint8:
      if (dt.minor == Scada.minorDt.scale) {
        units = units || '%';
      }
      else if (dt.minor == Scada.minorDt.angle) {
        units = units || '°';
      }

      result = value;
    break;
    case Scada.dt.time:
      var days = ['', 'dt_monday', 'dt_tuesday', 'dt_wednesday', 'dt_thursday', 'dt_friday', 'dt_saturday', 'dt_sunday'];

      if (value.minute < 10) {
        value.minute = '0' + value.minute.toNumber();
      }
      if (value.second < 10) {
        value.second = '0' + value.second.toNumber();
      }

      result = value.hour + ':' + value.minute + ':' + value.second + (value.day > 0 ? ', ' + $tr(days[ value.day ]) : '');
    break;
    case Scada.dt.date:
      if (value.day < 10) {
        value.day = '0' + value.day.toNumber();
      }
      if (value.month < 10) {
        value.month = '0' + value.month.toNumber();
      }

      result = value.day + '.' + value.month + '.' + value.year;
    break;
    case Scada.dt.string:
      result = '';

      for (i = 0; i < value.length; i++) {
        var ch = value.charCodeAt(i);

        if (dt.minor == Scada.minorDt.hex) {
          ch = ch.toString(16).toUpperCase();
          result += (ch.length == 1 ? '0' : '') + ch;
        }
        else if (ch >= 31 && ch < 127) {
          result += value.charAt(i);
        }
      }

      result = result.trim();
    break;
    default:
      result = vt == 'number' ? value.toString() : value;
    break;
  }

  return result + (units || '');
};

Scada.dtIsNumeric = function(dt) {
  var d = Scada.dt;
  return isNaN(dt) ? false : (
    dt == d.bit2    || dt == d.access ||
    dt == d.uint8   || dt == d.int8   ||
    dt == d.uint16  || dt == d.int16  ||
    dt == d.uint24  ||
    dt == d.uint32  || dt == d.int32  ||
    dt == d.float16 || dt == d.float32
  );
};

Scada.dtGetBounds = function(rawdt, params) {
  var dt = typeof rawdt == 'object' ? rawdt : Scada.normalizeDt(rawdt), bounds;

  switch (dt.major) {
    case Scada.dt.bool:
      bounds = [0, 1];
    break;
    case Scada.dt.bit2:
      bounds = [0, 3];
    break;
    case Scada.dt.bit4:
      bounds = [0, 15];
    break;
    /*
    case Scada.dt.char:
      bounds = [0, 255];
    break;
    */
    case Scada.dt.uint8:
      if (dt.minor == Scada.minorDt.scale) {
        bounds = [0, 100];
      }
      else if (dt.minor == Scada.minorDt.angle) {
        bounds = [0, 360];
      }
      else {
        bounds = [0, 255];
      }
    break;
    case Scada.dt.int8:
      bounds = [-128, 127];
    break;
    case Scada.dt.uint16:
      bounds = [0, 65535];
    break;
    case Scada.dt.int16:
      bounds = [-32768, 32767];
    break;
    case Scada.dt.float16:
      bounds = (dt.minor == Scada.minorDt.temperature) ? [-273, 670760] : [-670760, 670760];
    break;
    case Scada.dt.uint24:
      bounds = [0, 16777215];
    break;
    case Scada.dt.access:
    case Scada.dt.uint32:
      bounds = [0, 4294967295];
    break;
    case Scada.dt.int32:
      bounds = [-2147483648, 2147483647];
    break;
    case Scada.dt.float32:
      bounds = [-2147483648, 2147483647];
    break;
  }

  if (params) {
    var min = parseFloat(params.min), max = parseFloat(params.max);

    if (!isNaN(min)) {
      bounds[ 0 ] = Math.max(bounds[ 0 ], min);
    }
    if (!isNaN(max)) {
      bounds[ 1 ] = Math.min(bounds[ 1 ], max);
    }

    if (bounds[ 0 ] > bounds[ 1 ]) {
      bounds[ 0 ] = bounds[ 1 ];
    }
  }

  return bounds;
};

Scada.normalizeDt = function(major) {
  var minor = 0, mod;

  if (typeof major == 'string') {
    major = major.toNumber();
  }

  mod = major % 1;

  if (major >= 1000) {
    minor = major % 1000;
    major = Math.floor(major / 1000);
  }
  else if (mod) {
    minor = Math.floor(mod * 1000);
    major = Math.floor(major);
  }

  return { major: major, minor: minor };
};

Scada.findFreeAddress = function(store) {
  var i, addr, res = '', start = Config.autorange || '1/1/1';
  start = Scada.encodeGroupAddress(start);

  store = store || [];

  for (i = 0; i < 256 * 7; i++) {
    addr = start + i;
    if (!$O.store.getById(addr) && store.indexOf(addr) == -1) {
      res = addr;
      break;
    }
  }

  return res;
};

Scada.renderError = function(value) {
  var s = value.indexOf(':'), script, text;

  if (s > 0 && value.match(/:[0-9]+:/)) {
    script = value.split(':').shift();
    text = Ext.util.Format.htmlEncode(value.substring(s + 1));
    text = $tr('errorlog_error', text);

    if (script.indexOf('/usr/share/lua/') === 0) {
      script = script.replace('/usr/share/lua/', '').replace(/\.luas?/, '');

      if (script.indexOf('user/') === 0) {
        script = script.replace('user/', '');
        text = $tr('scripting_user_form') + ' "' + script + '", ' + text;
      }
      else if (script.indexOf('userlib') > 0) {
        text = $tr('scripting_tools_userlib') + ', ' + text;
      }
    }

    return text;
  }
  else {
    return Ext.util.Format.htmlEncode(value);
  }
};

Scada.Lang = 'en';

Scada.DateTimeLongFormat = 'd.m.Y H:i:s';
Scada.DateTimeFormat = 'd.m.Y H:i';
Scada.DateFormat = 'd.m.Y';

$tr = function() {
  if (arguments.length == 0) {
    return null;
  }

  var key = arguments[0];
  if (Translations[key]) {
    var result = Translations[key];
    for (var i = 0, j = arguments.length - 1; i < j; i++) {
      result = result.replace(new RegExp('\\{' + i + '\\}', 'g'), arguments[i + 1]);
    }
    return result;
  }

  return key;
};

String.prototype.toNumber = function(){
  var r = parseInt(this, 10);
  return isNaN(r) ? null : r;
};

Number.prototype.toNumber = function(){
  return (0 + this);
};

$compare = function(a, b) {
  var ta = typeof a, tb = typeof b, prop;

  if (ta === tb) {
    if (ta === 'object') {
      for (prop in a) {
        if (a[ prop ] !== b[ prop ]) {
          return false;
        }
      }

      for (prop in b) {
        if (a[ prop ] !== b[ prop ]) {
          return false;
        }
      }

      return true;
    }
    else {
      return (a === b);
    }
  }
  else {
    return false;
  }
};

Config = (typeof Config === "undefined") ? {} : Config;
Config.perpage = (Config.perpage && Config.perpage.toNumber()) || 25;
