Ext.grid.BoolClsColumn = Ext.extend(Ext.grid.Column, {
  trueCls: 'x-grid3-cell-bool-true',
  falseCls: 'x-grid3-cell-bool-false',

  constructor: function(cfg) {
    Ext.grid.BoolClsColumn.superclass.constructor.call(this, cfg);
    this.renderer = function(value, column, record) {
      if (this.getValue) {
        value = this.getValue(record);
      }

      if (typeof value == 'string') {
        value = value.toNumber();
      }

      value = !!value;
      column.css += (value ? this.trueCls : this.falseCls) + ' ';
    };
  }
});

Ext.grid.TranslateColumn = Ext.extend(Ext.grid.Column, {
  constructor: function(cfg) {
    Ext.grid.TranslateColumn.superclass.constructor.call(this, cfg);
    this.renderer = function(v, c) {
      if (typeof this.trPrefix == 'string') {
        v = this.trPrefix + '_' + v;
      }

      return $tr(v);
    };
  }
});

Ext.grid.KNXdtColumn = Ext.extend(Ext.grid.Column, {
  constructor: function(cfg) {
    Ext.grid.KNXdtColumn.superclass.constructor.call(this, cfg);
    this.renderer = function(value, column, record) {
      var val, data = record.data;
      if (data.datatype && data.eventtype != 'read') {
        val = Scada.decodeValue(value, data.datatype.toNumber(), data.units, data.enums);
      }
      else {
        val = '&mdash;';
      }

      return val;
    };
  }
});

Ext.grid.TimeStampColumn = Ext.extend(Ext.grid.Column, {
  constructor: function(cfg) {
    Ext.grid.TimeStampColumn.superclass.constructor.call(this, cfg);
    this.renderer = function(value) {
      var ts = value.toNumber();
      value = null;

      if (ts !== null) {
        var dt = Date.parseDate(ts, 'U', true);
        if (dt) {
          value = dt.format(Scada.DateTimeLongFormat);
        }
      }

      return value || '&mdash;';
    }
  }
});

Ext.grid.Column.types.boolclscolumn = Ext.grid.BoolClsColumn
Ext.grid.Column.types.translatecolumn = Ext.grid.TranslateColumn
Ext.grid.Column.types.knxdtcolumn = Ext.grid.KNXdtColumn
Ext.grid.Column.types.timestampcolumn = Ext.grid.TimeStampColumn

Ext.MessageBox.buttonText.yes = Translations.btn_yes;
Ext.MessageBox.buttonText.no = Translations.btn_no;

Ext.form.Action.Submit.prototype.run = function() {
  var o = this.options;
  var method = this.getMethod();
  var isGet = method == 'GET';

  if (o.clientValidation === false || this.form.isValid()) {
    var baseParams = {
      url: this.getUrl(isGet),
      method: method,
      headers: o.headers,
      params: (!isGet ? this.getParams() : null),
      isUpload: this.form.fileUpload
    }

    if (o.mode == 'json') {
      var vals = this.form.getValues(), exclude;

      if (o.exclude && vals[ o.exclude ]) {
        var exclude = vals[ o.exclude ];
        vals[ o.exclude ] = null;
      }

      baseParams.params = Scada.encodeData(vals);

      if (exclude) {
        baseParams.params[ o.exclude ] = exclude;
      }
    }
    else {
      baseParams.form = this.form.el.dom;
    }

    Ext.Ajax.request(
      Ext.apply(this.createCallback(o), baseParams)
    );
  }
  else if (o.clientValidation !== false) {
    this.failureType = Ext.form.Action.CLIENT_INVALID;
    this.form.afterAction(this, false);
  }
}

Ext.form.MenuTriggerField = Ext.extend(Ext.form.TriggerField, {
  menuAlign: 'tl-bl?',
  validationEvent: false,

  initComponent: function() {
    Ext.form.MenuTriggerField.superclass.initComponent.call(this);
    if (this.menu) {
      this.rawValues = [];
      this.mapping = {};

      this.menu = Ext.menu.MenuMgr.get(this.menu);
      this.mon(this.menu, {
        scope: this,
        hide: this.onMenuHide
      });

      this.triggerBlocked = false;
      this.menu.items.each(function(item) {
        if (item.value !== undefined) {
          this.mapping[ item.value ] = item.fieldText || item.text;
        }
      }, this);
    }
  },
  restoreClick : function() {
    this.triggerBlocked = false;
  },
  onMenuHide: function() {
    this.restoreClick.defer(250, this);

    this.rawValues = [];

    this.menu.items.each(function(item) {
      if (item.checked && item.value !== undefined) {
        this.rawValues.push(item.value);
      }
    }, this);

    this.setFieldValue();
  },
  beforeDestroy: function() {
    if (this.menu) {
      this.menu.destroy();
    }
  },
  onTriggerClick: function() {
    if (this.menu && !this.triggerBlocked) {
      this.triggerBlocked = true;
      this.menu.show(this.el, this.menuAlign);
    }
  },
  // set raw value with later mapping
  setValue: function(v) {
    if (v) {
      this.rawValues = v.split(',');
      this.setFieldValue();
    }
  },
  // get raw values as string
  getValue: function() {
    return this.rawValues.join(',');
  },
  // set text field value based on selected items
  setFieldValue: function() {
    if (!this.el) {
      return;
    }

    // set all text if all values are checked
    if (this.rawValues.length == this.menu.items.length && this.allText) {
      this.el.dom.value = this.allText;
    }
    // create joined text values
    else {
      var text = [];

      Ext.each(this.rawValues, function(id) {
        if (this.mapping[ id ]) {
          text.push(this.mapping[ id ]);
        }
      }, this);

      this.el.dom.value = text.join(', ');
    }

    // set correct checked value
    this.menu.items.each(function(item) {
      item.setChecked(this.rawValues.indexOf(item.value) != -1);
    }, this);
  }
});

Ext.reg('menutriggerfield', Ext.form.MenuTriggerField);

// combobox with muptiple delimeter-separated values
Ext.form.TagBox = Ext.extend(Ext.form.ComboBox, {
  typeAhead: false, // defaults
  hideTrigger: true,
  triggerAction: 'all',
  mode: 'local',
  autoAdd: true, // add new item automatically
  autoClean: true, // clean duplicate values
  delimeter: ',', // delimeter for tag separation
  fullDelimeter: ', ', // full delimeter with padding
  // constructor
  initComponent: function() {
    Ext.form.TagBox.superclass.initComponent.call(this);
    // override events
    this.on('beforequery', this.onBeforequery, this);
    this.on('specialkey', this.onSpecialkey, this);
  },
  // get trimmed chunks
  getChunks: function() {
    var c = this.getRawValue().split(this.delimeter);
    for (var i = 0, j = c.length; i < j; i++) {
      c[ i ] = c[ i ].trim();
    }
    return c;
  },
  onSelect: function(record, index) {
    if (this.fireEvent('beforeselect', this, record, index) !== false) {
      var value = record.data[this.valueField || this.displayField];
      var chunks = this.getChunks();
      // append value with extra padding if needed
      chunks[ chunks.length - 1 ] = value.trim();
      if (this.autoAdd) {
        chunks.push('');
      }

      var v = chunks.join(this.fullDelimeter);
      this.updateValue(v);
      this.collapse();
      this.fireEvent('select', this, record, index);
    }
  },
  onLoad: function() {
    if (!this.hasFocus) {
      return;
    }
    if (this.store.getCount() > 0) {
      this.expand();
      this.restrictHeight();
      if (this.lastQuery == this.allQuery) {
        if (!this.selectByValue(this.value, true)) {
          this.select(0, true);
        }
      }
      else {
        this.selectNext();
      }
    }
    else {
      this.collapse();
    }
  },
  updateValue: function(v) {
    if (this.autoClean) {
      var c = v.split(this.fullDelimeter);
      var n = [];
      for (var i = 0, j = c.length; i < j; i++) {
        if (n.indexOf(c[ i ]) == -1) {
          n.push(c[ i ]);
        }
      }
      v = n.join(this.fullDelimeter);
    }

    if (this.hiddenField) {
      this.hiddenField.value = v;
    }

    this.value = v;
    Ext.form.ComboBox.superclass.setValue.call(this, v);
  },
  updateTags: function(v) {
    // create store if tags were passed
    if (v.tags) {
      var tags = [], i, j, value;
      for (i = 0, j = v.tags.length; i < j; i++) {
        tags.push([v.tags[ i ].tag]);
      }

      if (this.store) {
        this.store.loadData(tags);
      }
      else {
        this.store = new Ext.data.ArrayStore({
          autoDestroy: true,
          mode: 'local',
          fields: ['tag'],
          data: tags
        });
      }

      this.valueField = 'tag';
      this.displayField = 'tag';
    }
  },
  setValue: function(v) {
    var value, tags;

    if (typeof v == 'object') {
      this.updateTags(v);

      // convert value to string
      if (v.value) {
        tags = [];
        for (i = 0, j = v.value.length; i < j; i++) {
          tags.push(v.value[ i ].tag);
        }
        tags.push('');
        value = tags.join(this.fullDelimeter);
      }
      else {
        value = '';
      }

      Ext.form.TagBox.superclass.setValue.call(this, value);
    }
    else {
      Ext.form.TagBox.superclass.setValue.call(this, v);
    }

    return this;
  },
  onBeforequery: function(cbEvent) {
    var chunks = this.getChunks();
    cbEvent.query = chunks[ chunks.length - 1 ];
  },
  onSpecialkey: function(tagBox, cbEvent) {
    if (this.autoAdd && cbEvent.getKey() == cbEvent.ENTER) {
      var chunks = this.getChunks();
      if (chunks[ chunks.length - 1 ]) {
        chunks.push('');
        var v = chunks.join(this.fullDelimeter);
        this.updateValue(v);
        this.collapse();
      }
    }
  },
  reset: function() {
    Ext.form.TagBox.superclass.setValue.call(this, '');
  }
});

Ext.reg('tagbox', Ext.form.TagBox);

// custom validation schemas
var knxgaTest = /^((1[0-5]|[0-9])\/(([0-7]\/(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9]))|(204[0-7]|20[0-3][0-9]|1[0-9]{3}|[1-9][0-9]{2}|[1-9]?[0-9])))$/;
var knxiaTest = /^(1[0-5]|[0-9])\.(1[0-5]|[0-9])\.(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])$/

// validate single cron parameter
var cronValidate = function(value, min, max) {
  value = value.trim();

  // empty value
  if (value === '') {
    return false;
  }

  // match every
  if (value === '*') {
    return true;
  }

  // match when the modulo is zero
  if (value[0] === '*') {
    var values = value.split('/');
    // must have exactly 2 parameters, first one must be single '*'
    if (values.length != 2 || values[0] !== '*') {
      return false;
    }

    // divisor must be a number between 2 and max-1
    var divisor = values[1].toNumber();
    return (divisor !== null && divisor > 1 && divisor < max);
  }

  // split by ',' and validate chunks
  var values = value.split(','), i, j;
  for (i = 0, j = values.length; i < j; i++) {
    var chunk = values[i];

    // validate range
    if (chunk.indexOf('-') > 0) {
      var numValues = chunk.split('-');
      // range can have only 2 values
      if (numValues.length != 2) {
        return false;
      }

      var numValueL = numValues[0].toNumber();
      var numValueH = numValues[1].toNumber();
      // both values must be a number between min and max, a < b
      if (numValueL === null || numValueH === null ||
          numValueL >= numValueH || numValueL < min ||
          numValueL > max || numValueH > max) {
        return false;
      }
    }
    // validate single value
    else {
      var numValue = chunk.toNumber();
      // must be a valid number between min and max
      if (numValue === null || numValue < min || numValue > max) {
        return false;
      }
    }
  }

  // previous loop wil return if value is invalid
  return true;
}

Ext.apply(Ext.form.VTypes, {
  // knx group address
  knxga: function(v) { return (v != '0/0' && v != '0/0/0' && knxgaTest.test(v)); },
  knxgaText: $tr('validation_knxga_error'),
  knxgaMask: /[\d\/]/,
  // knx group address + name
  knxgaName: function(v) { v = v.split(' ')[ 0 ]; return (v != '0/0' && v != '0/0/0' && knxgaTest.test(v)); },
  knxgaNameText: $tr('validation_knxga_error'),
  knxgaNameMask: /[\d\/]/,
  // knx ind address
  knxia: function(v) { return (v != '0.0.0' && knxiaTest.test(v)); },
  knxiaText: $tr('validation_knxia_error'),
  knxiaMask: /[\d\.]/,
  // cron minute
  cronMinute: function(v) { return cronValidate(v, 0, 59); },
  cronMinuteText: $tr('validation_cron_error'),
  cronMinuteMask: /[-,0-9\*\/]/,
  // cron hour
  cronHour: function(v) { return cronValidate(v, 0, 23); },
  cronHourText: $tr('validation_cron_error'),
  cronHourMask: /[-,0-9\*\/]/,
  // cron day
  cronDay: function(v) { return cronValidate(v, 1, 31); },
  cronDayText: $tr('validation_cron_error'),
  cronDayMask: /[-,0-9\*\/]/
});

// make the refresh button of the paging toolbar do a forced reload
Ext.override(Ext.PagingToolbar, {
  doRefresh: function() {
    delete this.store.lastParams;
    this.doLoad(this.cursor);
  }
});

Ext.override(Ext.grid.GridView, { onLoad: Ext.emptyFn });

$extend = function(o, d, v) {
  var k;
  // extend with default if not defined
  for (k in d) {
    if (d.hasOwnProperty(k)) {
      if (o[ k ] === undefined || v) {
        o[ k ] = d [ k ]
      }
    }
  }

  return o;
}

$clone = function(o) {
  var c = {}, k, v;
  for (k in o) {
    if (o.hasOwnProperty(k)) {
      v = o[ k ];
      c[ k ] = typeof(v) == 'object' ? $clone(v) : v
    }
  }

  return c;
}

// custom defaults
$def = {};

// Ext.Window
$def.window = function(o) {
  var d = {
    closable: true,
    width: 640,
    height: 480,
    closeAction: 'hide',
    draggable: false,
    resizable: false,
    modal: true,
    plain: true,
    layout: 'fit',
    shadow: false
  }
  return $extend(o, d);
}

// Ext.grid.GridPanel
$def.grid = function(o) {
  var d = {
    region: 'center',
    border: true,
    enableColumnHide: false,
    enableColumnMove: false,
    margins: '10 10 10 10',
    viewConfig: { forceFit: true },
    ctCls: 'scada-list-view-grid',
    sm: new Ext.grid.RowSelectionModel({ singleSelect: true })
  }
  return $extend(o, d);
}

Ext.grid.ColumnModel.prototype.defaults = {
  width: .1,
  sortable: true,
  menuDisabled: true
};

// Ext.Form inside Ext.Window
$def.form = function(o) {
  var d = {
    xtype: 'form',
    layout: 'form',
    frame: false,
    border: false,
    labelWidth: 150,
    defaults: { width: 250 },
    defaultType: 'textfield',
    ctCls: 'scada-window-form'
  }
  return $extend(o, d);
}

// Ext.ComboBox as standard select
$def.select = function(o) {
  var d = {
    xtype: 'combo',
    forceSelection: true,
    editable: false,
    triggerAction: 'all'
  }
  return $extend(o, d);
}

$def.formInfoPanel = function() {
  return {
    xtype: 'panel',
    autoHeight: true,
    autoWidth: true,
    baseCls: 'infopanel',
    html: $tr.apply(this, arguments)
  }
}

$def.fieldset = function(o) {
  var d = {
    margins: 0,
    xtype: 'fieldset',
    autoWidth: true,
    collapsible: false,
    autoHeight: true,
    frame: false,
    border: false
  }
  return $extend(o, d);
}

$def.formButtons = function(formId) {
  return [
    { text: $tr('save'), id: (formId + '-submit') },
    { text: $tr('cancel'), id: (formId + '-cancel') }
  ];
}

$def.tree = function(o) {
  var d = {
    xtype: 'treepanel',
    margins: '10 10 10 10',
    cmargins: '10 10 10 10',
    width: 250,
    bodyStyle: 'overflow-y: auto;',
    titleCollapse: true,
    animate: false,
    enableDD: false,
    animCollapse: false,
    rootVisible: false,
    root: { nodeType: 'async' }
  }
  return $extend(o, d);
}

$def.fileUpload = function(o) {
  var d = { inputType: 'file', allowBlank: false, name: 'file' }
  return $extend(o, d);
}

ExtFormWindow = function(config) {
  var o = {}, formConfig = config.form, windowConfig = config.window;

  // custom user event bindings
  o.events = {
    onShow: Ext.emptyFn,
    onHide: Ext.emptyFn,
    onFormSubmitComplete: Ext.emptyFn,
    onFormSubmitFailed: Ext.emptyFn,
    onFormLoadComplete: Ext.emptyFn,
    onFormLoadFailed: Ext.emptyFn
  }

  if (config.events) {
    $extend(o.events, config.events, true);
  }

  // set defaults
  o.id = config.id;
  formConfig.id = o.id;
  if (!formConfig.buttons) {
    formConfig.buttons = $def.formButtons(formConfig.id);
  }
  o.config = config;

  // data to load
  o.loadParams = '-';

  // show window helper
  o.show = function() {
    this.events.onShow(this);

    if (this.ready) {
      this.form.reset();
    }

    this.window.show().center();

    if (this.config.form.loadUrl) {
      this.form.load({
        url: this.config.form.loadUrl,
        params: this.config.getLoadParams ? this.config.getLoadParams() : this.loadParams,
        waitMsg: $tr('loading')
      })
    }
  }

  // set load params
  o.setLoadParams = function(params) {
    this.loadParams = params ? Scada.encodeData(params) : '-';
    return this;
  }

  // hide window helper
  o.hide = function() {
    this.events.onHide(this);
    this.window.hide();
  }

  // submit form helper
  o.submit = function() {
    var res;

    if (this.events.onBeforeSubmit) {
      res = this.events.onBeforeSubmit(this);
    }

    if (res !== false && this.form.isValid()) {
      if (this.form.fileUpload) {
        this.form.submit({ waitMsg: $tr('loading') });
      }
      else if (this.events.onSubmit) {
        this.events.onSubmit(this);
      }
      else {
        this.form.submit({ mode: 'json', waitMsg: $tr('loading') });
      }
    }
  }

  // form action (load or submit complete)
  o.formActionComplete = function(form, action) {
    if (action.type == 'load') {
      this.events.onFormLoadComplete(this, action, form);
    }
    else if (action.type == 'submit') {
      this.events.onFormSubmitComplete(this, action, form);
      this.hide();
    }
  }

  // form action failed (server-side load or validation)
  o.formActionFailed = function(form, action) {
    if (action.type == 'load') {
      this.events.onFormLoadFailed(this, action);
    }
    else if (action.type == 'submit') {
      this.events.onFormSubmitFailed(this, action);
    }
  }

  o.init = function() {
    if (!this.ready) {
      this.form = this.formPanel.getForm();

      this.submitBtn = Ext.getCmp(this.id + '-submit');
      this.cancelBtn = Ext.getCmp(this.id + '-cancel');

      this.submitBtn.setHandler(this.submit, this);
      this.cancelBtn.setHandler(this.hide, this);

      this.form.on('actioncomplete', this.formActionComplete, this);
      this.form.on('actionfailed', this.formActionFailed, this);

      this.ready = true;
    }
  }

  // create form and window instances
  o.formPanel = new Ext.form.FormPanel($def.form( formConfig ));
  windowConfig.items = o.formPanel;
  o.window = new Ext.Window($def.window( windowConfig ));

  o.window.on('afterrender', o.init, o);

  return o;
}

Scada.initImgSelect = function(module, mode) {
  var store, path, name, view, win, selected, callback;

  if (mode == 'icon') {
    store = Scada.Vis.iconStore;
    path = 'icons';
    name = 'icon';
    callback = 'showIconSelect';
  }
  else {
    store = Scada.Vis.imgStore;
    path = 'img';
    name = 'image';
    callback = 'showImgSelect';
  }

  view = new Ext.DataView({
    autoScroll: true,
    cls: 'scada-icon-grid',
    store: store,
    tpl: new Ext.XTemplate(
      '<tpl for=".">',
        '<div class="scada-icon-wrap scada-icon-selectable" data-id="{' + name + '}">',
            '<div class="scada-icon">',
              '<img src="/scada/resources/' + path  + '/{' + name + '}" title="{' + name + '}">',
            '</div>',
          '<div class="scada-icon-title">{' + name + '}</div>',
        '</div>',
      '</tpl>',
      '<div class="x-clear"></div>'
    )
  });

  win = new Ext.Window($def.window({
    title: $tr('vis_select_image'),
    width: 670,
    height: 410,
    items: view
  }));

  win.on('hide', function() {
    if (!selected) {
      module.imgSelectTrigger.setValue('');
      module.imgSelectTrigger.fireEvent('change');
    }

    selected = false;
  });

  view.on('click', function(view, index, el) {
    var img;
    el = Ext.get(el);

    if (el.dom.tagName.toLowerCase() != 'img') {
      el = el.child('img');
    }

    if (el) {
      img = el.getAttribute('title');
      module.imgSelectTrigger.setValue(img);
      module.imgSelectTrigger.fireEvent('change');
      selected = true;
      win.hide();
    }
  });

  module[ callback ] = function(event, el) {
    var img, items;

    if (el.tagName.toLowerCase() == 'img') {
      el = Ext.get(el).prev();
    }

    module.imgSelectTrigger = Ext.getCmp(el.id);

    win.show();
    img = module.imgSelectTrigger.getValue();
    items = win.getEl().query('.scada-icon-wrap');
    Ext.each(items, function(item) {
      item = Ext.get(item);

      if (item.getAttribute('data-id') == img) {
        item.addClass('scada-icon-active');
      }
      else {
        item.removeClass('scada-icon-active');
      }
    });
  };
};

Scada.bindImgSelect = function(module, trigger, mode) {
  var callback = mode == 'icon' ? 'showIconSelect' : 'showImgSelect';
  Ext.getCmp(trigger).onTriggerClick = module[ callback ];
};

if (document.title.match(/homelynk/i)) {
  Ext.ColorPalette.prototype.colors.push('009530', '4FA600', '87D300', '626469', '9FA0A4', 'FFD100', 'F09609', 'B10043', '42B4E6', 'FF0097', 'A200FF');
}

Ext.Resizable.Handle.prototype.onMouseDownOrig = Ext.Resizable.Handle.prototype.onMouseDown;
Ext.Resizable.Handle.prototype.onMouseDown = function(e) {
  if (!this.rz.window) {
    this.rz.preserveRatio = this.position == 'southeast';
  }
  this.onMouseDownOrig(e);
};

Ext.form.ComboBox.prototype.doQuery = function(q, forceAll) {
  q = Ext.isEmpty(q) ? '' : q;
  var qe = {
    query: q,
    forceAll: forceAll,
    combo: this,
    cancel: false
  };

  if (this.fireEvent('beforequery', qe) === false || qe.cancel) {
    return false;
  }

  q = qe.query;
  forceAll = qe.forceAll;

  if (forceAll === true || (q.length >= this.minChars)) {
    if (this.lastQuery !== q) {
      this.lastQuery = q;

      if (this.mode == 'local') {
        this.selectedIndex = -1;

        if (forceAll) {
          this.store.clearFilter();
        }
        else {
          if (this.regexp) {
            q = new RegExp(Ext.escapeRe(q));
          }

          this.store.filter(this.displayField, q);
        }

        this.onLoad();
      }
      else {
        this.store.baseParams[this.queryParam] = q;
        this.store.load({
          params: this.getParams(q)
        });
        this.expand();
      }
    }
    else{
      this.selectedIndex = -1;
      this.onLoad();
    }
  }
};

Ext.data.Store.prototype.getAll = function() {
  return this.snapshot ? this.snapshot.items : this.data.items;
};

Ext.data.Store.prototype.getExact = function(field, value) {
  var items, i, j, data;

  items = this.getAll();
  for (i = 0, j = items.length; i < j; i++) {
    data = items[ i ].data;

    if (data[ field ] == value) {
      return data;
    }
  }
};

Ext.form.FormPanel.prototype.onAddEvent = function() {};

String.prototype.toList = function() {
  var ret = [], val, i, j, arr = this.toLowerCase().split(',');

  for (i = 0, j = arr.length; i < j; i++) {
    val = arr[ i ].trim();

    if (val && ret.indexOf(val) == -1) {
      ret.push(val);
    }
  }

  return ret;
};

Ext.form.Field.prototype.getOriginalValue = function() {
  return this.initialConfig.value;
};

Ext.form.Checkbox.prototype.getOriginalValue = function() {
  return this.initialConfig.checked;
};

Ext.form.Field.prototype.initComponentOrig = Ext.form.Field.prototype.initComponent
Ext.form.Field.prototype.initComponent = function() {
  this.originalValue = this.getOriginalValue();
  this.initComponentOrig();
};
