[
  {
    text: 'Conditionals',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'If - Then',
        leaf: true,
        cls: 'file',
        script: "if (condition) then\n  dosomething()\nend"
      },
      {
        text: 'If (AND) - Then',
        leaf: true,
        cls: 'file',
        script: "if (condition1 and condition2) then\n  dosomething()\nend"
      },
      {
        text: 'If (OR) - Then',
        leaf: true,
        cls: 'file',
        script: "if (condition or condition2) then\n  dosomething()\nend"
      },
      {
        text: 'If - Then - Else',
        leaf: true,
        cls: 'file',
        script: "if (condition) then\n  dosomething()\nelse\n  dosomethingelse()\nend"
      },
      {
        text: 'If - Else If',
        leaf: true,
        cls: 'file',
        script: "if (condition) then\n  dosomething()\nelseif (othercondition) then\n  dosomethingelse()\nend"
      }
    ]
  },
  {
    text: 'Loops and iterators',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Hashtable iteration',
        leaf: true,
        cls: 'file',
        script: "-- hashtable = { a = 1, b = 2, c = 3 }\nfor key, value in pairs(hashtable) do\n  dosomething()\nend"
      },
      {
        text: 'Array iteration',
        leaf: true,
        cls: 'file',
        script: "-- arraytable = { 'a', 'b', 'c' }\nfor index, value in ipairs(arraytable) do\n  dosomething()\nend"
      },
      {
        text: 'Numeric for loop',
        leaf: true,
        cls: 'file',
        script: "-- for i = START, STOP, STEP\nfor i = 1, 10, 2 do\n  dosomething()\nend"
      },
      {
        text: 'While loop',
        leaf: true,
        cls: 'file',
        script: "-- loop while condition is met\nwhile (condition) do\n  dosomething()\nend"
      },
      {
        text: 'Repeat..until loop',
        leaf: true,
        cls: 'file',
        script: "-- loop until condition is met\nrepeat\n  dosomething()\nuntil (condition)"
      }
    ]
  },
  {
    text: 'Internal I/O',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Read digital output 1 status',
        leaf: true,
        cls: 'file',
        script: "-- read digital output 1 status\nstatus = digital.read(1)"
      },
      {
        text: 'Write to digital output',
        leaf: true,
        cls: 'file',
        script: "-- turn output 1 on\ndigital.write(1, true)"
      },
      {
        text: 'Read raw analog value (0..1023)',
        leaf: true,
        cls: 'file',
        script: "-- read from analog input channel 2\nvalue = analog.read(2)"
      },
      {
        text: 'Read voltage value (0..10V)',
        leaf: true,
        cls: 'file',
        script: "-- read from analog input channel 2\nvalue = analog.voltread(2)"
      },
      {
        text: 'Read binary value',
        leaf: true,
        cls: 'file',
        script: "-- read from analog input channel 2\n-- anything below 5V is false, otherwise true\nvalue = analog.binread(2)"
      }
    ]
  },
  {
    text: 'Math',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Absolute value',
        leaf: true,
        cls: 'file',
        script: "-- e.g. x = -12, y = 12\ny = math.abs(x)"
      },
      {
        text: 'Ceiling',
        leaf: true,
        cls: 'file',
        script: "-- e.g. x = 12.2, y = 13\ny = math.ceil(x)"
      },
      {
        text: 'Floor',
        leaf: true,
        cls: 'file',
        script: "-- e.g. x = 12.7, y = 12\ny = math.floor(x)"
      },
      {
        text: 'Round to integer',
        leaf: true,
        cls: 'file',
        script: "-- e.g. x = 12.7, y = 13; x = 12.3, y = 12\ny = math.floor(x + 0.5)"
      },
      {
        text: 'Random value',
        leaf: true,
        cls: 'file',
        script: "-- get random integer value between min and max (inclusive)\ny = math.random(min, max)"
      }
    ]
  },
  {
    text: 'Objects / KNX bus',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Get event value',
        leaf: true,
        cls: 'file',
        script: "-- object mapped to this event must have its data type set\nvalue = event.getvalue()"
      },
      {
        text: 'Get object value (by name)',
        leaf: true,
        cls: 'file',
        script: "-- get value of object named 'my object'\nvalue = grp.getvalue('|>my object<|')"
      },
      {
        text: 'Get object value (by address)',
        leaf: true,
        cls: 'file',
        script: "-- get value of object with group address 1/1/1\nvalue = grp.getvalue('|>1/1/1<|')"
      },
      {
        text: 'Group read (by name)',
        leaf: true,
        cls: 'file',
        script: "-- send read request to object named 'my object'\ngrp.read('|>my object<|')"
      },
      {
        text: 'Group read (by address)',
        leaf: true,
        cls: 'file',
        script: "-- send read request to 1/1/1\ngrp.read('|>1/1/1<|')"
      },
      {
        text: 'Group write (by name, constant)',
        leaf: true,
        cls: 'file',
        script: "-- write value (e.g. boolean 'true') to object named 'my object', datatype must be set for this object\ngrp.write('|>my object<|', true)"
      },
      {
        text: 'Group write (by address, constant)',
        leaf: true,
        cls: 'file',
        script: "-- write value (e.g. boolean 'true') to object with group address 1/1/1, datatype must be set for this object\ngrp.write('|>1/1/1<|', true)"
      },
      {
        text: 'Group write (by name, variable)',
        leaf: true,
        cls: 'file',
        script: "-- write value (e.g. number '12.1') to object named 'my object', datatype must be set for this object\nvalue = 12.1\ngrp.write('|>my object<|', value)"
      },
      {
        text: 'Group write (by address, variable)',
        leaf: true,
        cls: 'file',
        script: "-- write value (e.g. number '12.1') to object with group address 1/1/1, datatype must be set for this object\nvalue = 12.1\ngrp.write('|>1/1/1<|', value)"
      },
      {
        text: 'Update internal value (by name)',
        leaf: true,
        cls: 'file',
        script: "-- no bus telegram is sent, only database value is changed\ngrp.update('|>my object<|', 123)"
      },
      {
        text: 'Update internal value (by address)',
        leaf: true,
        cls: 'file',
        script: "-- no bus telegram is sent, only database value is changed\ngrp.update('|>1/1/1<|', 123)"
      },
      {
        text: 'Convert name to group address',
        leaf: true,
        cls: 'file',
        script: "address = grp.alias('|>my object<|')"
      },
      {
        text: 'Convert group address to name',
        leaf: true,
        cls: 'file',
        script: "name = grp.alias('|>1/1/1<|')"
      },
      {
        text: 'Single object functions',
        cls: 'folder',
        expanded: false,
        children: [
          {
            text: 'Find object (by address)',
            leaf: true,
            cls: 'file',
            script: "-- get object with address 1/1/1\nmyobject = grp.find('|>1/1/1<|')"
          },
          {
            text: 'Find object (by name)',
            leaf: true,
            cls: 'file',
            script: "-- get object with name 'my object'\nmyobject = grp.find('|>my object<|')"
          },
          {
            text: 'Write value to bus',
            leaf: true,
            cls: 'file',
            script: "-- write value (e.g. 127) to the object\nmyobject:write(127)"
          },
          {
            text: 'Send read request to bus',
            leaf: true,
            cls: 'file',
            script: "-- write value (e.g. 127) to the object\nmyobject:read()"
          }
        ]
      },
      {
        text: 'Multiple object functions (tags)',
        cls: 'folder',
        expanded: false,
        children: [
          {
            text: 'Find by single tag',
            leaf: true,
            cls: 'file',
            script: "-- get an array of objects tagged 'tag name'\nmyobjects = grp.tag('|>tag name<|')"
          },
          {
            text: 'Find by multiple tags (match all)',
            leaf: true,
            cls: 'file',
            script: "-- get an array of objects tagged 'tag name 1' AND 'tag name 2'\nmyobjects = grp.tag({'tag name 1', 'tag name 2'}, 'all')"
          },
          {
            text: 'Find by multiple tags (match any)',
            leaf: true,
            cls: 'file',
            script: "-- get an array of objects tagged 'tag name 1' OR 'tag name 2'\nmyobjects = grp.tag({'tag name 1', 'tag name 2'}, 'any')"
          },
          {
            text: 'Write value to bus',
            leaf: true,
            cls: 'file',
            script: "-- write value (e.g. 127) to the object\nmyobjects:write(127)"
          },
          {
            text: 'Send read request to bus',
            leaf: true,
            cls: 'file',
            script: "-- write value (e.g. 127) to the object\nmyobjects:read()"
          }
        ]
      }
    ]
  },
  {
    text: 'Storage',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Get data from storage',
        leaf: true,
        cls: 'file',
        script: "-- mydata will be set nil when data was not found\nmydata = storage.get('|>myobjectdata<|')"
      },
      {
        text: 'Get data (with default value)',
        leaf: true,
        cls: 'file',
        script: "-- mydata will be set to a predefined value (e.g. 127) when data was not found\nmydata = storage.get('|>myobjectdata<|', 127)"
      },
      {
        text: 'Save data to storage',
        leaf: true,
        cls: 'file',
        script: "-- set storage variable myobjectdata to a specified value (e.g. 127)\nstorage.set('|>myobjectdata<|', 127)"
      }
    ]
  },
  {
    text: 'Script control',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Enable script',
        leaf: true,
        cls: 'file',
        script: "script.enable('|>scriptname<|')"
      },
      {
        text: 'Disable script',
        leaf: true,
        cls: 'file',
        script: "script.disable('|>scriptname<|')"
      },
      {
        text: 'Get script status',
        leaf: true,
        cls: 'file',
        script: "-- return true/false if script is found, nil otherwise\nstatus = script.status('|>scriptname<|')"
      }
    ]
  },
  {
    text: 'Alerts and logs',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Alert',
        leaf: true,
        cls: 'file',
        script: "alert('|>My alert text<|')"
      },
      {
        text: 'Formatted alert',
        leaf: true,
        cls: 'file',
        script: "-- see string.format for more info\nalert('Group address %s, numeric value: %d', '1/1/1', 127)"
      },
      {
        text: 'Log variables',
        leaf: true,
        cls: 'file',
        script: "a = { f1 = 10, f2 = 20 }\nb = 127\nc = 'test'\n-- all passed arguments will appear in Logs module\nlog(a, b, c)"
      }
    ]
  },
  {
    text: 'Time functions',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Delay script execution',
        leaf: true,
        cls: 'file',
        script: "-- wait for 1.5 seconds\nos.sleep(|>1.5<|)"
      }
    ]
  },
  {
    text: 'Miscellaneous',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Send an e-mail',
        leaf: true,
        cls: 'file',
        script: "-- make sure mail settings are set in user function library before using this function\nsubject = 'LM e-mail test'\nmessage = 'Testing e-mail from LogicMachine'\nmail('user@example.com', subject, message)"
      },
      {
        text: 'Sunrise / sunset',
        leaf: true,
        cls: 'file',
        script: "-- returns number of minutes from 0:00 until sunrise and sunset\nlatitude = 57\nlongitude = 24.15\nsunrise, sunset = rscalc(latitude, longitude)"
      },
      {
        text: 'Convert sunrise / sunset to readable time',
        leaf: true,
        cls: 'file',
        script: "sunrise = string.format('%d:%.2d', math.floor(sunrise / 60), sunrise % 60)\nsunset = string.format('%d:%.2d', math.floor(sunset / 60), sunset % 60)"
      }
    ]
  },
  {
    text: 'Serial',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Open connection',
        leaf: true,
        cls: 'file',
        script: "require('serial')\nport = serial.open('/dev/ttyS2', {\n  baudrate = 38400,\n  databits = 8,\n  stopbits = 1,\n  parity = 'even',\n  duplex = 'half'\n})"
      },
      {
        text: 'Write to port',
        leaf: true,
        cls: 'file',
        script: "port:write('test data')"
      },
      {
        text: 'Blocking read',
        leaf: true,
        cls: 'file',
        script: "-- script will block until 10 characters are read\ndata = port:read(10)"
      },
      {
        text: 'Timeout read',
        leaf: true,
        cls: 'file',
        script: "-- script will wait for 10 characters for 20 seconds\ndata = port:read(10, 20)"
      },
      {
        text: 'Close serial port',
        leaf: true,
        cls: 'file',
        script: "port:close()"
      }
    ]
  },
  {
    text: 'Modbus',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Create Modbus TCP object',
        leaf: true,
        cls: 'file',
        script: "require('luamodbus')\nmb = luamodbus.tcp()"
      },
      {
        text: 'Create Modbus RTU object',
        leaf: true,
        cls: 'file',
        script: "require('luamodbus')\nmb = luamodbus.rtu()"
      },
      {
        text: 'Open Modbus TCP connection',
        leaf: true,
        cls: 'file',
        script: "-- IP: 192.168.1.2, port: 1234\nmb:open('192.168.1.2', 1234)\nmb:connect()"
      },
      {
        text: 'Open Modbus RTU connection',
        leaf: true,
        cls: 'file',
        script: "-- 38400 baud rate, even parity, 8 data bits, 1 stop bit, half duplex\nmb:open('/dev/ttyS2', 38400, 'E', 8, 1, 'H')\nmb:connect()"
      },
      {
        text: 'Set slave address',
        leaf: true,
        cls: 'file',
        script: "mb:setslave(123)"
      },
      {
        text: 'Read single coil',
        leaf: true,
        cls: 'file',
        script: "-- read 1 coil from address 1000\ncoil = mb:readcoils(1000)"
      },
      {
        text: 'Read multiple coils',
        leaf: true,
        cls: 'file',
        script: "-- read 3 coils from address 1000\ncoil1, coil2, coil3 = mb:readcoils(1000, 3)"
      },
      {
        text: 'Read discrete input',
        leaf: true,
        cls: 'file',
        script: "-- read from address 1000\nvalue = mb:readdiscreteinputs(1000)"
      },
      {
        text: 'Read registers',
        leaf: true,
        cls: 'file',
        script: "-- read from address 1000\nvalue = mb:readregisters(1000)"
      },
      {
        text: 'Read input registers',
        leaf: true,
        cls: 'file',
        script: "-- read from address 1000\nvalue = mb:readinputregisters(1000)"
      },
      {
        text: 'Write single bit',
        leaf: true,
        cls: 'file',
        script: "-- set bit at address 1000 to 'on'\nmb:writebits(1000, true)"
      },
      {
        text: 'Write multiple bits',
        leaf: true,
        cls: 'file',
        script: "-- set bit at address 1000 to 'on', bit at address 1001 to 'off'\nmb:writebits(1000, true, false)"
      },
      {
        text: 'Write single register',
        leaf: true,
        cls: 'file',
        script: "-- set register at address 1000 to 123\nmb:writeregisters(1000, 123)"
      },
      {
        text: 'Write multiple registers',
        leaf: true,
        cls: 'file',
        script: "-- set register at address 1000 to 123, register at address 1001 to 321\nmb:writeregisters(1000, 123, 321)"
      },
      {
        text: 'Close modbus connection',
        leaf: true,
        cls: 'file',
        script: "mb:close()"
      }
    ]
  },
  {
    text: 'DMX',
    cls: 'folder',
    expanded: false,
    children: [
      {
        text: 'Open connection',
        leaf: true,
        cls: 'file',
        script: "-- opens new DMX connection\nrequire('luadmx')\ndm, err = luadmx.open('/dev/ttyS2')\n\nif not dm then\n  alert('DMX error')\n  return\nend"
      },
      {
        text: 'Set number of channels to use',
        leaf: true,
        cls: 'file',
        script: "-- min 1, max 512\ndm:setcount(10)"
      },
      {
        text: 'Get number of channels to use',
        leaf: true,
        cls: 'file',
        script: "-- min 1, max 512\ncount = dm:getcount()"
      },
      {
        text: 'Set all channels to a given value',
        leaf: true,
        cls: 'file',
        script: "-- all 512 channel values will be set to 127\ndm:setall(127)"
      },
      {
        text: 'Set specific channel value',
        leaf: true,
        cls: 'file',
        script: "-- channel 5 value set to 127\ndm:setchannel(5, 127)"
      },
      {
        text: 'Get specific channel value',
        leaf: true,
        cls: 'file',
        script: "-- get channel 5\nvalue = dm:getchannel(5)"
      },
      {
        text: 'Send data to bus',
        leaf: true,
        cls: 'file',
        script: "-- data must be sent at least one per second\nvalue = dm:send()"
      }
    ]
  },
  {
    text: 'KNXlogic 1.01',
    cls: 'folder',
    expanded: false,
    children: [
	  {
		text: 'sms functions',
		cls: 'folder',
		expanded: false,
		children: [
		  {
			text: 'sms daemon (required)',
			leaf: true,
			cls: 'file',
			script: "--for all sms functionality it is required to run the deamon in a resident script with delay 1\nthe deamon assures that sms are send and received/processed by the default rules\n\nrequire 'sms\n\nsms.run('/dev/ttyUSB2') --use your actual modem port; add the pin as second argument if your SIMcard is locked"
		  },
		  {
			text: 'find the sms modem port',
			leaf: true,
			cls: 'file',
			script: "--Logs all found modems so you can find the port of the modem you want to use for sms sending\n--Add it temporarily to the deamon script\n--In the modem info you will also see if the simcard is pin-locked or not\n--Give the call 'USB' or 'serial' as argument if you only want to search USB/serial modems. No argument = serial and USB\nlog(sms.findModems('USB'))"
		  },
		  {
			text: 'send a simple sms',
			leaf: true,
			cls: 'file',
			script: "--assumes the daemon is running\nrequire 'sms'\nsms.send('+32xxxxxxxx','your text')"
		  },
		  {
			text: 'send a flash sms to multiple receivers',
			leaf: true,
			cls: 'file',
			script: "--assumes the daemon is running\nrequire 'sms'\nsms.send({'+32xxxxxxxx','+33xxxxxx'},'your text',true)"
		  },
		  {
			text: 'exclude a sender from the sms communication',
			leaf: true,
			cls: 'file',
			script: "--to insert in the deamon script\n--this could be the service number of your provider, to assure that the service messages from the operator are not processed, only reported\nsms.blackList = {'+32477306003'} --you can add as many as you want to this table, adding '+32' will exclude all numbers from Belgium"
		  },
		  {
			text: 'include a sender in a blacklisted range',
			leaf: true,
			cls: 'file',
			script: "--to insert in the deamon script\n--whiteList takes priority over blackList\nsms.whiteList = {'+32yourWhiteListedNumber'} --you can add as many as you want to this table, adding '+32477233' will allow all numbers starting with that string"
		  },
		  {
			text: 'define what action to take on an operator sms',
			leaf: true,
			cls: 'file',
			script: "--to insert in the deamon script\nsms.operatorList = {'+32yourOperatorNumber'} --you can add as many as you want to this table\nsms.operatorAction(msg)\n    --your code here to define what to do with the operator 'msg'\nend"
		  },
		  {
			text: 'alias for GA commands',
			leaf: true,
			cls: 'file',
			script: "--to insert in the deamon script\n--if an alias is only defined in 'set', it will default also for 'get' \nsms.shortcuts = {\n\t['set']={\n\t\tverg = '0/3/9',\n\t\tkkn = {'0/1/7','0/1/9'},\n\t},\n\t['get']={\n\t\tkkn = {'0/0/7','0/0/9'},\n\t},\n}"
		  },
		]
	  },
	  {
		text: 'mail functions',
		cls: 'folder',
		expanded: false,
		children: [
		  {
			text: 'mail daemon (required)',
			leaf: true,
			cls: 'file',
			script: "--for all mail functionality it is required to run the daemon in a resident script with delay of 30-60s\n--the deamon assures that sms are send and received/processed by the default rules:\n--  - the part of the body text between <LM><\LM>-tags is treated as lua code\n--  - if no tags are present, the full body is treated aslua code\n--  - additionally, lua- and tx-files in attachement are treated as lua code\n\nmail = require 'mailplus'\n\nmail.processMails()"
		  },
		  {
			text: 'mail server config',
			leaf: true,
			cls: 'file',
			script: "--add to the startup script, (or less optimal, to the daemon, prior to the daemon start command).\nmail = require 'mailplus'\nlocal mailcfg={}\n--format of mailcfg with default values:\nmailcfg.smtp={\n\thost  = '', --your smtp host. If using FQN, then DNS must have been set\n\tssl       = '' ,  -- can be '', 'tslv1', 'sslv23', 'sslv3' -> gmail: 'sslv23'\n\tport      = 25,   -- gmail-SSL: 465\n\t--user      = '', --do not set user/password if your smtp does not expect it\n\t--password  = '',\n\tfrom      = ''    -- format VERY important, example: '\"KNXlogic.be LM2\" <knxlogic@mydomain.eu>',\n}\nmailcfg.pop = {\n\thost   = '', --your pop server\n\tport      = 110,  -- gmail SSL: 995\n\tuser      = '',\n\tpassword  = '',\n\tssl       = '' ,  -- can be '', 'tslv1', 'sslv23', 'sslv3' -> gmail: 'sslv23'\n\tnoop      = false ,  --when debug shows losing connection at initialisation, try set to true\n}\nmail.setMailConfig(mailcfg)\n"
		  },
		  {
			text: 'send an email',
			leaf: true,
			cls: 'file',
			script: "--assumes the daemon is running\nmail = require('mailplus')\nmail.send('to', 'cc', 'bcc', 'yourSubject','your text')  --to, cc and bcc can be nil(=not applicable), a address string or a table of addresses"
		  },
		  {
			text: 'reply to an email (in email)',
			leaf: true,
			cls: 'file',
			script: "--this only works as a command in an email: will reply the wanted string:\nmail.reply('yourtext')"
		  },
		  {
			text: 'exclude mailaddresses from the mail communication',
			leaf: true,
			cls: 'file',
			script: "--to insert in the daemon script\n--you can add as many as you want to this table, adding a substring like '@' will exclude all addresses with this part in the address (all in this case), except those whiteListed\nmail.setBlackList = {'a@b.ce','.be'} "
		  },
		  {
			text: 'allow mailaddresses to the mail communication',
			leaf: true,
			cls: 'file',
			script: "--to insert in the deamon script\n--you can add as many as you want to this table, adding a substring will exclude all addresses with this part in the address\nmail setWhiteList = {'a@b.be','mydom.be'} "
		  },
		]
	  },
	  {
		text: 'Special libraries (license required)',
		cls: 'folder',
		expanded: false,
		children: [
		  {
			text: 'Zehnder-StorkAir WHR',
			cls: 'folder',
			expanded: false,
			children: [
			  {
				text: 'WHR daemon (required) - local serial port',
				leaf: true,
				cls: 'file',
				script: "--for all WHR functionality it is required to run the deamon in a resident script for delay 0\n\nrequire 'whr'\n\n--configure the group addresses\nwhr.GA.command={\n\tvolume = '6/0/0',\n\tTcomfort = '6/0/1', \n}\nwhr.GA.status={\n\tvolume   = '6/1/0',\n\tTcomfort = '6/1/1',}\n\nwhr.run('/dev/ttyS5') --4 available serial ports (232 and 485) on your device are ttyS2, ttyS3, ttyS4, ttyS5, from left to right  "
			  },
			  {
				text: 'Available WHR functions',
				leaf: true,
				cls: 'file',
				script: "--If you want to know all implemented status and commands use the following prior to your own config and look in the log\nrequire'whr'\nlog(whr.GA)"
			  },
			  {
				text: 'Adv.config: custom status update interval',
				leaf: true,
				cls: 'file',
				script: "\t--override the default update interval use this in your config (interval in seconds): \n\tvolume = {ga='6/0/0', int=30},"
			  },
			  {
				text: 'Adv.config: multiple group addresses',
				leaf: true,
				cls: 'file',
				script: "\t--write a status to several group addresses or read a command from multiple GA's: \n\tvolume = {'6/0/0', '1/0/0', '7/8/9'},"
			  },
			  {
				text: 'Adv.config: status with custom update interval and multiple group addresses',
				leaf: true,
				cls: 'file',
				script: "\t--write status to multiple GA at custom interval\n\tvolume = {ga={'6/0/0', '1/0/0', '7/8/9'}, int=30},"
			  },
			]
		  },
		  {
			text: 'Siemens SPC - X10 receiver',
			cls: 'folder',
			expanded: false,
			children: [
			  {
				text: 'SPC X10 daemon (required) - local serial port',
				leaf: true,
				cls: 'file',
				script: "--for all X10 receiver functionality it is required to run the deamon in a resident script for delay 0\n\nrequire 'spcx10'\n\n--configure basic groupaddress allocation/linking\nspcx10.mapping = {\n\tA1 = '3/0/0',\n\tA2 = '3/0/1',\n}\n\nspcx10.run('/dev/ttyS5') --4 available serial ports (232 and 485) on your device are ttyS2, ttyS3, ttyS4, ttyS5, from left to right  "
			  },
			  {
				text: 'Adv. config: link with simple activation switch',
				leaf: true,
				cls: 'file',
				script: "\t--need this GA link only at certain moments (eg. only at night):\n\tA3 = {ga='3/0/2', act = grp.getvalue('night')}"
			  },
			  {
				text: 'Adv. config: link with function activation switch',
				leaf: true,
				cls: 'file',
				script: "--a function as activation switch (when too complex to call every second):\nif running then\n\tisA3act = function()\n\t\t--your code\n\t\treturn yourvalue\n\tend\n\n\tnspcx10.mapping = {\n\t\tA1 = '3/0/0',\n\t\tA2 = '3/0/1',\n\t\tA3 = {ga='3/0/2', act = isA3act},\n\t} \n\trunning=true\nend"
			  },
			]
		  },
		]
	  },
    ]
  },
]
